//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL  license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'http://www.cecill.info'.

function [pos_car,jacob]=CL_co_ell2car(pos_ell,er,obl)
// Elliptical to cartesien coordinates
//
// Calling Sequence
// [pos_car[,jacob]] = CL_co_ell2car(pos_ell[,er[,obl]])
//
// Description
// <itemizedlist><listitem>
// Given a set of elliptical (or geodetic if planet is earth) coordinates <emphasis role="bold">(pos_ell)</emphasis>, they are transformed into cartesien coordinates <emphasis role="bold">(pos_car)</emphasis>using
// the given ellipsoid semi-major axis and oblateness.
// <para>If no optionnal arguments are passed, defaults are the earth ellipsoid semi-major axis(equatorial radius) and oblateness.</para>
// <para>Transformation jacobian is optionnally computed.</para></listitem>
// <listitem>Elliptical (geodetic) latitude is determined by the angle between the normal of the ellipsoid and the plane of the equator,
// whereas spherical (geocentric) latitude is determined around the centre :
// <para/><inlinemediaobject><imageobject><imagedata fileref="elliptical_coord.gif"/></imageobject></inlinemediaobject>
// <para> Warning : pos_ell contains longitude, elliptical latitude, and <emphasis role="bold">elliptical altitude</emphasis>
// (whereas in spherical coordinates it is longitude, spherical latitude and distance to center</para></listitem>
// <listitem>Last update : 27/6/2008</listitem>
// </itemizedlist>
//
// Parameters
// pos_ell : [lon;lat;alt] position in elliptical (or geodetic) coordinates vector [rad],[m](3xN)
// er : (optional) reference ellipsoid semi-major axis (default is earth equatorial radius %CL_eqRad) [m]
// obl : (optional) reference ellipsoid oblateness (default is earth oblateness %CL_obla)
// pos_car : [x;y;z] position cartesien coordinates vector (in terrestrial frame) [m] (3xN)
// jacob : (optional) transformation jacobian (3x3xN)
//
// Authors
// CNES - DCT/SB
//
// Bibliography
// 1 Mecanique Spatiale, Cnes - Cepadues Editions, Tome I, section 3.2.3 (Les reperes de l'espace et du temps, Relations entre les coordonnees)
// 2 CNES - MSLIB FORTRAN 90, Volume T (mt_geod_car)
//
// See also
// CL_co_car2ell
// CL_co_sph2car
// CL_co_car2sph
//
// Examples
// // Example 1 :
// pos_ell = [ CL_deg2rad(12.125) ; CL_deg2rad(108.2) ; 110.0];
// [pos_car,jacob1] = CL_co_ell2car(pos_ell);

// Declarations:
global %CL_eqRad %CL_obla;

// Code:

[lhs,rhs]=argn();
if ~exists('er','local') then er=%CL_eqRad; end
if ~exists('obl','local')
  obl=%CL_obla;
else
  if obl>=1 then error('oblatenes bigger or equal to 1'); end
end
select lhs
  case 1
    compute_jacob = %f
  case 2
    compute_jacob = %t
  else
    error('check number of output arguments')
end

//get latitude, longitude and altitude
lat = pos_ell(2,:);
lon = pos_ell(1,:);
hg = pos_ell(3,:);

//e and N calculation (see ref. 1)
ex2 = obl*(2-obl)
correc = 1 - ex2.*(sin(lat)^2)  //correc>0, might be 0 if obl=1
Rphi = er./sqrt(correc) //radius at given latitude
RphiplusH = Rphi + hg

//cartesien coordinates:
x = RphiplusH.*cos(lat).*cos(lon);
y = RphiplusH.*cos(lat).*sin(lon);
z = (Rphi.*(1-ex2)+hg).*sin(lat);

pos_car = [x;y;z];

//JACOBIAN COMPUTATION
if compute_jacob
  jacob = zeros(3,3,size(pos_ell,2))

  K = (1 - ex2)./correc //K in [0,1]
  //if obl = 0 : ex2 = 0 ; correc = 1 ; Rphi = er; K = 1

  KRphiplusH = K.*Rphi + hg

  jacob(1,2,:) = - KRphiplusH .* sin(lat).*cos(lon)
  jacob(1,1,:) = - RphiplusH .* cos(lat).*sin(lon)
  jacob(1,3,:) = cos(lat) .* cos(lon)

  jacob(2,2,:) = - KRphiplusH .* sin(lat) .* sin(lon)
  jacob(2,1,:) = RphiplusH .* cos(lat) .* cos(lon)
  jacob(2,3,:) = cos(lat) .* sin(lon)

  jacob(3,2,:) = KRphiplusH .* cos(lat)
  jacob(3,1,:) = 0
  jacob(3,3,:) = sin(lat)

  if size(pos_ell,2)==1 then jacob=jacob(:,:,1); end

end

endfunction

