//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL  license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'http://www.cecill.info'.

function [cjd] = CL_dat_cal2cjd(year,month,day,hour,minute,second)
// Calendar date to CNES Julian Day
//
// Calling Sequence
// [cjd] = CL_dat_cal2cjd(year[,month[,day[,hour[,minute[,second]]]]])
//
// Description
// <itemizedlist><listitem>
// This function converts a calendar date (year,month,day,hour,minute,second) to a Cnes Julian Date(number of fractionnal days since January 1st 1950).
// <para>The fractional part of cjd (cjd-int(cjd)) gives the time of the day as a decimal fraction of one day with 0.5 representing midday UT.</para>
// <para>ATTENTION: this algorithm is only valid for dates before 28 February 2100, because of the calculation
// over the leap years (with modulo). With the taken margin, it accepts only dates before 01-01-2100 at 0 hours.</para>
// </listitem>
// <listitem>Last update : 27/6/2008</listitem>
// </itemizedlist>
//
// Parameters
// year: integer in [1950,2099] (1xN)
// month: (optional) integer in [1,12] (1xN) (default is 1)
// day: (optional) integer between 1 and 28, 29, 30 or 31 depending on the month of the year (1xN) (default is 1)
// hour: (optional) integer in [0,23] (1xN) (default is 0)
// minute: (optional) integer in [0,59] (1xN) (default is 0)
// second: (optional) real in [0,60) (1xN) (default is 0)
// cjd: Cnes Julian Date (number of fractionnal days since January 1st 1950) (1xN)
//
// Authors
// CNES - DCT/SB
//
// See also
// CL_dat_cal2jd
// CL_dat_cjd2cal
//
// Bibliography
// 1 CNES - MSLIB FORTRAN 90, Volume D (md_calend_julien)
//
// Examples
// // Example 1
// cjd = CL_dat_cal2cjd(1968,2,29,0,0,0)  // february 29 1968
//
// // Example 2
// day = [29 5 28];
// month = [2 4 2];
// year = [1968 1980 1986];
// hour = [0 12 4];
// minute = [0 30 20];
// second = [0 5 12];
// cjd = CL_dat_cal2cjd(year,month,day,hour,minute,second);
//
// [year2,month2,day2,hour2,minute2,second2] = CL_dat_cjd2cal(cjd);


// Declarations:


// Code:

if ~exists('month','local') then month=1; end
if ~exists('day','local') then day=1; end
if ~exists('hour','local') then hour=0; end
if ~exists('minute','local') then minute=0; end
if ~exists('second','local') then second=0; end

admonth = [-1,30,58,89,119,150,180,211,242,272,303,333]; //acumulated days by month
nbday = [31,28,31,30,31,30,31,31,30,31,30,31];  //number of days by month

//check input date validity
i_year = find( (year < 1950)|(year > 2099) )
if i_year~=[]
   warning('bad year: returned negative cjd');
end

i_month = find ( (month < 1)|(month > 12) )
if i_month~=[]
   error('bad month');
end

i_day = find(day < 1)
if i_day~=[]
  error('bad day');
end

leapyear = (modulo(year,4) == 0);  //calcul valid for years < ou = 2099
i_ly = find(leapyear & modulo(year,100)==0);
leapyear(i_ly) = leapyear(i_ly) & modulo(year(i_ly),400)==0;

i_day1 = find( (month <> 2)&(day > nbday(month)) ) //day > number of days in the month and not february
i_day2 = find( (month == 2)&leapyear ) //february and leap year
i_day3 = find( (month == 2)&(~leapyear) ) //february and not leap year

if i_day1~=[] then error('bad number of days in the month'); end
nj2 = nbday(month)
nj2(i_day2) = 29
nj2(i_day3) = 28

i_day4 = find( day>nj2 )
if i_day4~=[] then error('bad number of days'); end

//check input time validity
i_hour = find( (hour < 0)|(hour >= 24) )
if i_hour~=[] then error('bad hour'); end

i_minute = find( ( minute<0)|(minute>=60) )
if i_minute~=[] then error('bad minute'); end

i_second = find( (second < 0)|(second >= 60) )
if i_second~=[] then error('bad second'); end

//calculation of Julian days
iy = year - 1950;

//leap years contribution (integer number of days) of number of leap years -> integer division by 4
ia = floor((iy+1)./4);
cjd = 365.0.*iy + ia + admonth(month) + day;

i_cjd = find( (leapyear)&(month>2) )
cjd(i_cjd) = cjd(i_cjd) + 1 //if current is leap year and month>2 add one day

//number of seconds in the day
cjdsec= hour.*3600 + minute.*60 + second;

//add fraction of the day
cjd = cjd + cjdsec/3600/24;

endfunction
