//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL  license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'http://www.cecill.info'.

function [jd] = CL_dat_cal2jd(year,month,day,hour,minute,second)
// Calendar date to Julian Date
//
// Calling Sequence
// [jd] = CL_dat_cal2jd(year[,month[,day[,hour[,minute[,second]]]]])
//
// Description
// <itemizedlist><listitem>
// This function converts a calendar date into a Julian Date (number of fractionnal days since noon UT on January 1st 4713 BC).
// <para>The fractional part of jd (jd-int(jd)) gives the time of the day since noon UT as a decimal fraction of one day with 0.5 representing midnight UT.</para>
// </listitem>
// <listitem>Last update : 27/6/2008</listitem>
// </itemizedlist>
//
// Parameters
// year: integer. Astronomical year numbering is used, thus 1 BC is 0, 2 BC is -1, 4713 BC is -4712 and 2007 is 2007. (1xN)
// month: (optional) integer in [1,12] (1xN) (default is 1)
// day: (optional) integer between 1 and 28, 29, 30 or 31 following the month of the year (1xN) (default is 1)
// hour: (optional) integer in [0,23] (1xN) (default is 0)
// minute: (optional) integer in [0,59] (1xN) (default is 0)
// second: (optional) real in [0,60) (1xN) (default is 0)
// jd: Julian Date (number of fractionnal days since noon UT on January 1st 4713 BC) (1xN)
//
// Authors
// CNES - DCT/SB
//
// Bibliography
// 1 Explanatory Supplement to the Astronomical Almanac, edited by P. Kenneth Seidelmann. University Science Books, 1992.
// 2 Orbital Mechanics for Engineering Students, H D Curtis, Chapter 5.4
//
// See also
// CL_dat_jd2cal
// CL_dat_cal2cjd
//
// Examples
// // Example 1
// [jd] = CL_dat_cal2jd(2000,1,1,12,0,0)
//
// // Example 2
// year=[2004,1996]
// month=[5,10]
// day=[12,26]
// hour=[14,14]
// minute=[45,20]
// second=[30,0]
// jd = CL_dat_cal2jd(year,month,day,hour,minute,second);
// [year2,month2,day2,hour2,minute2,second2] = CL_dat_jd2cal(jd);

// Declarations:


// Code:

if ~exists('month','local') then month=1; end
if ~exists('day','local') then day=1; end
if ~exists('hour','local') then hour=0; end
if ~exists('minute','local') then minute=0; end
if ~exists('second','local') then second=0; end

a = floor((14-month)./12);
y = year+4800-a;
m = month+12.*a-3;

jdn = day + floor((153.*m+2)./5) + 365.*y + floor(y./4) - floor(y./100) + floor(y./400) - 32045;
fp = (hour-12)./24 + minute./1440 + second./86400;
jd = jdn + fp;

endfunction

