//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL  license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'http://www.cecill.info'.

function [year,month,day,hour,minute,second]=CL_dat_cjd2cal(cjd)
// CNES Julian date to calendar date
//
// Calling Sequence
// [year,month,day,hour,minute,second]=CL_dat_cjd2cal(cjd)
//
// Description
// <itemizedlist><listitem>
// This function converts a CNES Julian Date (number of fractionnal days since January 1st 1950) to a calendar date.
// <para>The fractional part of cjd (cjd-int(cjd)) gives the time of the day as a decimal fraction of one day with 0.5 representing midday UT.</para>
// </listitem>
// <listitem>Last update : 27/6/2008</listitem>
// </itemizedlist>
//
// Parameters
// cjd: CNES Julian Date (number of fractionnal days since January 1st 1950) (1xN)
// year: year (1xN)
// month: month (1xN)
// day: day (1xN)
// hour: hour (1xN)
// minute: minutes (1xN)
// second: seconds (1xN)
//
// Authors
// CNES - DCT/SB
//
// Bibliography
// 1 CNES - MSLIB FORTRAN 90, Volume D (md_julien_calend)
//
// See also
// CL_dat_cal2cjd
// CL_dat_cjd2jd
//
// Examples
// cjd = [6633 729.1 2400.2315 2282.0264];
// [year,month,day,hour,minute,second] = CL_dat_cjd2cal(cjd);
// cjd2 = CL_dat_cal2cjd(year,month,day,hour,minute,second);

// Declarations:


// Code:

[lhs rhs]=argn();

if rhs~=1
  error('check number of input arguments');
end

normal = [31,59,90,120,151,181,212,243,273,304,334,365]';
bissex = [31,60,91,121,152,182,213,244,274,305,335,366]';

cumul = normal*ones(1,size(cjd,2));

year = [];
month = [];
day = [];

cjdsec=(cjd-int(cjd)).*24.*3600;
cjd=int(cjd);

//normalisation of CNES Julian day (cjdsec in [0,86400[)
//no need because the input is cjd+cjdsec/3600/24
//if cjdsec>86400 -> cjdsec/3600/24>1 and days are added to cjd

//COMPUTATION OF LEAP YEARS SINCE 01/01/1950
njul = int( cjd + 731 );
na = int( njul ./ 365 );
nb = int( (na-1)./4 );

//COMPUTATION OF CALENDAR DATE
//year
days = int( njul - 365 .* na - nb );

year = zeros(cjd);
//if nb_days>0
i_year = find(days>0);
  year(i_year) = int( 1948 + na(i_year) );
//else
i_year2 = find(~(days>0));
  year(i_year2) = int( 1948 + na(i_year2) - 1 );
//end

leapyear = modulo(year,4); //valid for years < or = 2099
i_lp = find(leapyear==0);
  cumul(:,i_lp) = bissex*ones(1,size(i_lp,2));

//if nb_jour<=0
  //if leapyear
  i_ly = find( (days<=0)&(leapyear==0) );
    days(i_ly) = 366 + days(i_ly);
  //else
  i_nly = find( (days<=0)&(leapyear~=0) );
    days(i_nly) = 365 + days(i_nly);
  //end
//end

//month and day
num = zeros(cjd);
month = zeros(cjd);

n3 = cumul(3,:);
n6 = cumul(6,:);
n9 = cumul(9,:);
n12 = cumul(12,:);

//if between 01/01 and 31/03
i1 =find( (days>=1)&(days<=n3) ) ;
  num(i1) = 0;
  month(i1) = 0;
//end
//if between 01/04 and 30/06
i2 =find( (days>=n3+1)&(days<=n6) ) ;
  num(i2) = n3(i2);
  month(i2) = 3;
//end
//if between 01/07 and 30/09
i3 =find( (days>=n6+1)&(days<=n9) ) ;
  num(i3) = n6(i3);
  month(i3) = 6;
//end
//if later than 30/09
i4 =find( (days>=n9+1)&(days<=n12) ) ;
  num(i4) = n9(i4);
  month(i4) = 9;
//end

nm = num;
month = month + 1;
num = diag(cumul(month,:))';
//if days>num
//kk=0
ii = find(days>num);
  while ii~=[]
    nm(ii) = num(ii);
    month(ii) = month(ii) + 1;
    if size(ii,2)>1
      //kk=kk+1
      //disp(kk)
      num(ii) = diag(cumul(month(ii),ii))';
    else
      num(ii) = cumul(month(ii),ii);
    end
    ii = find(days>num);
  end
//end

day = days - nm;
decimal_hour = cjdsec./3600;
hour = int(decimal_hour);
decimal_minute = (decimal_hour - hour).*60;
minute = int(decimal_minute);
second = (decimal_minute - minute).*60;

endfunction
