//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL  license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'http://www.cecill.info'.

function [year,month,day,hour,minute,second] = CL_dat_jd2cal(jd)
// Julian Date to calendar date
//
// Calling Sequence
// [year,month,day,hour,minute,second] = CL_dat_jd2cal(jd)
//
// Description
// <itemizedlist><listitem>
// This function converts a Julian Date (number of fractionnal days since noon UT on January 1st 4713 BC) into a calendar date (year,month,day,hour,minute,second).
// <para>The fractional part of jd (jd-int(jd)) gives the time of the day since noon UT as a decimal fraction of one day with 0.5 representing midnight UT.</para>
// </listitem>
// <listitem>Last update : 27/6/2008</listitem>
// </itemizedlist>
//
// Parameters
// jd: Julian Date (number of fractionnal days since noon UT on January 1st 4713 BC) (1xN)
// year: year [1900..2100] (1xN)
// month: month [1..12] (1xN)
// day: day [1..28,29,30 or 31] (1xN)
// hour: hour [0..23] (1xN)
// minute: minute [0..59] (1xN)
// second: second [0.0..59.999] (1xN)
//
// Authors
// CNES - DCT/SB
//
// Bibliography
// 1 Fundamentals of Astrodynamics and Applications, David A. Vallado, Chapter 3.6.6
//
// See also
// CL_dat_cal2jd
// CL_dat_cal2cjd
//
// Examples
// // Example 1
// [year,month,day,hour,minute,second] = CL_dat_jd2cal(2449877.3458762)
//
// // Example 2
// jd=[2449877.3458762,2451603.5];
// [year,month,day,hour,minute,second] = CL_dat_jd2cal(jd);
// jd2 = CL_dat_cal2jd(year,month,day,hour,minute,second);

//acumulated days of month vector/matrix

// Declarations:


// Code:

s = size(jd,2);
//n = [0,31,59,90,120,151,181,212,243,273,304,334,365]'; //0 is to compute dayOfYear-acumulatedDaysOfPrecedentMonth
n = [31,28,31,30,31,30,31,31,30,31,30,31]';
Lmonth = n*ones(1,s);

//find year and days of year
tu = (jd-2415019.5)./365.25;
year = 1900+floor(tu);
leapyears = floor((year-1900-1).*0.25);
days = (jd-2415019.5)-((year-1900).*365+leapyears);

//check for case of beginning of a year
ii = find(days<1);
year(ii)=year(ii)-1;
leapyears(ii) = floor((year(ii)-1900-1).*0.25);
days(ii) = (jd(ii)-2415019.5)-((year(ii)-1900).*365+leapyears(ii));

dayofyear = floor(days); //get integer part of day

//find month and day of month
ii = (modulo(year-1900,4) == 0);
Lmonth(2,ii) = Lmonth(2,ii)+1;

month = zeros(1,s);
day = zeros(1,s);
for k=1:s
  i = 1;
  inttemp = 0;
  while ( (dayofyear(k) > inttemp+Lmonth(i,k)) & (i<12) )
    inttemp = inttemp + Lmonth(i,k);
    i = i+1;
  end
  month(k) = i;
  day(k) = dayofyear(k)-inttemp;
end

//find hour, minute and second
aux = (days - dayofyear).*24;
hour = fix(aux);
aux = (aux-hour).*60;
minute = fix(aux);
second = (aux-minute).*60;

endfunction
