//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL  license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'http://www.cecill.info'.

function M=CL_fr_G502J2000Mat(cjd,ut1_utc,xp,yp,dPsi,dEps,conv)
// Gamma50 (Veis) to EME2000 (J2000) matrix
//
// Calling Sequence
// M=CL_fr_G502J2000Mat(cjd,[,ut1_utc[,xp,yp[,dPsi,dEps[,conv]]]])
//
// Description
// <itemizedlist><listitem>
// Computes the frame transfer matrix from Gamma50 (Veis) to EME2000 frame.
// <para>Gamma50 (Veis) (Veis frame at date) is defined as:</para>
// <para>- origin at the center of the Earth</para>
// <para>- plan OXY is the true equatorial plan of the date</para>
// <para>- OX axis goes through the Veis equinox of the date</para>
// <para>- OZ axis goes through the true celestial pole of the date</para>
// <para>- OY completes the orthonormal frame</para>
// </listitem>
// <listitem>To convert a vector vect_G50 to vect_J2000, you need to use the (hyper)matrix M : vect_J2000 = M * vect_G50
// <para>To convert a vector vect_J2000 to vect_G50, you need to use the (hyper)matrix M' : vect_G50 = M' * vect_J2000</para></listitem>
// <listitem>Last update : 27/6/2008</listitem>
// </itemizedlist>
//
// Parameters
// cjd: CNES Julian Day (+ut/24). Date of the Gamma50 (Veis) frame. (1xN)
// ut1_utc : (optional) ut1-utc [seconds] (default is 0) (1xN)
// xp : (optional) x polar coordinate [radians] (default is 0) (1xN)
// yp : (optional) y polar coordinate [radians] (default is 0) (1xN)
// dPsi : (optional) Nutation corrections [radians] (default is 0) (1xN)
// dEps : (optional) Nutation corrections [radians] (default is 0) (1xN)
// conv : (optional) Convention IERS. Only iers 1996 (Lieske/Wahr) is implemented (default is "iers_1996")
// M: GAMMA50 to J2000 matrix (3x3xN)
//
// Authors
// CNES - DCT/SB
//
// Bibliography
// 1 CNES - MSLIB FORTRAN 90, Volume R (mr_tsid_veis)
//
// See also
// CL_fr_ter2J2000Mat
// CL_fr_teme2J2000Mat
// CL_fr_H0n2J2000Mat
//
// Examples
//  //conversion of 3 coordinates
//  //tpv: time, position and velocity
// tpv = [3952930.5 3127929.25 4128420.75 -6244.61914 3103.701904 3619.265381]';
// tpv(:,2) = [3945680.5 3125978.25 5687420.75 -6654.61914 4563.701904 3123.265381]';
// cjd = 20870+125/86400;
// position_veis = tpv(1:3,:);
// velocity_veis = tpv(4:6,:);
// M = CL_fr_G502J2000Mat(cjd);
// position_J2000 = M*position_veis;
// velocity_J2000 = M*velocity_veis;
// coord_J2000 = [position_J2000;velocity_J2000];
//
// See 'CelestLab &gt; Demos &gt; ORBIT_PROPERTIES &gt; beta_sun' for more examples
//

// Declarations:


// Code:

[lhs,rhs]=argn();

if ~exists('ut1_utc','local') then ut1_utc=zeros(cjd); end
cjd_ut1 = cjd + ut1_utc/86400.;
tsid = CL_mod_sidTimeG50(cjd_ut1)
Veis = CL_rot_angles2matrix(3,tsid);
//.*.ones... is to make R same size as NP, which could be hypermatrix. If NP is hypermatrix and R is matrix,
//NP*R does not work.


//terrestrial to J2000 matrix
select rhs
  case 1
    NP = CL_fr_ter2J2000Mat(cjd);
  case 2
    NP = CL_fr_ter2J2000Mat(cjd,ut1_utc);
  case 3
    NP = CL_fr_ter2J2000Mat(cjd,ut1_utc,xp);
  case 4
    NP = CL_fr_ter2J2000Mat(cjd,ut1_utc,xp,yp);
  case 5
    NP = CL_fr_ter2J2000Mat(cjd,ut1_utc,xp,yp,dPsi);
  case 6
    NP = CL_fr_ter2J2000Mat(cjd,ut1_utc,xp,yp,dPsi,dEps);
  case 7
    NP = CL_fr_ter2J2000Mat(cjd,ut1_utc,xp,yp,dPsi,dEps,conv);
  else
    error("check number of input arguments");
end

M = NP*Veis;

endfunction
