//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL  license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'http://www.cecill.info'.

function [pos_ter,vel_ter,jacob] = CL_fr_G502ter(cjd,pos_G50,vel_G50,ut1_utc)
// Gamma50 (Veis) to terrestrial frame vector transformation
//
// Calling Sequence
// [pos_ter,[vel_ter,jacob]] = CL_fr_G502ter(cjd,pos_G50,[vel_G50,ut1_utc])
//
// Description
// <itemizedlist><listitem>
// Given the positions <emphasis role="bold">pos_G50</emphasis> and optionnaly <emphasis role="bold">vel_G50</emphasis> in the Gamma50 (Veis) frame,
// this function computes positions <emphasis role="bold">pos_ter</emphasis>, and optionnaly <emphasis role="bold">vel_ter</emphasis> in the terrestrial frame.
// <para>Jacobian of the transformation is also optionnaly computed.</para>
// <para>Gamma50 (Veis) (Veis frame at date) is defined as:</para>
// <para>- origin at the center of the Earth</para>
// <para>- plan OXY is the true equatorial plan of the date</para>
// <para>- OX axis goes through the Veis equinox of the date</para>
// <para>- OZ axis goes through the true celestial pole of the date</para>
// <para>- OY completes the orthonormal frame</para>
// <para> Transformation from Gamma50 to terrestrial frame is a single rotation of the Veis sideral time(as calculated in CL_mod_sidTimeG50)
//  around the Z axis.</para>
// </listitem>
// <listitem>Last update : 27/6/2008</listitem>
// </itemizedlist>
//
// Parameters
// cjd: CNES Julian dates (dates of the Gamma50 (Veis) frame) (1xN)
// pos_G50: position in Gamma50 (Veis) frame [X;Y;Z] (3xN)
// vel_G50: (optionnal) vector in Gamma50 (Veis) frame [X;Y;Z] (3xN)
// ut1_utc : (optional) ut1-utc [seconds] (default is 0) (1xN)
// pos_ter: position in terrestrial frame [X;Y;Z] (3xN)
// vel_ter: (optional) velocity in terrestrial frame [X;Y;Z] (3xN)
// jacob: (optional) jacobian of the transformation [d(x,y,z,vx,vy,vz)_ter/d(x,y,z,vx,vy,vz)_G50] (6x6xN)
//
// Authors
// CNES - DCT/SB
//
// Bibliography
// 1 CNES - MSLIB FORTRAN 90, Volume R (mr_veis_TerVrai)
//
// See also
// CL_fr_ter2J2000Mat
// CL_fr_G502terMat
// CL_fr_ter2G50
// CL_mod_sidTimeG50
//
// Examples
// // Conversion of position G50 to terrestrial
// pos_G50 = [ [3952930.5;3127929.25;4128420.75] , [3945680.5;3125978.25;5687420.75]];
// cjd = [21010 , 21011];
// pos_ter=CL_fr_G502ter(cjd,pos_G50);
//
// // Conversion of position and velocity : G50 to terrestrial  + jacobian
// pos_G50 = [ [3952930.5;3127929.25;4128420.75] , [3945680.5;3125978.25;5687420.75]];
// vel_G50 = [ [1.e3;3.e3;7.e3] , [1.e3;3.e3;7.e3]];
// cjd = [21010 , 21011];
// [pos_ter,vel_ter,jacob]=CL_fr_G502ter(cjd,pos_G50,vel_G50);


// Declarations:


// Code:

compute_vel = %f;
compute_jacob = %f;
vel_ter = [];
jacob = [];

[lhs,rhs] = argn()
if (rhs<2 | rhs > 4) then error('check number of input arguments'); end

if (rhs==3)
  if(lhs ==2)
    compute_vel = %t;
  end
  if(lhs ==3)
    compute_vel = %t;
    compute_jacob = %t;
  end
end

if ~exists('ut1_utc','local') then ut1_utc=zeros(cjd); end

// pos_ter computation :
cjd_ut1 = cjd + ut1_utc/86400.;
[tsid,tsidt] = CL_mod_sidTimeG50(cjd_ut1);

costsid = cos(tsid);
sintsid = sin(tsid);

pos_ter = [ costsid .* pos_G50(1,:) + sintsid .* pos_G50(2,:) ;
           -sintsid .* pos_G50(1,:) + costsid .* pos_G50(2,:) ;
          pos_G50(3,:) ];

// vel_ter computation :
if compute_vel
N = size(pos_ter,2)


vel_ter_iner = [ costsid .* vel_G50(1,:) + sintsid .* vel_G50(2,:) ;
           -sintsid .* vel_G50(1,:) + costsid .* vel_G50(2,:) ;
          vel_G50(3,:) ];

   // calcul du vecteur rotation
   vect_omega = [ zeros(1,N) ;
                  zeros(1,N) ;
          tsidt.*ones(1,N)];


   // vitesse d'entrainement : omega vectoriel position
   vel_entrainement = CL_cross(vect_omega,pos_ter);

   // vitesse par rapport au repere terrestre :
   vel_ter = vel_ter_iner - vel_entrainement;
end


// jacob computation :
if compute_jacob

    jacob = zeros(6,6,N);

  jacob(1,1,:) = costsid;
  jacob(1,2,:) = sintsid;
  jacob(2,1,:) = -sintsid;
  jacob(2,2,:) = costsid;
  jacob(3,3,:) = ones(1,N);

    jacob(4,4,:) = costsid;
  jacob(4,5,:) = sintsid;
  jacob(5,4,:) = -sintsid;
  jacob(5,5,:) = costsid;
  jacob(6,6,:) = ones(1,N);

    // dans les lignes 4 a 6, colonnes 1 a 3 on trouve le resultat de :
  // matrice associee au vecteur rotation * matrice de passage
    //  0    om3  -om2   *  ct  st   0  (et seul om3 est non nul dans notre cas)
    // -om3   0   om1      -st  ct   0
  //  om2  -om1  0        0    0   1

  jacob(4,1,:) = -vect_omega(3,:).*sintsid;
    jacob(4,2,:) = vect_omega(3,:).*costsid;
  jacob(5,1,:) = -vect_omega(3,:).*costsid;
  jacob(5,2,:) = -vect_omega(3,:).*sintsid;


end

endfunction
