//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL  license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'http://www.cecill.info'.

function M=CL_fr_G502terMat(cjd,ut1_utc)
// Gamma50 (Veis) to terrestrial matrix
//
// Calling Sequence
// M=CL_fr_G502terMat(cjd[,ut1_utc])
//
// Description
// <itemizedlist><listitem>
// Computes the frame transfer matrix from Gamma50 to terrestrial frame at given dates.
// <para>Gamma50 (Veis) (Veis frame at date) is defined as:</para>
// <para>- origin at the center of the Earth</para>
// <para>- plan OXY is the true equatorial plan of the date</para>
// <para>- OX axis goes through the Veis equinox of the date</para>
// <para>- OZ axis goes through the true celestial pole of the date</para>
// <para>- OY completes the orthonormal frame</para>
// <para> Transformation from Gamma50 to terrestrial frame is a single rotation of the Veis sideral time(as calculated in CL_mod_sidTimeG50)
//  around the Z axis.</para>
// </listitem>
// <listitem>To convert a vector vect_G50 to vect_ter, you can use the (hyper)matrix M : vect_ter = M * vect_G50
// <para>To convert a vector vect_ter to vect_G50, you can use the (hyper)matrix M' : vect_G50 = M' * vect_ter</para>
// <para><emphasis role="bold">For big hypermatrixes, it is faster to use the functions CL_fr_G502ter and CL_fr_ter2G50</emphasis></para></listitem>
// <listitem>Last update : 27/6/2008</listitem>
// </itemizedlist>
//
// Parameters
// cjd: CNES Julian dates (dates of the Gamma50 (Veis) frame) (1xN)
// ut1_utc : (optional) ut1-utc [seconds] (default is 0) (1xN)
// M: Gamma50 (Veis) to terrestrial frame transfer (hyper)matrix (3x3xN)
//
// Authors
// CNES - DCT/SB
//
// Bibliography
// 1 CNES - MSLIB FORTRAN 90, Volume R (mr_tsid_veis)
//
// See also
// CL_fr_ter2J2000Mat
// CL_fr_G502ter
// CL_fr_ter2G50
// CL_mod_sidTimeG50
// CL_fr_teme2J2000Mat
// CL_fr_H0n2J2000Mat
//
// Examples
// // Conversion G50 to terrestrial
// pos_G50 = [ [3952930.5;3127929.25;4128420.75] , [3945680.5;3125978.25;5687420.75]];
// cjd = [21010 , 21011];
// M=CL_fr_G502terMat(cjd);
// pos_ter = M * (pos_G50);
// pos_ter = CL_fr_G502ter(cjd,pos_G50);
//
// // Conversion terrestrial to G50
// pos_ter = CL_co_sph2car([ [0;%pi/4;%CL_eqRad+700.e3] , [0;-%pi/4;%CL_eqRad+800.e3]]);
// cjd = [21010 , 21011];
// M=CL_fr_G502terMat(cjd);
// pos_G50 = M' * (pos_ter);
// pos_G50 = CL_fr_ter2G50(cjd,pos_ter);

// Declarations:


// Code:

if ~exists('ut1_utc','local') then ut1_utc=zeros(cjd); end

//************************************************************************
//              EXPLICATIONS SUR LES CONSTANTES UTILISEES
//     .................................................................
//     par abus de langage, et pour plus de clarte dans les explications
//     on utilisera 2.pi avec des valeurs en degres dans les formules.
//     .................................................................
//     la reference Georges Veis - the system reference,
//     in Sao special report 200, 1er volume, 1966
//     donne les constantes
//             a1=100.075542 degres
//             a2=360.985612288 degres par jour

//     pour un calcul du temps sideral:
//                        tsid = ( a1 + a2*jour_frac ) modulo 2.pi

//  or jour_frac est exprime dans une echelle de temps cnes (tuc)
//     differente de l'echelle de temps tu1 utilisee pour la formulation
//     du temps sideral ci-dessus. La formule est donc adaptee en:
//                        tsid = ( a1 + a2*jour_frac + a3*rdtu1 ) modulo 2.pi

//        avec a3    = a2/86400 degres par seconde
//             rdtu1 = ecart en seconde entre tu1 et echelle de
//                     datation de jour_frac

//     pour des raisons de pertes de precision numerique l'algorithme
//     est adapte en:
//         tsid = (a1 + 2.pi*frac(jour_frac) + eps*jour_frac + a3*rdtu1 ) mod 2.pi

//        avec frac(jour_frac) = partie fractionnaire de jour_frac
//             eps        = a2 - 360 (en degres par jour)

//      remarque1: on trouve souvent dans la litterature la valeur
//                 erronnee a3 = 0.004177 degres par seconde
//                 qui conduit a une
//                 valeur erronne = 0.729024d-04 radians par secondes
//      remarque2: les valeurs en radians des parameter ont ete verifiees
//                 sous maple
//************************************************************************

cjd_ut1 = cjd + ut1_utc/86400.;
tsid = CL_mod_sidTimeG50(cjd_ut1);
M = CL_rot_angles2matrix(3,tsid);

endfunction
