//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL  license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'http://www.cecill.info'.

function M=CL_fr_H0n2J2000Mat(lon,cjd,ut1_utc,xp,yp,dPsi,dEps,conv)
// H0-n to EME2000 matrix
//
// Calling Sequence
// M=CL_fr_H0n2J2000Mat(lon,cjd,[,ut1_utc[,xp,yp[,dPsi,dEps[,conv]]]])
//
// Description
// <itemizedlist><listitem>
// Computes the frame transfer matrix from H0-n to EME2000 frame.
// <para>H0-n is a planetocentric inertial frame defined as:</para>
// <para>- origin at the center of the planet</para>
// <para>- plan OXY is the planet equator plan</para>
// <para>- OZ axis goes through the planet North pole</para>
// <para>- OX axis is directed according to the direction of the meridian longitude defined by a given
//     longitude (<emphasis role="bold">lon</emphasis>), with respect to the meridian of origin of the planet (at a given date)</para>
// <para>- OY completes the orthonormal frame</para>
// </listitem>
// <listitem><para/><inlinemediaobject><imageobject><imagedata fileref="H0n.gif"/></imageobject></inlinemediaobject></listitem>
// <listitem>To convert a vector vect_H0n to vect_J2000, you need to use the (hyper)matrix M : vect_J2000 = M * vect_H0n
// <para>To convert a vector vect_J2000 to vect_H0n, you need to use the (hyper)matrix M' : vect_H0n = M' * vect_J2000</para></listitem>
// <listitem>Last update : 27/6/2008</listitem>
// </itemizedlist>
//
// Parameters
// lon: east longitude of X axis of H0-n frame with respect to the meridian of origin of the planet[rad]
// cjd: CNES Julian Day (+ut/24). Date of the H0-n frame. (1xN)
// ut1_utc : (optional) ut1-utc [seconds] (default is 0) (1xN)
// xp : (optional) x polar coordinate [radians] (default is 0) (1xN)
// yp : (optional) y polar coordinate [radians] (default is 0) (1xN)
// dPsi : (optional) Nutation corrections [radians] (default is 0) (1xN)
// dEps : (optional) Nutation corrections [radians] (default is 0) (1xN)
// conv : (optional) Convention IERS. Only iers 1996 (Lieske/Wahr) is implemented (default is "iers_1996")
// M: H0-n to EME200 matrix (3x3xN)
//
// Authors
// CNES - DCT/SB
//
// See also
// CL_fr_ter2J2000Mat
// CL_fr_teme2J2000Mat
//
// Examples
//  //conversion of 3 coordinates of Arianne5 ATV launch
// launch_date = CL_dat_cal2cjd(2006,4,14,19,13,43);
// H31= [3952930.5 3127929.25 4128420.75 -6244.61914 3103.701904 3619.265381]';
// K22 = [-3755942.75 -3308836.25 -4361083.5 6356.73584 -2869.25708 -3293.44238]';
// H41 = [-1877905.62 -3909427.35 -5026024.14 7427.6591 -1541.7598 -1576.4651]';
// coord_H03 = [H31 K22 H41];
// position_H03 = coord_H03(1:3,:);
// velocity_H03 = coord_H03(4:6,:);
// // Launch pad ELA3 at the French Guayana Space Centre (CSG) with reference to WGS72 - {8}
// CSG = [CL_deg2rad(-52.7686020300),CL_deg2rad(5.2403222778),-4.215] [deg], latitude of launch pad ELA3 at the French Guayana Space Centre (CSG) with reference to WGS72 - {8}
// M = CL_fr_H0n2J2000Mat(CSG(1),launch_date);
// position_J2000 = M*position_H03;
// velocity_J2000 = M*velocity_H03;
// coord_J2000 = [position_J2000;velocity_J2000];
//

// Declarations:


// Code:

[lhs,rhs]=argn();
select rhs
  case 2
    NP = CL_fr_ter2J2000Mat(cjd);
  case 3
    NP = CL_fr_ter2J2000Mat(cjd,ut1_utc);
  case 4
    NP = CL_fr_ter2J2000Mat(cjd,ut1_utc,xp);
  case 5
    NP = CL_fr_ter2J2000Mat(cjd,ut1_utc,xp,yp);
  case 6
    NP = CL_fr_ter2J2000Mat(cjd,ut1_utc,xp,yp,dPsi);
  case 7
    NP = CL_fr_ter2J2000Mat(cjd,ut1_utc,xp,yp,dPsi,dEps);
  case 8
    NP = CL_fr_ter2J2000Mat(cjd,ut1_utc,xp,yp,dPsi,dEps,conv);
  else
    error("check number of input arguments");
end

R = CL_rot_angles2matrix(3,-lon.*.ones(1,size(cjd,2)));
//.*.ones... is to make R same size as NP, which could be hypermatrix. If NP is hypermatrix and R is matrix,
//NP*R does not work.

M = NP*R;

endfunction
