//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL  license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'http://www.cecill.info'.

function [pos_G50,vel_G50,jacob] = CL_fr_J20002G50(cjdutc,pos_J2000,vel_J2000,ut1_utc,xp,yp,dPsi,dEps,conv)
// EME2000 (J2000) to Gamma50 (Veis) frame vector transformation
//
// Calling Sequence
// [pos_G50,[vel_G50,jacob]] = CL_fr_J20002G50(cjdutc,pos_J2000,[vel_J2000,ut1_utc,xp,yp,dPsi,dEps,conv])
//
// Description
// <itemizedlist><listitem>
// Computes the frame transfer of <emphasis role="bold">pos_J2000</emphasis> and optionally <emphasis role="bold">vel_J2000</emphasis> (EME2000 frame)
// to <emphasis role="bold">pos_G50</emphasis> and optionally <emphasis role="bold">vel_G50</emphasis> (Gamma50 (Veis) frame) at given dates (<emphasis role="bold">cjdutc</emphasis>).
// <para>Jacobian of the transformation is also optionally computed.</para>
// <para>Gamma50 (Veis) (Veis frame at date) is defined as:</para>
// <para>- origin at the center of the Earth</para>
// <para>- plan OXY is the true equatorial plan of the date</para>
// <para>- OX axis goes through the Veis equinox of the date</para>
// <para>- OZ axis goes through the true celestial pole of the date</para>
// <para>- OY completes the orthonormal frame</para>
// </listitem>
// <listitem><para/><inlinemediaobject><imageobject><imagedata fileref="J2000.gif"/></imageobject></inlinemediaobject></listitem>
// <listitem>Last update : 27/6/2008</listitem>
// </itemizedlist>
//
// Parameters
// cjdutc: CNES Julian Date (1950) in TUC time (1xN)
// pos_J2000: position in EME2000 frame (3xN)
// vel_J2000: (optional) velocity in EME2000 frame (3xN)
// ut1_utc : (optional) ut1-utc [seconds] (default is 0) (1xN)
// xp : (optional) x polar coordinate [radians] (default is 0) (1xN)
// yp : (optional) y polar coordinate [radians] (default is 0) (1xN)
// dPsi : (optional) Nutation corrections [radians] (default is 0) (1xN)
// dEps : (optional) Nutation corrections [radians] (default is 0) (1xN)
// conv : (optional) Convention IERS. Only iers 1996 (Lieske/Wahr) is implemented (default is "iers_1996")
// pos_G50: position in Gamma50 (Veis) frame (3xN)
// vel_G50: (optional) velocity in Gamma50 (Veis) frame (3xN)
// jacob: (optional) jacobian of the transformation (6x6xN)
//
// Authors
// CNES - DCT/SB
//
// Bibliography
// 1 CNES - MSLIB FORTRAN 90, Volume R (mr_tsid_veis)
//
// See also
// CL_fr_G502J2000Mat
// CL_fr_G502J2000
//
// Examples
// // Conversion of position J2000 to G50
// pos_J2000 = [ [3952930.5;3127929.25;4128420.75] , [3945680.5;3125978.25;5687420.75]];
// cjd = [21010 , 21011];
// pos_G50=CL_fr_J20002G50(cjd,pos_J2000);
//
// // Conversion of position and velocity : J2000 to G50  + jacobian
// pos_J2000 = [ [3952930.5;3127929.25;4128420.75] , [3945680.5;3125978.25;5687420.75]];
// vel_J2000 = [ [1.e3;3.e3;7.e3] , [1.e3;3.e3;7.e3]];
// cjd = [21010 , 21011];
// [pos_G50,vel_G50,jacob]=CL_fr_J20002G50(cjd,pos_J2000,vel_J2000);

// Declarations:


// Code:

[lhs,rhs]=argn();

if ~exists('ut1_utc','local') then ut1_utc=zeros(cjdutc); end

select rhs
  case 2
    [pos_ter] = CL_fr_J20002ter(cjdutc,pos_J2000);
  [pos_G50] = CL_fr_ter2G50(cjdutc,pos_ter);
  case 3
    // no jacobian :
    if(lhs ==2)
    [pos_ter,vel_ter] = CL_fr_J20002ter(cjdutc,pos_J2000,vel_J2000);
    [pos_G50,vel_G50] = CL_fr_ter2G50(cjdutc,pos_ter,vel_ter);
  elseif(lhs ==3)
      [pos_ter,vel_ter,jacob1] = CL_fr_J20002ter(cjdutc,pos_J2000,vel_J2000);
    [pos_G50,vel_G50,jacob2] = CL_fr_ter2G50(cjdutc,pos_ter,vel_ter);
    jacob = jacob2*jacob1;
    else
       error("check number of output arguments");
  end
  else
       error("check number of input arguments");
end


endfunction
