//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL  license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'http://www.cecill.info'.

function [vect_qsw] = CL_fr_inertial2qsw(pos_car,vel_car,vect_geo)
// Inertial to local orbital frame (q,s,w) vector transformation
//
// Calling Sequence
// [vect_qsw] = CL_fr_inertial2qsw(pos_car,vel_car,vect_geo)
//
// Description
// <itemizedlist><listitem>
// Given the vector <emphasis role="bold">vect_geo</emphasis> expressed in a geocentric inertial frame, this function computes
// its coordinates <emphasis role="bold">vect_qsw</emphasis> in the local orbital frame (<emphasis role="bold">q</emphasis>,<emphasis role="bold">s</emphasis>,<emphasis role="bold">w</emphasis>) defined
// by satellite's position (<emphasis role="bold">pos_car</emphasis>) and velocity (<emphasis role="bold">vel_car</emphasis>) in the geocentric inertial frame.
// </listitem>
// <listitem>The QSW local orbital frame is defined as follows:
// <para>origin: center of gravity of the satellite</para>
// <para>q: a unit vector from the planet center towards the satellite</para>
// <para>w: a unit vector in the direction of the osculating orbit's kinetic moment</para>
// <para>s: a unit vector chosen so that qsw is a direct orthonormal frame</para>
// <inlinemediaobject><imageobject><imagedata fileref="qsw.gif"/></imageobject></inlinemediaobject></listitem>
// <listitem>Last update : 07/11/2007</listitem>
// </itemizedlist>
//
// Parameters
// pos_car: satellite's cartesien position in a geocentric inertial frame [m] (3xN)
// vel_car: satellite's cartesien velocity in a geocentric inertial frame [m/s] (3xN)
// vect_geo: vector cartesien coordinates in a geocentric inertial frame [m] (3xN)
// vect_qsw: vector vect_geo in the local orbital frame (q,s,w) [m] (3xN)
//
// Bibliography
// 1 Mecanique spatiale, CNES - Cepadues 1995, Tome I, section 10.2.2.3 (Definition du Repere orbital local)
// 2 CNES - MSLIB FORTRAN 90, Volume O (mo_geo_qsw)
//
// Authors
// CNES - DCT/SB
//
// See also
// CL_fr_qsw2inertial
// CL_fr_qswMat
// CL_fr_tnwMat
// CL_fr_inertial2tnw
// CL_fr_tnw2inertial
//
// Examples
// // Example 1
// pos_car = [- 700700.,5.325E-11,6.718E-11;- 700700.,5.325E-11,6.718E-11]'
// vel_car = [- 2.919E-12,- 14807.462,- 18682.367;- 2.919E-12,- 14807.462,- 18682.367]'
// vect_geo = [1,0,0;1,1,1]'
// [vect_qsw] = CL_fr_inertial2qsw(pos_car,vel_car,vect_geo)
//

// Declarations:


// Code:

[lhs,rhs] = argn()
if rhs~=3 then error('check number of input arguments'); end

//calcul du vecteur q
q = CL_unitVector(pos_car);
// calcul du produit vectoriel pos^vel
w = CL_unitVector(CL_cross(pos_car,vel_car));
// calcul du produit vectoriel w^q
s = CL_cross(w,q)

//vector coordinates in (q,s,w)
vect_qsw = [ vect_geo(1,:).*q(1,:) + vect_geo(2,:).*q(2,:) + vect_geo(3,:).*q(3,:) ;
             vect_geo(1,:).*s(1,:) + vect_geo(2,:).*s(2,:) + vect_geo(3,:).*s(3,:) ;
       vect_geo(1,:).*w(1,:) + vect_geo(2,:).*w(2,:) + vect_geo(3,:).*w(3,:)];

endfunction
