//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL  license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'http://www.cecill.info'.

function [M] = CL_fr_lvlhMat(pos,vel)
// Local orbital frame (LVLH) definition in an inertial frame
//
// Calling Sequence
// [M] = CL_fr_lvlhMat(pos,vel)
//
// Description
// <itemizedlist><listitem>
// Given satellite's position and velocity vectors in a geocentric inertial frame,
// this function computes the frame transfer matrix from the inertial frame to the local orbital frame LVLH.</listitem>
// <listitem>The LVLH local frame is defined as follows:
// <para>origin: center of gravity of the target</para>
// <para>x: a unit vector that completes the direct orthogonal trihedral (if the orbit is circular, x coincides with the velocity vector)</para>
// <para>z: a unit vector directed towards the planet center (opposite to the position vector)</para>
// <para>y: a unit vector in the opposite direction of the angular momemtum </para>
// <inlinemediaobject><imageobject><imagedata fileref="lvlh.gif"/></imageobject></inlinemediaobject></listitem>
// <listitem>Rows of matrix M are the following : M(1,:) = x, M(2,:) = y and M(3,:) = z </listitem>
// <listitem>To convert vectors from and to LVLH local frame, you can use one of the examples</listitem>
// <listitem>Last update : 31/10/2007</listitem>
// </itemizedlist>
//
// Parameters
// pos: satellite's cartesien position in a geocentric inertiel frame [m] (3xN)
// vel: satellite's cartesien velocity in a geocentric inertiel frame [m/s] (3xN)
// M : frame transfer matrix from an inertial frame to the LVLH local frame (3x3xN)
//
// Bibliography
// 1 Mecanique spatiale, CNES - Cepadues 1995, Tome II, section 16.3.2.1
// 2 Orbital Mechanics for engineering students, H D Curtis, Chapter 7
//
// Authors
// CNES - DCT/SB
//
// See also
// CL_fr_qswMat
// CL_fr_tnwMat
//
// Examples
// // Inertial to LVLH :
// pos_car = [- 700700.,5.325E-11,6.718E-11;- 700700.,5.325E-11,6.718E-11]'
// vel_car = [- 2.919E-12,- 14807.462,- 18682.367;- 2.919E-12,- 14807.462,- 18682.367]'
// [M] = CL_fr_lvlhMat(pos_car,vel_car)
// pos_lvlh = M*pos_car;
//
// // LVLH to inertial :
// pos_car = M'*pos_lvlh;

// Declarations:


// Code:

[lhs,rhs] = argn()

if rhs~=2 then error('check number of input arguments'); end

N = size(pos,2);


z = -CL_unitVector(pos);
y = -CL_unitVector(CL_cross(pos,vel));
x = CL_cross(y,z);


// Matrix M :
M = hypermat([3 3 N] , [x(1,:); y(1,:); z(1,:); ...
            x(2,:); y(2,:); z(2,:); ...
            x(3,:); y(3,:); z(3,:)]);
if(N == 1)
    M = M(:,:,1);
end


endfunction
