//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL  license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'http://www.cecill.info'.

function [M] = CL_fr_qswMat (pos_car,vel_car)
// Local orbital frame (q,s,w) definition in an inertial frame
//
// Calling Sequence
// [M] = CL_fr_qswMat(pos_car,vel_car)
//
// Description
// <itemizedlist><listitem>
// Given satellite's position and velocity vectors in a geocentric inertial frame,
// this function computes the frame transfer matrix from the inertial frame to the local orbital frame QSW.</listitem>
// <listitem>The QSW local orbital frame is defined as follows:
// <para>origin: center of gravity of the satellite</para>
// <para>q: a unit vector from the planet center towards the satellite</para>
// <para>w: a unit vector in the direction of the osculating orbit's kinetic moment</para>
// <para>s: a unit vector chosen so that qsw is a direct orthonormal frame</para>
// <inlinemediaobject><imageobject><imagedata fileref="qsw.gif"/></imageobject></inlinemediaobject></listitem>
// <listitem>To convert a vector pos_car to pos_qsw, you can use the (hyper)matrix M : pos_qsw = M*pos_car
// <para>To convert a vector pos_qsw to pos_car, you can use the (hyper)matrix M' : pos_car = M'*pos_qsw</para>
// <para><emphasis role="bold">For big hypermatrixes, it is faster to use the functions CL_fr_qsw2inertial and CL_fr_inertial2qsw</emphasis></para></listitem>
// <listitem>Last update : 07/11/2007</listitem>
// </itemizedlist>
//
// Parameters
// pos_car: satellite's cartesien position in a geocentric inertiel frame [m] (3xN)
// vel_car: satellite's cartesien velocity in a geocentric inertiel frame [m/s] (3xN)
// M : frame transfer matrix from an inertial frame to the QSW local frame (3x3xN)
//
// Bibliography
// 1 Mecanique spatiale, CNES - Cepadues 1995, Tome I, section 10.2.2.3 (Definition du Repere orbital local)
// 2 CNES - MSLIB FORTRAN 90, Volume O (Les Reperes orbitaux locaux)
//
// Authors
// CNES - DCT/SB
//
// See also
// CL_fr_qsw2inertial
// CL_fr_inertial2qsw
// CL_fr_tnwMat
// CL_fr_inertial2tnw
// CL_fr_tnw2inertial
//
// Examples
// // Inertial to QSW :
// pos_car = [- 700700.,5.325E-11,6.718E-11;- 700700.,5.325E-11,6.718E-11]'
// vel_car = [- 2.919E-12,- 14807.462,- 18682.367;- 2.919E-12,- 14807.462,- 18682.367]'
// [M] = CL_fr_qswMat(pos_car,vel_car)
// pos_qsw = M*pos_car;
//
// // QSW to inertial :
// pos_car = M'*pos_qsw;

// Declarations:


// Code:

[lhs,rhs] = argn()

if rhs~=2 then error('check number of input arguments'); end

N = size(pos_car,2);

//calcul du vecteur q
q = CL_unitVector(pos_car);

// calcul du produit vectoriel pos^vel
w = CL_unitVector(CL_cross(pos_car,vel_car));

// calcul du produit vectoriel w^q
s = CL_cross(w,q)

// Matrix M :
M = hypermat([3 3 N] , [q(1,:); s(1,:); w(1,:); ...
            q(2,:); s(2,:); w(2,:); ...
            q(3,:); s(3,:); w(3,:)]);
if(N == 1)
    M = M(:,:,1);
end

endfunction
