//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL  license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'http://www.cecill.info'.

function M=CL_fr_teme2J2000Mat(cjdutc,ut1_utc,dPsi,dEps,conv)
// TEME to J2000 matrix
//
// Calling Sequence
// M=CL_fr_teme2J2000Mat(cjdutc[,ut1_utc[,dPsi,dEps[,conv]]])
//
// Description
// <itemizedlist><listitem>
// Computes the frame transfer matrix from TEME (true equator and mean equinox) to EME2000 (J2000) frame.</listitem>
// <listitem>To convert a vector vect_TEME to vect_J2000, you need to use the (hyper)matrix M : vect_J2000 = M * vect_TEME
// <para>To convert a vector vect_J2000 to vect_TEME, you need to use the (hyper)matrix M' : vect_TEME = M' * vect_J2000</para></listitem>
// <listitem>Last update : 27/6/2008</listitem>
// </itemizedlist>
//
// Parameters
// cjdutc: CNES Julian Date (1950) in TUC time (1xN)
// ut1_utc : (optional) ut1-utc [seconds] (default is 0) (1xN)
// dPsi : (optional) Nutation corrections [radians] (default is 0) (1xN)
// dEps : (optional) Nutation corrections [radians] (default is 0) (1xN)
// conv : (optional) Convention IERS. Only iers 1996 (Lieske/Wahr) is implemented (default is "iers_1996")
// M: TEME to EME2000 matrix (3x3xN)
//
// Authors
// CNES - DCT/SB
//
// Bibliography
// 1 IERS Conventions (1996), Dennis D. McCarthy
// 2 Explanatory Supplement to the Astronomical Almanac, Seidelman (1992)
// 3 Vallado, D.A., J.H. Seago, "The coordinate frames of the US Space Objects Catalogs", AIAA 2000-4025
//
// See also
// CL_mod_precessionMatrix
// CL_mod_nutationMatrix
// CL_mod_equinoxesEquation
// CL_fr_ter2J2000Mat
// CL_fr_H0n2J2000Mat
//
// Examples
// cart_teme = [1 2 3 4 5 6]'
// pos_teme = cart_teme(1:3)
// vel_teme = cart_teme(4:6)
// M = CL_fr_teme2J2000Mat(20557+70254/86400.0)
// pos_j2000 = M*pos_teme
// vel_j2000 = M*vel_teme
// cart_j2000 = [pos_j2000;vel_j2000]
//

// Declarations:


// Code:

if ~exists('ut1_utc','local') then ut1_utc=zeros(cjdutc); end
if ~exists('dPsi','local') then dPsi=zeros(cjdutc); end
if ~exists('dEps','local') then dEps=zeros(cjdutc); end
//error("4 par en entree: il manque une correction de nutation...");
if ~exists('conv','local') then conv="iers_1996"; end

//verif dimension des args en entree
sz=size(cjdutc);
if (sz(1)~=1) then error("check dim of input arguments (column vect.)"); end
sz=size(ut1_utc);
if (sz(1)~=1) then error("check dim of input arguments (column vect.)"); end
sz=size(dPsi);
if (sz(1)~=1) then error("check dim of input arguments (column vect.)"); end
sz=size(dEps);
if (sz(1)~=1) then error("check dim of input arguments (column vect.)"); end

//Conversion TUC --> TAI
[cjdtai]=CL_dat_utc2tai(cjdutc);
jj_tai = floor(cjdtai);
sec_tai = (cjdtai-jj_tai)*86400.0;

[NUT,F] = CL_mod_nutationAngles(jj_tai,sec_tai,"p");
eps0 = CL_mod_meanObliquity(jj_tai,sec_tai,"n");

//equinoxes equation and greenwich mean sidereal time
[equi,equip] = CL_mod_equinoxesEquation(NUT(1,:),eps0,F(5,:));
Q = CL_rot_angles2matrix(3,-equi);

//nutation
NUT = NUT+[dPsi;dEps];
[N,OM,OMP] = CL_mod_nutationMatrix([eps0; NUT;],[],[],"n");

//precession
[K,KP,KPP] = CL_mod_precessionAngles(jj_tai,sec_tai,"n");
[P,OM,OMP] = CL_mod_precessionMatrix(K,KP,KPP,"n");

//transformation matrix
M = P*N*Q;

endfunction
