//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL  license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'http://www.cecill.info'.

function M = CL_fr_ter2J2000Mat(cjdutc,ut1_utc,xp,yp,dPsi,dEps,conv)
// Terrestrial to EME2000 (J2000) matrix
//
// Calling Sequence
// M = CL_fr_ter2J2000Mat(cjdutc[,ut1_utc[,xp,yp[,dPsi,dEps[,conv]]]])
//
// Description
// <itemizedlist><listitem>
// Computes the frame transfer matrix from terrestrial to EME2000 at given dates (in CNES julian days and TUC time)
// </listitem>
// <listitem><para/><inlinemediaobject><imageobject><imagedata fileref="J2000.gif"/></imageobject></inlinemediaobject></listitem>
// <listitem>To convert a vector vect_ter to vect_J2000, you can use the (hyper)matrix M : vect_J2000 = M * vect_ter
// <para>To convert a vector vect_J2000 to vect_ter, you can use the (hyper)matrix M' : vect_ter = M' * vect_J2000</para>
// <para><emphasis role="bold">For big hypermatrixes, it is faster to use the functions CL_fr_ter2J2000 and CL_fr_J20002ter</emphasis></para></listitem>
// <listitem>Last update : 27/6/2008</listitem>
// </itemizedlist>
//
// Parameters
// cjdutc: CNES Julian Date (1950) in TUC time (1xN)
// ut1_utc : (optional) ut1-utc [seconds] (default is 0) (1xN)
// xp : (optional) x polar coordinate [radians] (default is 0) (1xN)
// yp : (optional) y polar coordinate [radians] (default is 0) (1xN)
// dPsi : (optional) Nutation corrections [radians] (default is 0) (1xN)
// dEps : (optional) Nutation corrections [radians] (default is 0) (1xN)
// conv : (optional) Convention IERS. Only iers 1996 (Lieske/Wahr) is implemented (default is "iers_1996")
// M: terrestrial to EME2000 transfer (hyper)matrix (3x3xN)
//
// Authors
// CNES - DCT/SB
//
// Bibliography
// 1 IERS Conventions (1996), Dennis D. McCarthy
// 2 Explanatory Supplement to the Astronomical Almanac, Seidelman (1992)
//
// See also
// CL_mod_precessionMatrix
// CL_mod_nutationMatrix
// CL_mod_polarMotionMatrix
//
// Examples
// // Conversion J2000 to terrestrial
// pos_J2000 = [ [3952930.5;3127929.25;4128420.75] , [3945680.5;3125978.25;5687420.75]];
// cjd = [21010 , 21011];
// M=CL_fr_ter2J2000Mat(cjd);
// pos_ter = M' * (pos_J2000);
// // Conversion terrestrial to J2000
// pos_ter = CL_co_sph2car([ [0;%pi/4;%CL_eqRad+700.e3] , [0;-%pi/4;%CL_eqRad+800.e3]]);
// cjd = [21010 , 21011];
// M=CL_fr_ter2J2000Mat(cjd);
// pos_J2000 = M * (pos_ter);

// Declarations:


// Code:

[lhs,rhs]=argn();

if ~exists('ut1_utc','local') then ut1_utc = zeros(cjdutc); end
if ~exists('xp','local') then xp = zeros(cjdutc); end
if ~exists('yp','local') then yp = zeros(cjdutc); end
if ~exists('dPsi','local') then dPsi = zeros(cjdutc); end
if ~exists('dEps','local') then dEps = zeros(cjdutc); end
if ~exists('conv','local') then conv = "iers_1996"; end

if conv~="iers_1996" then
      error("only model iers_1996 is implemented, sorry...");
end

//verif dimension des args en entree
sz=size(cjdutc);
if (sz(1)~=1) then error("check dim of input arguments (column vect.)"); end
sz=size(ut1_utc);
if (sz(1)~=1) then error("check dim of input arguments (column vect.)"); end
sz=size(xp);
if (sz(1)~=1) then error("check dim of input arguments (column vect.)"); end
sz=size(yp);
if (sz(1)~=1) then error("check dim of input arguments (column vect.)"); end
sz=size(dPsi);
if (sz(1)~=1) then error("check dim of input arguments (column vect.)"); end
sz=size(dEps);
if (sz(1)~=1) then error("check dim of input arguments (column vect.)"); end


W=CL_mod_polarMotionMatrix(xp,yp);

//Conversion TUC --> TAI
[cjdtai]=CL_dat_utc2tai(cjdutc);
jj_tai = floor(cjdtai);
sec_tai = (cjdtai-jj_tai)*86400.0;

cjdut1 = cjdutc + ut1_utc/86400.0;

TempsSideralMoyen = CL_mod_sidTime(cjdut1);

eps0 = CL_mod_meanObliquity(jj_tai,sec_tai,"n");

[NUT,F] = CL_mod_nutationAngles(jj_tai,sec_tai,"p");

[equi,equip] = CL_mod_equinoxesEquation(NUT(1,:),eps0,F(5,:));
TempsSideralVrai = TempsSideralMoyen+equi;
R = CL_rot_angles2matrix(3,-TempsSideralVrai);

NUT=NUT+[dPsi;dEps];
[N,OM,OMP] = CL_mod_nutationMatrix([eps0; NUT;],[],[],"n");

[K,KP,KPP]=CL_mod_precessionAngles(jj_tai,sec_tai,"n");
[PREC,OM,OMP] = CL_mod_precessionMatrix(K,KP,KPP,"n");

M=PREC*N*R*W;

endfunction
