//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL  license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'http://www.cecill.info'.

function [M] = CL_fr_tnwMat (pos_car,vel_car)
// Local orbital frame (t,n,w) definition in an inertial frame
//
// Calling Sequence
// [M] = CL_fr_tnwMat(pos_car,vel_car)
//
// Description
// <itemizedlist><listitem>
// Given satellite's position and velocity vectors in an inertial frame,
// this function computes the frame transfer matrix from the inertial frame to the local orbital frame TNW.</listitem>
// <listitem>The TNW local frame is defined as follows:
// <para>origin: center of gravity of satellite</para>
// <para>t: unit vector in the direction of satellite's velocity</para>
// <para>w: unit vector of the kinetic moment of the osculating orbit</para>
// <para>n: unit vector chosen so TNW is a direct orthonormal frame</para>
// <inlinemediaobject><imageobject><imagedata fileref="tnw.gif"/></imageobject></inlinemediaobject></listitem>
// <listitem>To convert a vector pos_car to pos_tnw, you can use the (hyper)matrix M : pos_tnw = M*pos_car
// <para>To convert a vector pos_tnw to pos_car, you can use the (hyper)matrix M' : pos_car = M'*pos_tnw;</para>
// <para><emphasis role="bold">For big hypermatrixes, it is faster to use the functions CL_fr_inertial2tnw and CL_fr_tnw2inertial</emphasis></para></listitem>
// <listitem>Last update : 07/11/2007</listitem>
// </itemizedlist>
//
// Parameters
// pos_car: satellite's cartesien position in a geocentric inertiel frame [m] (3xN)
// vel_car: satellite's cartesien velocity in a geocentric inertiel frame [m/s] (3xN)
// M : frame transfer matrix from an inertial frame to the TNW local frame (3x3xN)
//
// Bibliography
// 1 Mecanique spatiale, CNES - Cepadues 1995, Tome I, section 10.2.2.3 (Definition du Repere orbital local)
// 2 CNES - MSLIB FORTRAN 90, Volume O (mo_def_tnw)
//
// Authors
// CNES - DCT/SB
//
// See also
// CL_fr_qsw2inertial
// CL_fr_inertial2qsw
// CL_fr_qswMat
// CL_fr_inertial2tnw
// CL_fr_tnw2inertial
//
// Examples
// // Inertial to TNW :
// pos_car = [- 700700.,5.325E-11,6.718E-11;- 700700.,5.325E-11,6.718E-11]'
// vel_car = [- 2.919E-12,- 14807.462,- 18682.367;- 2.919E-12,- 14807.462,- 18682.367]'
// [M] = CL_fr_tnwMat(pos_car,vel_car)
// pos_tnw = M*pos_car;
//
// // TNW to inertial :
// pos_car = M'*pos_tnw;

// Declarations:


// Code:

[lhs,rhs] = argn()

if rhs~=2 then error('check number of input arguments'); end

N = size(pos_car,2);

//vector t, tangent to velocity vector
t = CL_unitVector(vel_car)

//CL_cross product pos_car^vel
w0 = CL_cross(pos_car,vel_car)

//vector w, giving direction of kinetic moment
w = CL_unitVector(w0)

//CL_cross product: n = w^t
n = CL_cross(w,t)

// Matrix M :
M = hypermat([3 3 N] , [t(1,:); n(1,:); w(1,:); ...
            t(2,:); n(2,:); w(2,:); ...
            t(3,:); n(3,:); w(3,:)]);
if(N == 1)
    M = M(:,:,1);
end

endfunction
