//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL  license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'http://www.cecill.info'.

function [pos_ter,vel_ter,jacob]=CL_fr_topoN2ter(orig,pos_topoN,vel_topoN,er,obl)
// Topocentric north to terrestrial frame vector transformation
//
// Calling Sequence
// [pos_ter[,vel_ter[,jacob]] = CL_fr_topoN2ter(orig[,pos_topoN[,vel_topoN[,er[,obl]]]])
//
// Description
// <itemizedlist><listitem>
// Given position vector in topocentric North frame (<emphasis role="bold">pos_topoN</emphasis>) and frame origin (<emphasis role="bold">lon</emphasis>,<emphasis role="bold">lat</emphasis>), this function
// computes its components in earth reference frame (<emphasis role="bold">pos_ter</emphasis>).
// <para>Velocity and jacobian are optionally computed.</para>
// <para>NOTE: position and velocity are not necessary to compute jacobian. In this case, zero vectors are given for pos_ter
// and vel_ter.</para>
// <para>Warning : The origin of a topocentric North frame is not the center of the planet : it is the given point of longitude and latitude</para>
// </listitem>
// <listitem>The topocentric north frame is defined as follows:
// <para>x: a unit vector towards norh</para>
// <para>y: a unit vector (local paralel tangent) towards west</para>
// <para>z: a unit vector in direction of the local vertical </para>
// <inlinemediaobject><imageobject><imagedata fileref="topoN.gif"/></imageobject></inlinemediaobject></listitem>
// <listitem>Last update : 24/07/2007</listitem>
// </itemizedlist>
//
// Parameters
// orig: [LON;LAT;H] topocentric frame origin elliptical (geodetic) coordinates [rad],[m] (3x1 : same origin for all positions or 3xN : one origin for each position)
// pos_topoN: (optional) [X;Y;Z] position coordinates in topocentric North frame [m] (3xN)
// vel_topoN: (optional) [Vx;Vy;Vz] velocity coordinates in topocentric North frame [m/s] (3xN)
// er: (optional) ellipsoid equatorial radius (default is %CL_eqRad) [m]
// obl: (optional) ellipsoid oblateness (default is %apla)
// pos_ter: [X;Y;Z] position coordinates in earth reference frame [m] (3xN)
// vel_ter: (optional) [Vx;Vy;Vz] velocity coordinates in earth reference frame [m/s] (3xN)
// jacob: (optional) transformation jacobian d(x,y,z,vx,vy,vz)_ter/d(x,y,z,vx,vy,vz)_topoN (6x6x size(orig))
//
// Authors
// CNES - DCT/SB
//
// Bibliography
// CNES - MSLIB FORTRAN 90, Volume T (mt_topo_N_ref)
//
// See also
// CL_fr_topoNMat
// CL_fr_ter2topoN
//
// Examples
// [pos,vel,jacob] = CL_fr_topoN2ter([%pi;%pi/2;100]) //compute only jacobian
// [pos,vel] = CL_fr_topoN2ter([%pi;%pi/2;100],[50;25;32],[70;100;0]) //compute position and velocity
// [pos] = CL_fr_topoN2ter([%pi/3;%pi/4;100],[50,25,32;50,50,0]') //compute only position (2 positions vectorized)
//

// Declarations:
global %CL_eqRad %CL_obla;

// Code:

[lhs,rhs] = argn()

No = size(orig,2);
if exists('pos_topoN','local')
  Np = size(pos_topoN,2);
  if (No ~= 1 & No ~= Np) then error('orig must be either 3x1 or the same size of pos_topoN (3xN)'); end
  N=Np;
else
  N=No;
end

select lhs
  case 3
    compute_jacob=%t
    compute_vel=%t
    compute_pos=%t
    if ~exists('vel_topoN','local')
      vel_ter = zeros(3,N)
      warning('Zero vel_ter vector given');
      compute_vel=%f
    end
    if ~exists('pos_topoN','local')
      pos_ter = zeros(3,N)
      warning('Zero pos_ter vector given');
      compute_pos=%f
    end
  case 2
    compute_jacob=%f
    compute_vel=%t
    compute_pos=%t
  case 1
    compute_jacob=%f
    compute_vel=%f
    compute_pos=%t
  else
    error('bad number of arguments')
end

if ~exists('er','local') then er=%CL_eqRad; end
if ~exists('obl','local') then obl=%CL_obla; end

//topocentric frame vectors
lon = orig(1,:)*ones(1,N);
lat = orig(2,:)*ones(1,N);

//vect_i computation: horizontal towards norh
//vect_i = (-sin(lat)cos(long),-sin(lat)sin(long),cos(lat))
vect_i(1,:) = -sin(lat).*cos(lon);
vect_i(2,:) = -sin(lat).*sin(lon);
vect_i(3,:) = cos(lat);

//vect_j computation: local paralel tangent towards west
//vect_j = (sin(long),-cos(long),0)
vect_j(1,:) = sin(lon);
vect_j(2,:) = -cos(lon);
vect_j(3,:) = zeros(1,No);

//vect_k computation: local vertical
//vect_k = (cos(lat)cos(long),cos(lat)sin(long),sin(lat))
vect_k(1,:) = cos(lat).*cos(lon);
vect_k(2,:) = cos(lat).*sin(lon);
vect_k(3,:) = sin(lat);

if compute_pos
  //topocentric frame origin cartesian coordinates
  sta = CL_co_ell2car(orig,er,obl)
  if(No ==1)
  sta = sta.*.ones(1,N)
  end

  //satellite's position components in geocentric frame
  pos_tmp = [pos_topoN(1,:).*vect_i(1,:) + pos_topoN(2,:).*vect_j(1,:) + pos_topoN(3,:).*vect_k(1,:) ;
       pos_topoN(1,:).*vect_i(2,:) + pos_topoN(2,:).*vect_j(2,:) + pos_topoN(3,:).*vect_k(2,:) ;
       pos_topoN(1,:).*vect_i(3,:) + pos_topoN(2,:).*vect_j(3,:) + pos_topoN(3,:).*vect_k(3,:) ] ;
  pos_ter = pos_tmp + sta;
end

//JACOB
//partial derivates of position and velocity vectors
if compute_jacob

  if(No ==1)
  jacob = zeros(6,6);

  jacob(1:3,1) = vect_i(:,1)
  jacob(1:3,2) = vect_j(:,1)
  jacob(1:3,3) = vect_k(:,1)
  jacob(4:6,4) = vect_i(:,1)
  jacob(4:6,5) = vect_j(:,1)
  jacob(4:6,6) = vect_k(:,1)

  else
  jacob = hypermat([6,6,N])

  jacob(1:3,1,:) = vect_i
  jacob(1:3,2,:) = vect_j
  jacob(1:3,3,:) = vect_k
  jacob(4:6,4,:) = vect_i
  jacob(4:6,5,:) = vect_j
  jacob(4:6,6,:) = vect_k
  if N==1 then jacob = jacob(:,:,1); end
  end


end

//velocity on geocentric frame
if compute_vel
  vel_ter = [vel_topoN(1,:).*vect_i(1,:) + vel_topoN(2,:).*vect_j(1,:) + vel_topoN(3,:).*vect_k(1,:) ;
       vel_topoN(1,:).*vect_i(2,:) + vel_topoN(2,:).*vect_j(2,:) + vel_topoN(3,:).*vect_k(2,:) ;
       vel_topoN(1,:).*vect_i(3,:) + vel_topoN(2,:).*vect_j(3,:) + vel_topoN(3,:).*vect_k(3,:) ];
end

endfunction
