//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL  license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'http://www.cecill.info'.

function M = CL_rot_angles2matrix(axes,angles)
// Rotation angles matrix
//
// Calling Sequence
// M = CL_rot_angles2matrix(axes,angles)
//
// Parameters
// axes: vecteur with rotations order (1xN, Nmax = 3 ).
// angles: Euler angles (NxP, Nmax = 3 ) [rad]
// M: frame matrix. If N = 1, and P=1 M is a (3x3) matrix; if P >1 , M is a (3x3xP) hypermatrix.
//
// Description
// <itemizedlist><listitem>
// Computes the matrix M such that M' is the rotation matrix from angle <emphasis role="bold">angles(1)</emphasis> around first axis,
// then <emphasis role="bold">angles(2)</emphasis> around second axis, then <emphasis role="bold">angles(3)</emphasis> around third axis.
// <para>axes =[1],angles =[a],M=CL_rot_angles2matrix(axes,angles) -> M' is a rotation of 'a' on axe 1</para>
// <para>axes =[2],angles =[a],M=CL_rot_angles2matrix(axes,angles) -> M' is a rotation of 'a' on axe 2</para>
// <para>axes =[3],angles =[a],M=CL_rot_angles2matrix(axes,angles) -> M' is a rotation of 'a' on axe 3</para>
// <para>axes =[1,2],angles =[a;b],M=CL_rot_angles2matrix(axes,angles) -> M=CL_rot_angles2matrix(2,b)*CL_rot_angles2matrix(1,a)</para>
// <para>axes =[1,2,3],angles =[a;b;c],M=CL_rot_angles2matrix(axes,angles) -> M=CL_rot_angles2matrix(3,c)*CL_rot_angles2matrix(2,b)*CL_rot_angles2matrix(1,a)</para>
// <para>axes =[3,2,3],angles =[a;b;c],M=CL_rot_angles2matrix(axes,angles) -> M=CL_rot_angles2matrix(3,c)*CL_rot_angles2matrix(2,b)*CL_rot_angles2matrix(3,a)</para>
// <para>For example: if R is a reference frame and S a satellite frame issued from successive rotations angles =[a,b,c] then
// the coordinates of a unique vector v (with coordinates vR in R and vS in S) are linked by:</para>
// </listitem>
// <listitem><para/><inlinemediaobject><imageobject><imagedata fileref="dmrot.gif"/></imageobject></inlinemediaobject></listitem>
// <listitem>Last update : 20/11/2009</listitem>
// </itemizedlist>
//
// Authors
// CNES - DCT/SB
//
// See also
// CL_rot_quat2matrix
// CL_rot_matrix2quat
//
// Examples
// M = CL_rot_angles2matrix([1],[30*%pi/180])
// M = CL_rot_angles2matrix([1,2],[%pi/3;%pi/2])
// M = CL_rot_angles2matrix([1,2,3],[%pi/6;%pi/2;%pi/4])
//
//  //R is a reference frame; S is a frame obtained from R with a rotation of 10 deg on the Z axis
// ang = CL_deg2rad(10)
// M = CL_rot_angles2matrix([3],ang)
// vR = [1;0;0]  //X axis of the R reference frame expressed on the R reference frame
// vS = M*vR     //vS is the X axis of R expressed on the S frame
// wR = M'*vR    //wR is vector vR rotated 10 deg on the Z axis, all expressed on the R reference frame
//

// Declarations:


// Code:

sz = size(axes);
szs = sz(2);

select szs

  case 1
//    angles=angles(:)';
  nn=size(angles,1);
    n=size(angles,2);
    if nn~=1 error("check dim of input arguments"); end,
    select axes(1)
    //attention: la fonction Sscilab 'hypermat' accepte les elements colonne
    //           par colonne, M11;M21;M31;M12;M22...
      case 1
        M = hypermat([3 3 n] , [ones(angles); zeros(angles); zeros(angles); zeros(angles); ...
     cos(angles); -sin(angles); zeros(angles); sin(angles); cos(angles)]);
      case 2
        M = hypermat([3 3 n] , [cos(angles); zeros(angles); sin(angles); zeros(angles); ...
     ones(angles); zeros(angles); -sin(angles); zeros(angles); cos(angles)]);
      case 3
        M = hypermat([3 3 n] , [cos(angles); -sin(angles); zeros(angles); sin(angles); ...
         cos(angles); zeros(angles); zeros(angles); zeros(angles); ones(angles)]);
      else
         error("check value of parameter ''axes'' = "+string(axes));
    end

  case 2
     n=size(angles,1);
     coherence = szs==2 & n==2
     if ~coherence error("check dim of input arguments"); end,
     I = axes(1);
     J = axes(2);
     M=CL_rot_angles2matrix(J,angles(2,:))*CL_rot_angles2matrix(I,angles(1,:));

   case 3
     n=size(angles,1);
     coherence = szs==3 & n==3
     if ~coherence error("check dim of input arguments"); end,
     I = axes(1);
     J = axes(2);
     K = axes(3);
     M=CL_rot_angles2matrix(K,angles(3,:))*CL_rot_angles2matrix(J,angles(2,:))*CL_rot_angles2matrix(I,angles(1,:));

   else
     error("check number of input arguments");

end

if n==1 then M=M(:,:,1); end,

endfunction
