//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL  license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'http://www.cecill.info'.

function q = CL_rot_defQuat(r,i1,i2,i3)
// Quaternion definition from its real or imaginary components
//
// Calling Sequence
// [q] = CL_rot_defQuat(r,i1,i2,i3)
// [q] = CL_rot_defQuat(r,i)
// [q] = CL_rot_defQuat(ri)
// [q] = CL_rot_defQuat(ri,str)
//
// Parameters
// r: real part (1xN)
// i1,i2,i3: imaginary part (1xN)
// i: imaginary part (3xN)
// ri: real and imaginary part (4xN)
// str: string (use with ri). 'p' -> real part first line (default), 'd' -> real part last line.
//
// Description
// <itemizedlist><listitem>
// Returns a quaternion given its real or imaginary components. Output quaternions are NOT normed.
// All these operations define the same quaternion:
// <para>q=CL_rot_defQuat(1,[2 3 4]')</para>
// <para>q=CL_rot_defQuat([1:4]')</para>
// <para>q=CL_rot_defQuat(1,2,3,4)</para>
// <para>q=CL_rot_defQuat([1;2;3;4],"p") //real part first column (default)</para>
// <para>q=CL_rot_defQuat([2;3;4;1],"d") //real part last column</para>
// <para><emphasis role="bold">Defined (overloaded) operations for quaternions:</emphasis></para>
// <para>1) scalar / quaternion:</para>
// <para>   alpha scalar: q*alpha (same as alpha*q), q/alpha, alpha/q (same as alpha*conj(q) if q is normed)</para>
// <para>   alpha vector:  q.*alpha (same as alpha.*q), q./alpha, alpha./q (same as alpha.*conj(q) if q is normed).  (see examples)</para>
// <para>2) logical:</para>
// <para>   q1==q2</para>
// <para>3) quaternion / quaternion:</para>
// <para>   q1*q2 , q1/q2, q1+q2 , q1-q2 </para>
// <para>4) others:</para>
// <para>   size(q), norm(q), [q1 q2 ...], conj(q)==~q, real(q), imag(q);</para></listitem>
// <listitem>To access real and imaginary parts of a quaternion, you can use q.r and q.i or real(q) and imag(q).</listitem>
// <listitem>Applications with normed quaternions: An attitude quaternion is defined by the rotation (alpha around u) of the satellite going from reference
// (J2000 for instance) to its actual attitude, where alpha is a simple rotation angle and cos(beta_x), cos(beta_y)
// and cos(beta_z) are the "direction cosines" locating the axis of rotation u (Euler's Theorem).
// <para/><inlinemediaobject><imageobject><imagedata fileref="ualph2q.gif"/></imageobject></inlinemediaobject></listitem>
// <listitem>Last update : 27/6/2008</listitem>
// </itemizedlist>
//
// Authors
// CNES - DCT/SB
//
// See also
// CL_rot_matrix2quat
// CL_rot_quat2matrix
// CL_rot_rotVect
//
// Examples
// qq=CL_rot_defQuat(rand(4,1000)); // 1000 random quaternions
// qq=qq./norm(qq);          // normalisation
// size(qq);
//
//  //Differences between q*alpha, q.*alpha, q/alpha, q./alpha, alpha/q, alpha./q
// q=CL_rot_defQuat([1 2 3 4;5 6 7 8]')  //two quaternions
// q=q./norm(q);
// alpha1 = 2;
// alpha2 = [2 3];
// q1 = q*alpha1   //both quaternions are multiplied by 2
// q2 = q.*alpha2  //first quaternion is multiplied by 2 while second is multiplied by 3
// q3 = q/alpha1   //both quaternions are divided by 2
// q4 = q./alpha2  //first quaternion is divided by 2 while second is divided by 3
// q5 = alpha1/q   //both inverse of q multiplied by 2; alpha1*(inverse of q); inverse of q is 'conj(q)/(q*conj(q))' (equal to conj(q) if q normed);
// q6 = alpha2./q  //first inverse of q is multiplied by 2 while second is multiplied by 3; alpha.*(inverse of q)
//
// See 'CelestLab > Demos > ORBITAL_EVENTS > events' for more examples
//

// Declarations:


// Code:

[lhs, rhs]=argn(0);
MSG=["Check dimension of input arguments";
     "Check type of input arguments";
     "Check value of input string"];

li=[];
co=[];

select rhs
  case 4

    [li(1) co(1)]=size(r);
    [li(2) co(2)]=size(i1);
    [li(3) co(3)]=size(i2);
    [li(4) co(4)]=size(i3);
    if (sum(li)~=4 | prod(li)~=1) error(MSG(1)); end,
    if length(unique(co))~=1 error(MSG(1)); end, //checks all inputs same #columns

    st=type(r)+type(i1)+type(i2)+type(i3);
    pt=type(r)*type(i1)*type(i2)*type(i3);
    if (st~=4 | pt~=1) error(MSG(2)); end,

    q=mlist(['CLquat','r','i'],r,[i1;i2;i3]);

  case 2

    if (type(r)==1 & type(i1)==1) // 2 constants

      [li(1) co(1)]=size(r);
      [li(2) co(2)]=size(i1);
      if (li(1)~=1 | li(2)~=3) error(MSG(1)); end,
      if length(unique(co))~=1 error(MSG(1)); end,
      q=mlist(['CLquat','r','i'],r,i1);

    elseif (type(r)==1 & type(i1)==10) // 1 constante et une chaine

      [li(1) co(1)]=size(r);
      [li(2) co(2)]=size(i1);
      if (li(1)~=4 | li(2)~=1 | co(2)~=1) error(MSG(1)); end,
      if i1=="d" then
        q=mlist(['CLquat','r','i'],r(4,:),r(1:3,:));
      elseif i1=="p"
        q=mlist(['CLquat','r','i'],r(1,:),r(2:4,:));
      else
        error(MSG(3));
      end

    else

      error(MSG(2));

    end

  case 1

    [li(1) co(1)]=size(r);
    if (li(1)~=4) error(MSG(1)); end,
    q=mlist(['CLquat','r','i'],r(1,:),r(2:4,:));

end

endfunction
