//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL  license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'http://www.cecill.info'.

function [q] = CL_rot_eul2quat(phi,theta,psi)
// Euler '123' angles to quaternion
//
// Calling Sequence
// [q] = CL_rot_eul2quat(phi,theta,psi)
//
// Description
// <itemizedlist><listitem>
// Computes the quaternion <emphasis role="bold">q</emphasis> given Euler angles (rotation  order (123))<emphasis role="bold">phi</emphasis>, <emphasis role="bold">theta</emphasis> and <emphasis role="bold">psi</emphasis>.
// </listitem>
// <listitem><para/><inlinemediaobject><imageobject><imagedata fileref="eul2q.gif"/></imageobject></inlinemediaobject></listitem>
// <listitem>Last update : 3/6/2008</listitem>
// </itemizedlist>
//
// Parameters
// phi: rotation angle about the X axis [rad] (1xN)
// theta: rotation angle about the Y axis [rad] (1xN)
// psi: rotation angle about the Z axis [rad] (1xN)
// q: quaternion (dim N)
//
// Authors
// CNES - DCT/SB
//
// Bibliography
// Mecanique spatiale - CNES Cepadues 1995, Tome I, 7.2.2.3 Les quaternions
//
// See also
// CL_rot_quat2eul
// CL_rot_defQuat
//
// Examples
// // Example 1
//   phi = %pi/8
//   theta = %pi/3
//   psi = %pi/2
//   q = CL_rot_eul2quat(phi,theta,psi);
//   [phi2,theta2,psi2] = CL_rot_quat2eul(q);
//   R = [phi;theta;psi]; //reference values
//   S = [phi2;theta2;psi2];  //solution

// Declarations:


// Code:

cphi2 = cos(0.5.*phi);
sphi2 = sin(0.5.*phi);
ctheta2 = cos(0.5.*theta);
stheta2 = sin(0.5.*theta);
cpsi2 = cos(0.5.*psi);
spsi2 = sin(0.5.*psi);

q0 = cpsi2.*ctheta2.*cphi2 + spsi2.*stheta2.*sphi2;
q1 = cpsi2.*ctheta2.*sphi2 - spsi2.*stheta2.*cphi2;
q2 = cpsi2.*stheta2.*cphi2 + spsi2.*ctheta2.*sphi2;
q3 = spsi2.*ctheta2.*cphi2 - cpsi2.*stheta2.*sphi2;

q = CL_rot_defQuat([q0;q1;q2;q3]);

endfunction




