//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL  license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'http://www.cecill.info'.

function [phi,theta,psi] = CL_rot_quat2eul(q)
// Quaternion to Euler '123' angles
//
// Calling Sequence
// [phi,theta,psi] = CL_rot_quat2eul(q)
//
// Description
// <itemizedlist><listitem>
// Computes Euler angles (rotation order (123)) from the given quaternion <emphasis role="bold">q</emphasis>.
// <para>NOTE: One must be aware of singularities in the Euler angle
// parametrization when the pitch (theta) approaches +-%pi or +-%pi/2. These cases must be
// handled specifically: results may not be accurate!</para>
// </listitem>
// <listitem><para/><inlinemediaobject><imageobject><imagedata fileref="q2eul.gif"/></imageobject></inlinemediaobject></listitem>
// <listitem>Last update : 3/6/2008</listitem>
// </itemizedlist>
//
// Parameters
// q: quaternion (dim N)
// phi: X axis angle rotation [rad] (1xN)
// theta: Y axis angle rotation [rad] (1xN)
// psi: Z axis angle rotation [rad] (1xN)
//
// Authors
// CNES - DCT/SB
//
// Bibliography
// Mecanique spatiale - CNES Cepadues 1995, Tome I, 7.2.2.3 Les quaternions
//
// See also
// CL_rot_eul2quat
// CL_rot_defQuat
//
// Examples
// M = [cos(%pi/4),sin(%pi/4),0;-sin(%pi/4),cos(%pi/4),0;0,0,1];  // rotation of angle %pi/4 around Z
// q = CL_rot_matrix2quat(M); // quaternion of this rotation
// [phi,theta,psi] = CL_rot_quat2eul(q) // psi = %pi/4

//Euler angles from quaternion
//phi, theta, psi
//there are limitations


// Declarations:


// Code:

q0 = q.r
q1 = q.i(1,:)
q2 = q.i(2,:)
q3 = q.i(3,:)

phi = atan( ( 2.*(q0.*q1+q2.*q3) )./( 1-2.*(q1.^2+q2.^2) ) );
theta = asin( 2.*(q0.*q2-q3.*q1) );
psi = atan( ( 2.*(q0.*q3+q1.*q2) )./( 1-2.*(q2.^2+q3.^2) ) );

if or( (abs(theta)>%pi-0.0001) | ((abs(theta)>%pi/2-0.0001)&(abs(theta)<%pi/2+0.0001)) )
  warning('theta near pi or pi/2: results may not be accurate!');
end



endfunction
