//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL  license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'http://www.cecill.info'.

function M = CL_rot_quat2matrix(q)
// Quaternion to matrix
//
// Calling Sequence
// M=CL_rot_quat2matrix(q)
//
// Description
// <itemizedlist><listitem>
// Computes the frame transfer matrix <emphasis role="bold">M</emphasis> from quaternion <emphasis role="bold">q</emphasis>.
// Computed matrix M is the transposed of the rotation operator matrix corresponding to q. That is matrix A
// from Wertz called 'transformation matrix' (so rotation matrix = A'=M', see reference 2 below).
// <para>If q contains N quaternions: size(q)=N -> size(M)=[3 3 N]</para>
// <para>For example: if R is a reference frame and S a satellite frame with attitude quaternion <emphasis role="bold">q</emphasis>, then
// the coordinates of a unique vector v (with coordinates vR in R and vS in S) are linked by:</para>
// </listitem>
// <listitem><para/><inlinemediaobject><imageobject><imagedata fileref="dmrot.gif"/></imageobject></inlinemediaobject></listitem>
// <listitem>Last update : 27/6/2008</listitem>
// </itemizedlist>
//
// Parameters
// q: quaternion (dim N)
// M: frame transfer hypermatrix (3x3xN)
//
// Bibliography
// 1 Mecanique Spatiale - CNES Cepadues 1995, Tome I, 7.2.2.3 Les quaternions, eq.15
// 2 James R. Wertz, Spacecraft attitude determination and control (volume 73 of Astrophyisics and Space Science Library), D. Reidel Publishing Company, 1980, appendix D-E
//
// Authors
// CNES - DCT/SB
//
// See also
// CL_rot_defQuat
// CL_rot_matrix2quat
// CL_rot_angles2matrix
//
// Examples
// ang = CL_deg2rad(10)
// M1 = CL_rot_angles2matrix(3,ang)
// q1 = CL_rot_matrix2quat(M1)
// q2 = CL_rot_eul2quat(0,0,ang)  //same as q1
// M2 = CL_rot_quat2matrix(q2)    //same as M1
//

// Declarations:


// Code:

[lhs,rhs]=argn();

if typeof(q)~="CLquat"
  M=[];
  error("error - quaternion needed as input (see CL_rot_defQuat)");
else
  q4=q.r;
  q1=q.i(1,:);
  q2=q.i(2,:);
  q3=q.i(3,:);
end

n=size(q);

M=hypermat([3 3 n],[ q1.^2 - q2.^2 - q3.^2 + q4.^2; 2*(q1.*q2 - q3.*q4); 2*(q1.*q3 + q2.*q4); ...
                     2*(q1.*q2 + q3.*q4); (-q1.^2 + q2.^2 - q3.^2 + q4.^2) ;  2*(q2.*q3 - q1.*q4) ; ...
                     2*(q1.*q3 - q2.*q4); 2*(q2.*q3 + q1.*q4) ; (-q1.^2 - q2.^2 + q3.^2 + q4.^2)]);

if n==1 M=M(:,:,1); end; //avoid hypermat if only one quaternion

endfunction
