//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL  license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'http://www.cecill.info'.

function q = CL_rot_quatSlerp(q1,q2,t)
// Spherical linear interpolation between 2 quaternions
//
// Calling Sequence
// q = CL_rot_quatSlerp(q1,q2,t)
//
// Description
// <itemizedlist><listitem>
// Computes spherical linear interpolation (slerp) between quaternions <emphasis role="bold">q1</emphasis> and
// <emphasis role="bold">q2</emphasis> (shortest way), given an interpolation parameter between 0 and
// 1 (<emphasis role="bold">t</emphasis>). <emphasis role="bold">Input quaternions are normed before any calculations</emphasis>.
// <para>CL_rot_quatSlerp(q1,q2,0)=q1 and CL_rot_quatSlerp(q1,q2,1)=q2</para>
// <para>NOTE: q1 and q2 must be same size. In addition, to use this function in a vectorized way
// there are two options: </para>
// <para>1:  N=P</para>
// <para>2:  N~=P with N=1 or P=1</para>
// <para>NOTE: t&lt;0 gives same result as t=0; t&gt;1 gives same result as t=1.</para>
// </listitem>
// <listitem>Last update : 27/6/2008</listitem>
// </itemizedlist>
//
// Parameters
// q1: start quaternion (type quat, see CL_rot_defQuat) (size N)
// q2: end quaternion (type quat)  (size N)
// t: interpolation parameter [0..1] (1xP)
//
// Authors
// CNES - DCT/SB
//
// See also
// CL_rot_defQuat
// CL_rot_matrix2quat
//
// Examples
// q1 = CL_rot_defQuat([0.1 0.2 0.3 0.4]')
// q2 = CL_rot_defQuat([0.2 0.3 0.4 0.5]')
// q1_normed = q1./norm(q1)
// q2_normed = q2./norm(q2)
// q = CL_rot_quatSlerp(q1,q2,0)
// qq = CL_rot_quatSlerp(q1,q2,1)
// qqq = CL_rot_quatSlerp(q1,q2,0.5)
//

// Declarations:


// Code:

N = size(q1);
N2 = size(q2);
P = size(t,2);

if N~=N2 then error('q1 and q2 must be same size'); end
if N~=1 & P~=1 & N~=P
  error('q1 and t must be same size or one of them size 1')
end

q1 = q1./norm(q1);
q2 = q2./norm(q2);

q = CL_rot_defQuat(zeros(4,max(N,P)));

q = conj(q1)*q2;
rotAmp = 2.0*acos(q.r);
// Shortest way:
i = find(rotAmp > %pi);
if i~=[]
  rotAmp(i) = abs(rotAmp(i)-2*%pi);
  q(i) = CL_rot_defQuat(-q(i).r,-q(i).i);
end

sts2 = CL_colNorm(q.i);
j = find(sts2 > 1.d-7); // au dela de 10-5 deg environ.
j2 = find(~(sts2 > 1.d-7));
if j~=[]
  Vect = q(j).i ./ (CL_colNorm(q(j).i)*ones(3,1));
  teta = rotAmp(j) .* t(j);
  q(j) = q1(j)*(CL_rot_defQuat(cos(teta/2),sin(teta/2)*ones(3,1).*Vect));
end
if j2~=[]
  q(j2) = q1(j2);
end

endfunction
