//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL  license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'http://www.cecill.info'.

function w = CL_rot_rotVect(q,v)
// Rotation(defined by a quaternion) of a vector
//
// Calling Sequence
// w = CL_rot_rotVect(q,v)
//
// Description
// <itemizedlist><listitem>
// Calculates <emphasis role="bold">w</emphasis>: frame transfer of vector <emphasis role="bold">v</emphasis> defined by quaternion <emphasis role="bold">q</emphasis> (see examples)
// <para>If <emphasis role="bold">q</emphasis> is an attitude quaternion, then <emphasis role="bold">w</emphasis> gives the coordinates (in satellite frame)
// of vector <emphasis role="bold">v</emphasis> (v expressed on reference frame).</para>
// </listitem>
// <listitem>Last update : 27/6/2008</listitem>
// </itemizedlist>
//
// Parameters
// q: quaternion (dim N)
// v: vector (3xN)
// w: frame transformed vector (3xN)
//
// Authors
// CNES - DCT/SB
//
// See also
// CL_rot_defQuat
// CL_rot_matrix2quat
// CL_rot_quat2matrix
//
// Examples
// q=CL_rot_defQuat(rand(4,2));
// q=q./norm(q);
// M1=CL_rot_quat2matrix(q(1));
// v=[1;2;3];
// w1=M1*v
// w1=CL_rot_rotVect(q(1),v)
// M2=CL_rot_quat2matrix(q(2));
// w2=[w1 M2*v]
// qq=CL_rot_matrix2quat(M2*M1);
// qqq=q(1)*q(2);
// w3=CL_rot_rotVect(qq,v)
// w3=CL_rot_rotVect(qqq,v)
//
//  //R is a reference frame. S is a frame obtained with a rotation of 10 deg on the Z axis of R
// ang = CL_deg2rad(10)
// q = CL_rot_eul2quat(0,0,ang)
// w = [1;0;0]               //X axis of R expressed on R reference frame
// v = [cos(ang);sin(ang);0] //vector w rotated 10 deg on Z axis (X axis of S frame expressed on R frame)
// w = CL_rot_rotVect(q,v)          //w = vector v (which was expressed on R) expressed on S
//

// Declarations:


// Code:

[lhs rhs]=argn(0);
if rhs~=2 error("check number of input arguments"); end,
if typeof(q)~="CLquat" error("q is not of type CLquat"); end,
if typeof(v)~="constant" error("v is not of type constant"); end,
[li co]=size(v);
if li~=3 error("v dimension"); end,

if (co==1 & size(q)>=1) then

  //rotation de v (unique) sur le quaternion(s) q
  w=imag(conj(q)*CL_rot_defQuat(zeros(q.r),v*ones(1,size(q)))*q);

elseif co==size(q) then

    //rotation de v(:,I) sur le quaternion q(I)
  w=imag(conj(q)*CL_rot_defQuat(zeros(q.r),v)*q);

elseif co>1 & size(q)==1 then

  Q=CL_rot_defQuat(q.r*ones(1,co),q.i*ones(1,co));
  V=CL_rot_defQuat(zeros(1,co),v);
  w=imag( conj(Q)*V*Q );

else
  error("check dimension of input arguments");
end

endfunction
