//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL  license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'http://www.cecill.info'.

function t3 = CL_ev_dicholin(t1,t2,v1,v2,precision,torb,meanlyd,ev_type,param,attitude)
// Dichotomie to find the time of 0 value for GF function
//
// Calling Sequence
//
// Description
// <itemizedlist><listitem>
// For all events corresponding to a NULL value of the "Geometric Function" GF
// iterative computation to find the time with accuracy of "precision".
// </listitem>
// </itemizedlist>
//
// Parameters
// t1, t2      : (Cnes Jul Day) time of first and second points
// v1, v2      : (rad) Value of GF at t1 and t2
// evtyp       : type of event:
//             1 = ascending node CL_crossing
//             2 = solar time "evtpar"(rad [-pi,pi]) (0=12h) CL_crossing
//             3 = Sun eclipse by Earth with "evtpar"(rad) margin
//             4 = Sun eclipse by Moon with "evtpar"(rad) margin
//             5 = GRB Limb with "evtpar"(rad) margin CL_crossing
//                 evtpar(2:3) = ra dec of GRB (rad)
//             6 = GRB Moon with "evtpar(1)"(rad) margin CL_crossing
//                 evtpar(2:3) = ra dec of GRB (rad)
//             7 = SAA with polygon defined in evtpar(radians)
//             10 = station nr "evtpar"(int) geometric visibility
//             20 or 30 = station nr "evtpar"(int) RF (Antenna Ant) visibility
//             50 = sensor nr "evtpar"(int) visibility
// precision   : (sec) evts time accuracy
// torb        : (Cnes Jul Day) time of orbit mean parameters
// meanlyd     : (m,rad) mean lyddan parameters
// evtpar   [OPTIONAL]: [] parameter vector linked to evtyp.
// att_veis [OPTIONAL]: transfer matrix Sat->Veis
// Ant      [OPTIONAL]: Antenna definition: [x,y,z,semi]
//                x,y,z = antenna axis in satellite frame;  semi (rad) = semi-angle
// tCL_cross       : (Cnes Jul Day) time of event GF CL_crosses 0 with precision
//
// GLOBAL  stations    : (typSta) stations parameters
// GLOBAL  sensors     : (typSta) stations parameters
//
// Authors
// CNES - DCT/SB
//

// Declarations:


// Code:

if ~exists('param','local') then param=[]; end
if ~exists('attitude','local') then attitude=[]; end

maxiter = 10;
epsilon = 1.e-10;
// Parameters control
if t1>t2 then
  warning('t1>t2');
elseif v1*v2>0 then
  warning('v1*v2>0');
end
//
if (abs(v2-v1)>epsilon) & v1*v2<0
  t3 = (v2*t1-v1*t2)/(v2-v1)  //t3 obtained suposing v3=0
else
  t3 = (t1+t2)/2; //if v1 and v1 are both positive or both negative, we chose t3 in the middle
end
ik=0; //number of iterations
while (min(abs(t3-t1),abs(t2-t3)) > precision/86400) & (ik <= maxiter)  //while time step higher than required precission
  ik=ik+1;
  [meanext,oscext] = CL_ex_lyddane(torb,meanlyd,t3); //extrapolate orbit until t3
  pos = CL_oe_kep2car(oscext);
  v3 = CL_ev_geomF([t3;pos],ev_type,param,attitude)
  v3 = v3(:,:,1); //v3 will always be only one number inside dicholin
  if v3*v1>0
    t1=t3; v1=v3;
  else
    t2=t3; v2=v3;
  end
  if (abs(v2-v1)>epsilon) & (v1*v2<0) then
    t3 = (v2*t1-v1*t2)/(v2-v1)
  else
    t3 = (t1+t2)/2;
  end
end //end while

if ik>maxiter then warning('geom. visi. convergence problem'); end

endfunction
