//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL  license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'http://www.cecill.info'.

function events = CL_ev_events(cjd_ini,mean_G50,simulation_period,param,step,prec,compev,att_plan,propagation_type,er,mu,zonals,obla)
// Events computation
//
// Calling Sequence
// events = CL_ev_events(cjd_ini,mean_G50,simulation_period,param[,step[,prec[,compev[,att_plan[,propagation_type[,er[,mu[,zonals[,obla]]]]]]]]])
//
// Description
// <itemizedlist><listitem>
// This function computes different types of events in the period of time <emphasis role="bold">simulation_period</emphasis>.
// <para>Satellite's positions are extrapolated with Lyddane,Eckstein-Hechler or a simple secular J2 propagator (depending on <emphasis role="bold">propagation_type</emphasis>).</para>
// <para>Time step (<emphasis role="bold">step</emphasis>) and prec of events computation can be optionnaly given.</para>
// <para></para>
// <para><emphasis role="bold">compev</emphasis> is a boolean vector of dimension 10 describing with events are to be computed.</para>
// <para>Possible events are:</para>
// <para>  - compev(1): anc - ascending node crossing</para>
// <para>  - compev(2): stc - one or several solar time crossing</para>
// <para>  - compev(3): see - sun eclipse (penumbra) by earth</para>
// <para>  - compev(4): sem - sun eclipse (penumbra) by moon</para>
// <para>  - compev(5): gre - inertial direction (star, GRB, ...) above earth limb (with threshold)</para>
// <para>  - compev(6): grm - moon aproaching threshold cone around inertial direction (star, GRB, ...)</para>
// <para>  - compev(7): saa - area(for example, area of south atlantic anomaly) crossing</para>
// <para>  - compev(8): sgv - station-satellite geometric visibility</para>
// <para>  - compev(9): sav - station-antenna rf visibility</para>
// <para>  - compev(10): ssv - sensor visibility</para>
// <para></para>
// <para>Some events need parameters to be computed. For instance, to compute geometric visibility, location and properties of stations must be given.</para>
// <para><emphasis role="bold">param</emphasis> is an mlist containing the following :</para>
// <para>  - param.stc: solar time hours for wich crossing is computed (1xN)</para>
// <para>  - param.see: margin for sun eclipse by earth [rad]</para>
// <para>  - param.sem: margin for sun eclipse by moon [rad]</para>
// <para>  - param.sgv: contains stations (see oe_dsation)</para>
// <para>  - param.gre: threshold and inertial direction for 'gre' event computation [margin;right ascension of direction;declination of direction] RA and DEC in G50 (3x1) [rad]</para>
// <para>  - param.grm: threshold and inertial direction for 'grm' event computation [margin;right ascension of direction;declination of direction] RA and DEC in G50 (3x1) [rad]</para>
// <para>  - param.saa: polygon defining an area (for example, area of south atlantic anomaly), each point given by geocentric latitude and longitude
// [lon1 lon2 lon3 ...;lat1 lat2 lat3 ...]. The area can be convex or not. (2xN) [rad]</para>
// <para>  - param.svv: contains parameters for 'sav' and 'ssv' event computations because it contains sensors including antennas (see CL_ev_defSensor, give type 0 for antennas)</para>
// <para>Note : You only need to fill the param fields that correspond to the events you are computing</para>
// <para></para>
// <para><emphasis role="bold">att_plan</emphasis> is a tlist containing the satellite attitude planning.
// <emphasis role="bold">att_plan = tlist(['attPlan','t','quat','duration'])</emphasis>. Field <emphasis role="bold">att_plan.t</emphasis> contains a vector
// of dates; <emphasis role="bold">att_plan.quat</emphasis> contains quaternions describing new attitude (see CL_rot_defQuat) for each date in att_plan.t;
// and <emphasis role="bold">att_plan.duration</emphasis> contains a vector defining duration of each attitude maneuver. Duration of maneuvers is only used
// to compute end of maneuver event date. There is no quaternion interpolation; so attitude changes instantly at dates given.
// Dates, quaternions, and durations must be of the same size.
// Moreover, when used, att_plan has to contain at least one quaternion with
// a date precedent or equal to the bulletin date.</para>
// <para></para>
// <para><emphasis role="bold">Notes :</emphasis></para>
// <para>  - due to the number of optional events, it is recomended to call this function using signed parameters. Example:
// <emphasis role="bold">events=CL_ev_events(ephe,1,params,compev=[%t;%f;%f;%f;%f;%t;%t;%t;%f;%f],step=120,prec=prec);</emphasis></para>
// <para>  - some calculations use global constants that you can always modify if needed:</para>
// <para>       * sun eclipse by earth (compev.see) uses <emphasis role="bold">%CL_radiusSun</emphasis>(sun radius) and <emphasis role="bold">%rayatmo</emphasis> (atmosphere upper limit for penumbra);</para>
// <para>       * sun eclipse by moon (compev.sem) uses <emphasis role="bold">%CL_radiusSun</emphasis>, <emphasis role="bold">%leoLim</emphasis> (limit for sun parallax) and <emphasis role="bold">%CL_radiusMoon</emphasis> (moon radius);</para>
// <para>       * inertial direction (compev.gre) uses <emphasis role="bold">%rayatmo</emphasis>; </para>
// <para>       * sensor visibility (compev.ssv) uses <emphasis role="bold">%rayatmo</emphasis>.</para>
// <para><emphasis role="bold">Warning :</emphasis> computations for events 'gre' and 'grm' are made in CNES GAMMA50 frame. </para>
// </listitem>
// <listitem>Last update : 21/10/2009</listitem>
// </itemizedlist>
//
// Parameters
// cjd_ini: initial date in CNES julian days [cjd] (1x1)
// mean_G50: mean keplerian parameters in Gamma50 Veis frame [a e i w omega M]' [m,rad] (6x1)
// simulation_period: time in days where to search events from start date in bulletin [days]
// param: necessary parameters for some events computations
// step: (optional) step of propagation [seconds] (default is 60 seconds)
// prec: (optional) prec in search of start and end dates of events [seconds] (default is 1 second)
// compev: (optional) events to compute [booleans] (10x1) (default is all true)
// att_plan: (optional but necessary for events involving attitude ; for instance, RF visibility) tlist containing attitude (dates, quaternions in G50 and duration) [dates in cnes julian days + ut/24 and duration in seconds] (default is no attitude)
// propagation_type: (optional) 'j2' for secular J2, 'lyd' for lyddane or 'eh' for Eckstein Hechler (default is lyddane) (1x1)
// er : (optional) Earth semi-major axis (default is earth equatorial radius %CL_eqRad) [m] (1x1)
// mu: (optional) geocentric gravitational constant [m^3/s^2] (default value is %CL_mu)
// zonals: (optional) vector of zonals coefficients J1 to Jn (troncated to J5) to be used (default is %CL_j1jn(1:5)) (1 x N)
// obla : (optional) Earth oblateness, setting it to 0 will result in a faster computation (default is earth oblateness %CL_obla) (1x1)
// events: computed events (position in GAMMA50)(see CL_ev_defEvent)
//
// Authors
// CNES - DCT/SB
//
// See also
// CL_gm_visibility
// CL_ev_defSensor
// CL_ev_defEvent
// CL_ev_defStation
// CL_rot_defQuat
//
// Examples
// // Find geometric visibility from Kourou for 10 days
// cjd_ini = 21900;
// mean_G50 = [ 6978000 1.e-6 CL_deg2rad(30) 0 0 0]'; //G50
// kourou = CL_ev_defStation('kourou','kru',CL_deg2rad(10),CL_deg2rad(307.2),CL_deg2rad(5.25),112);
// param = mlist(['param','sgv'],kourou);
// compev = zeros(1,10); //all events set to 0 (%f)
// compev(8) = %t; //set sgv event to true
// geom_visi = CL_ev_events(cjd_ini,mean_G50,10,param,compev=compev); //compute visibility
//  //ups! I forgot Maldini station!
// maldini = CL_ev_defStation('maldini','mal',CL_deg2rad(10),CL_deg2rad(40.1945),CL_deg2rad(-2.99),0);
// param.sgv = [kourou maldini];
// geom_visi = CL_ev_events(cjd_ini,mean_G50,10,param,compev=compev); //compute visibility
//  //now I only want visibility from maldini station
// maldini_visibility = geom_visi(find(abs(geom_visi.code)==100.2));
//  //now i want only events in first 5 days
// fiveDays_visi = geom_visi(find(geom_visi.date&lt;=bulletin(1)+5));
//
// // Find eclipses of sun or moon in 2 months (60 days)
// cjd_ini = 21900;
// mean_G50 = [6978000 1.e-6 CL_deg2rad(30) 0 0 0]'; //G50
// compev = zeros(1,10); //all events set to 0 (%f)
// compev([3 4]) = %t; //set eclipse events to true
// param = mlist(['param','see','sem'],0,0);
// eclipses = CL_ev_events(cjd_ini,mean_G50,60,param,compev=compev,step=5*60); //compute eclipse events (step of 5 minutes instead of 60 seconds)
//
// REMARK: for a complete example computing all events, see 'CelestLab &gt; Demos &gt; ORBITAL_EVENTS &gt; events'
//

// Declarations:
global %CL_eqRad %CL_j1jn %CL_mu %CL_obla;

// Code:

if ~exists('step','local') then step=60; end
if ~exists('prec','local') then prec=1; end
if ~exists('compev','local') then compev=[%t;%t;%t;%t;%t;%t;%t;%t;%t;%t]; end
if ( compev(9)|compev(10) ) & ~exists('att_plan','local')
  error('you must give at least a constant attitude for visibility events computation');
end
if ~exists('att_plan','local')
  att_plan=tlist(['manPlan','t','quat','duration'],[],[],[]);
  mans=%f;
else
  mans=%t;
  if att_plan.t(1)>cjd_ini then error('first attitude date must be precedent or equal to cjd_ini'); end
end

if ~exists('propagation_type','local') then propagation_type='lyd'; end
if ~exists('er','local') then er=%CL_eqRad; end
if ~exists('mu','local') then mu=%CL_mu; end
if ~exists('zonals','local') then zonals=%CL_j1jn(1:5); end
if ~exists('obla','local') then obla=%CL_obla; end

//Computing station geometric visibility (sgv) is needed to compute station-antenna visibility (sav)
//if sgv events are not demanded, they will be actually computed but not shown
show_sgv = compev(8); //sgv event will be shown according if they are demanded
if compev(9) then compev(8)=%t; end //if sav events are demanded, sgv events must be computed


//extrapolation to build ephemeride tpv and get attitude matrixs. All in GAMMA50
tbull = cjd_ini; //date of bulletin
dates = tbull:step/86400:tbull+simulation_period; //dates of extrapolation with step (time step) and simulation_period (time of observation)
meanlyd = mean_G50;
[tpv,attitude] = CL_ev_TPVAttitude(tbull,meanlyd,dates,att_plan,propagation_type,er,mu,zonals);  //get position, velocity and attitude matrixes for dates
tp = tpv(1:4,:);  //time and position
clear mean_G50

//variable to store events
events = CL_ev_defEvent(); //create empty event

//EVENTS DUE TO ATTITUDE MANEUVERS
event_type = 'man';
manStart_dates = att_plan.t( att_plan.t>tbull & att_plan.t<tbull+simulation_period ); //get maneuvers in period of time between tbull and tbull+simulation_period
man_n = size(manStart_dates,2); //number of maneuvers
if man_n>0  //if there are maneuvers
  manStart_dates = gsort(manStart_dates,'c','i'); //sort maneuver start dates in ascending order
  manEnd_dates = manStart_dates + att_plan.duration/86400;  //compute end dates for each maneuver
  if man_n==1 //if there is only one maneuver
    man_problem = %f; //no problem concerning one maneuver starting before end of previous one
  else  //if there is more than one maneuver
    man_problem = or(manStart_dates(2:$)-manEnd_dates(1:$-1)<0);  //check if there is one maneuver starting before end of previous one
  end //end if man_n==1
  if ~man_problem //if there is no problem concerning start and end of maneuvers
    manEnd_dates = manEnd_dates(manEnd_dates<tbull+simulation_period);  //get only end dates that finish before tbull+simulation_period
    man_dates = [manStart_dates manEnd_dates];            //start and end dates
    [man_tpv] = CL_ev_TPVAttitude(tbull,meanlyd,man_dates,att_plan,propagation_type,er,mu,zonals);//compute satellite position for man_dates
    man_pos = man_tpv(2:4,:); //satellite position
    man_code = CL_ev_ev2codeName(event_type)*[ones(manStart_dates) -ones(manEnd_dates)];  //code for each maneuver events
    man_name = CL_ev_ev2codeName(man_code);  //get name for these events
    events = events + CL_ev_defEvent(man_dates,man_code,man_name,man_pos); //add maneuver events to all events
  else  //if there is some maneuvers starting before end of previous ones
    error('one maneuver starts before end of previous one');
  end //end if  ~man_problem
end //end if man_n>0
clear manStart_dates man_n manEnd_dates man_problem man_tpv man_code man_name man_dates man_pos


//ASCENDING NODE CROSSING 'anc'
if compev(1)
  event_type = 'anc';
  anc_events = CL_ev_geomF(tp,event_type,er=er,obla=obla);  //compute function of events (change of sign if event)
  ii = (anc_events(1:$-1)<0) & (anc_events(2:$)>0); //positive CL_crossing only (search of 0's)
  [ir ic] = find(ii); //fins indexs where function changes sign (ic index of date t1 for dichotomie)
  n = size(ic,2);//number of events
  anc_code = CL_ev_ev2codeName(event_type)*ones(1,n); //code for this type of events
  ic2 = ic+1; //index to get date t2 for dichotomie
  anc_dates = [];  //variable to store date of events
  for i=1:n //search of dates corresponding to 0's of event function
    t1 = dates(ic(i));  //date just before change of sign of event function
    t2 = dates(ic2(i)); //date just after change of sign of event function
    anc_dates(1,i) = CL_ev_dicholin(t1,t2,anc_events(ir(i),ic(i)),anc_events(ir(i),ic2(i)),prec,tbull,meanlyd,..
                             event_type); //date of 0 in event function (with prec given)
end //end for i
if ( anc_dates ~= [] ) then
  anc_tpv = CL_ev_TPVAttitude(tbull,meanlyd,anc_dates,att_plan,propagation_type,er,mu,zonals);  //satellite positions in event dates
  anc_pos = anc_tpv(2:4,:); //satellite position
  anc_name = CL_ev_ev2codeName(anc_code);  //name of events
  events = events + CL_ev_defEvent(anc_dates,anc_code,anc_name,anc_pos); //add ascending node CL_crossing events to all events
clear  anc_tpv anc_name anc_pos
end
clear anc_events anc_code anc_dates
end


//SOLAR LOCAL TIME 'stc'
if compev(2)
  event_type = 'stc';
  hours = param.stc;  //get hours
  stc_events = CL_ev_geomF(tp,event_type,hours,er=er,obla=obla);  //compute event function (change of sign if event)
  ii = stc_events(:,1:$-1,:)<0 & stc_events(:,2:$,:)>0; //positive CL_crossing (search of 0's)
  [ir ic ia] = find(ii);  //find indexs where function changes sign (ic to get date t1 for dichotomie)
  n = size(ic,2); //number of events
  stc_code = CL_ev_ev2codeName(event_type)*ones(1,n); //code for this type of event
  ic2 = ic+1; //index to get date t2 for dichotomie
  stc_dates = []; //variable to store date of events
  for i=1:n //search of dates corresponding to 0's of event function
    t1 = dates(ic(i));  //date just before change of sign of event function
    t2 = dates(ic2(i)); //date just after change of sign of event function
    stc_dates(1,i) = CL_ev_dicholin(t1,t2,stc_events(ir(i),ic(i),ia(i)),stc_events(ir(i),ic2(i),ia(i)),prec,tbull,meanlyd,...
                              event_type,hours(ia(i))); //date of 0 in event function (with prec given)
    if abs(hours(ia(i)))>1e-5 then stc_code(i)=stc_code(i)+1; end //change code if not noon CL_crossing
end  //end for i
if ( stc_dates ~= [] ) then
  stc_tpv = CL_ev_TPVAttitude(tbull,meanlyd,stc_dates,att_plan,propagation_type,er,mu,zonals); //satellite positions in event dates
  stc_pos = stc_tpv(2:4,:);  //satellite position
  stc_name = CL_ev_ev2codeName(stc_code);  //name of events
  events = events + CL_ev_defEvent(stc_dates,stc_code,stc_name,stc_pos); //add solar time events to all events
clear  stc_tpv stc_pos stc_name
end
clear hours stc_events stc_code stc_dates
end


//SUN ECLIPSE BY EARTH 'see'
if compev(3)
  event_type = 'see';
  margin = param.see;
  see_events = CL_ev_geomF(tp,event_type,margin,er=er,obla=obla); //
  ii = see_events(1:$-1).*see_events(2:$) < 0; //positive and negative CL_crossing
  [ir ic] = find(ii);
  n = size(ic,2);
  see_code = CL_ev_ev2codeName(event_type)*ones(1,n);
  ic2 = ic+1; //index to get date t2 for dichotomie
  see_dates = [];
  for i=1:n
    t1 = dates(ic(i));
    t2 = dates(ic2(i));
    see_dates(1,i) = CL_ev_dicholin(t1,t2,see_events(ir(i),ic(i)),see_events(ir(i),ic2(i)),prec,tbull,meanlyd,..
                        event_type,margin);
    if see_events(ir(i),ic2(i))>0 then see_code(i)=-see_code(i); end
end
if ( see_dates ~= [] ) then
  see_tpv = CL_ev_TPVAttitude(tbull,meanlyd,see_dates,att_plan,propagation_type,er,mu,zonals);
  see_pos = see_tpv(2:4,:);
  see_name = CL_ev_ev2codeName(see_code);
  events = events + CL_ev_defEvent(see_dates,see_code,see_name,see_pos);

end
clear see_events see_code see_dates see_name see_pos see_tpv margin
end


//SUN ECLIPSE BY MOON 'sem'
if compev(4)
  event_type = 'sem';
  margin = param.sem;
  sem_events = CL_ev_geomF(tp,event_type,margin,er=er,obla=obla); //
  ii = sem_events(1:$-1).*sem_events(2:$) < 0; //positive and negative CL_crossing
  [ir ic] = find(ii);
  n = size(ic,2);
  sem_code = CL_ev_ev2codeName(event_type)*ones(1,n);
  ic2 = ic+1; //index to get date t2 for dichotomie
  sem_dates = [];
  for i=1:n
    t1 = dates(ic(i));
    t2 = dates(ic2(i));
    sem_dates(1,i) = CL_ev_dicholin(t1,t2,sem_events(ir(i),ic(i)),sem_events(ir(i),ic2(i)),prec,tbull,meanlyd,..
                        event_type,margin);
    if sem_events(ir(i),ic2(i))>0 then sem_code(i)=-sem_code(i); end
  end

  if ( sem_dates ~= [] ) then
  sem_tpv = CL_ev_TPVAttitude(tbull,meanlyd,sem_dates,att_plan,propagation_type,er,mu,zonals);
  sem_pos = sem_tpv(2:4,:);
  sem_name = CL_ev_ev2codeName(sem_code);
  events = events + CL_ev_defEvent(sem_dates,sem_code,sem_name,sem_pos);
end
  clear sem_events sem_code sem_dates sem_name sem_pos sem_tpv margin
end


//GRB rise above earth limb 'gre'
if compev(5)
  event_type = 'gre';
  margin = param.gre;
  gre_events = CL_ev_geomF(tp,event_type,margin,er=er,obla=obla); //
  ii = gre_events(1:$-1).*gre_events(2:$) < 0; //positive and negative CL_crossing
  [ir ic] = find(ii);
  n = size(ic,2);
  gre_code = CL_ev_ev2codeName(event_type)*ones(1,n);
  if margin(1)>0.00001 then gre_code = gre_code+1; end
  ic2 = ic+1; //index to get date t2 for dichotomie
  gre_dates = [];
  for i=1:n
    t1 = dates(ic(i));
    t2 = dates(ic2(i));
    gre_dates(1,i) = CL_ev_dicholin(t1,t2,gre_events(ir(i),ic(i)),gre_events(ir(i),ic2(i)),prec,tbull,meanlyd,..
                        event_type,margin);
    if gre_events(ir(i),ic2(i))>0 then gre_code(i)=-gre_code(i); end
end
  if ( gre_dates ~= [] ) then
  gre_tpv = CL_ev_TPVAttitude(tbull,meanlyd,gre_dates,att_plan,propagation_type,er,mu,zonals);
  gre_pos = gre_tpv(2:4,:);
  gre_name = CL_ev_ev2codeName(gre_code);
  events = events + CL_ev_defEvent(gre_dates,gre_code,gre_name,gre_pos);
end
clear gre_events gre_code gre_dates gre_name gre_pos gre_tpv margin
end


//GRB MOON 'grm'
if compev(6)
  event_type = 'grm';
  margin = param.grm;
  grm_events = CL_ev_geomF(tp,event_type,margin,er=er,obla=obla); //
  ii = grm_events(1:$-1).*grm_events(2:$) < 0; //positive and negative CL_crossing
  [ir ic] = find(ii);
  n = size(ic,2);
  grm_code = CL_ev_ev2codeName(event_type)*ones(1,n);
  if margin(1)>0.00001 then grm_code = grm_code+1; end
  ic2 = ic+1; //index to get date t2 for dichotomie
  grm_dates = [];
  for i=1:n
    t1 = dates(ic(i));
    t2 = dates(ic2(i));
    grm_dates(1,i) = CL_ev_dicholin(t1,t2,grm_events(ir(i),ic(i)),grm_events(ir(i),ic2(i)),prec,tbull,meanlyd,..
                        event_type,margin);
    if grm_events(ir(i),ic2(i))>0 then grm_code(i)=-grm_code(i); end
end
 if ( grm_dates ~= [] ) then
  grm_tpv = CL_ev_TPVAttitude(tbull,meanlyd,grm_dates,att_plan,propagation_type,er,mu,zonals);
  grm_pos = grm_tpv(2:4,:);
  grm_name = CL_ev_ev2codeName(grm_code);
  events = events + CL_ev_defEvent(grm_dates,grm_code,grm_name,grm_pos);
end
clear grm_events grm_code grm_dates grm_name grm_pos grm_tpv margin
end


//SAA (south atlantic anomaly) 'saa'
if compev(7)
  event_type = 'saa'
  SAA = param.saa;
  saa_events = CL_ev_geomF(tp,event_type,SAA,er=er,obla=obla);
  ii = saa_events(1:$-1).*saa_events(2:$) < 0; //positive and negative CL_crossing
  [ir ic] = find(ii);
  n = size(ic,2);
  saa_code = CL_ev_ev2codeName(event_type)*ones(1,n);
  ic2 = ic+1; //index to get date t2 for dichotomie
  saa_dates = [];
  for i=1:n
    t1 = dates(ic(i));
    t2 = dates(ic2(i));
    saa_dates(1,i) = CL_ev_dicholin(t1,t2,saa_events(ir(i),ic(i)),saa_events(ir(i),ic2(i)),prec,tbull,meanlyd,..
                        event_type,SAA);
    if saa_events(ir(i),ic2(i))>0 then saa_code(i)=-saa_code(i); end
end
if ( saa_dates ~= [] ) then
  saa_tpv = CL_ev_TPVAttitude(tbull,meanlyd,saa_dates,att_plan,propagation_type,er,mu,zonals);
  saa_pos = saa_tpv(2:4,:);
  saa_name = CL_ev_ev2codeName(saa_code);
  events = events + CL_ev_defEvent(saa_dates,saa_code,saa_name,saa_pos);
end
clear saa_events saa_code saa_dates saa_tpv saa_name saa_pos SAA
end


//GEOMETRIC VISIBILITY 'sgv'
if compev(8)
  event_type = 'sgv';
  stations = param.sgv;  //get stations
  sgv_events = CL_ev_geomF(tp,event_type,stations,er=er,obla=obla); //compute function of events (change of sign if event)
  ii = sgv_events(:,1:$-1,:).*sgv_events(:,2:$,:) < 0; //positive and negative crossing (search of 0's)
  if size(stations)>1
    [ir ic ia] = find(ii);  //find indexs where function changes sign (ic index of date t1 for dichotomie)
  else
    [ir ic] = find(ii);  ia=ones(ir);
  end
  n = size(ic,2); //number of events
  sgv_code = CL_ev_ev2codeName(event_type)*ones(1,n);  //code for this type of events
  ic2 = ic+1; //index to get date t2 for dichotomie
  sgv_dates = []; //variable to store date of events
  for i=1:n //search of dates corresponding to 0's of event function
    t1 = dates(ic(i));  //date just before change of sign of event function
    t2 = dates(ic2(i)); //date just after change of sign of event function
    sgv_dates(1,i) = CL_ev_dicholin(t1,t2,sgv_events(ir(i),ic(i),ia(i)),sgv_events(ir(i),ic2(i),ia(i)),prec,tbull,meanlyd,...
                             event_type,stations(ia(i)));  //date of 0 in event function (with prec given)
    sgv_code(i) = sgv_code(i)+ia(i)./10; //add number of station in code as decimal (only integer part will be shown, but information on station will be accessible in decimal part of code)
    if sgv_events(ir(i),ic2(i),ia(i))>0 then sgv_code(i)=-sgv_code(i); end  //change sign of code if end of visibility
end //end for i

if ( sgv_dates ~= [] ) then
  sgv_tpv = CL_ev_TPVAttitude(tbull,meanlyd,sgv_dates,att_plan,propagation_type,er,mu,zonals);  //satellite positions in event dates
  sgv_pos = sgv_tpv(2:4,:);
  sgv_name = CL_ev_ev2codeName(sgv_code) + string(abs(10*(sgv_code-int(sgv_code))))  //add number of station in name of event
  sgv_ant_events = CL_ev_defEvent(sgv_dates,sgv_code,sgv_name,sgv_pos); //variable not cleared to be used in 'star' events
  if show_sgv then events=events+sgv_ant_events; end  //add visibility events to all events
end
clear sgv_events sgv_code sgv_dates sgv_name sgv_pos sgv_tpv stations
end


//STATION RF VISIBILITY 'sav'
if compev(9)
  event_type = 'sav';
  sensors = param.ssv;  //get sensors
  antennas = sensors(find(sensors.type==0));  //get only antennas
  stations = param.sgv; //get stations
  Ns = size(stations);  //number of station
  rf_events = CL_ev_defEvent();  //to store rf visibility events
  for i=1:Ns  //for each station
    sta_events = sgv_ant_events(find(abs(sgv_ant_events.code)==100+i/10));  //get geometric visibility events for station i
    Nsta = size(sta_events);  //number of events of station i
    sta_code = sta_events.code; //get codes of these events
    sta_dates = sta_events.date;  //get dates of these events
    if sta_code(1)<0 //if there is visi in progress at beginning of period
      sta_dates = [dates(1) sta_dates];   //add initial date
      Nsta = Nsta+1;  //one more event (initial date - start of visibility)
    end //
    if sta_code($)>0  //if there is visi in progress at end of period
      sta_dates = [sta_dates dates($)]; //add last date
      Nsta = Nsta+1;  //one more event (last date - end of visibility)
    end
    rf_dates = [];  //variable to store dates of rf visibility events for station i
    rf_code = []; //variable to store codes of rf visibility events for station i
    for j=find(sta_code>0) //for each time there is visibility
      rf_t = [];  //variable to store dates of rf visibility for station i and event j
      rf_c = [];  //same for codes
      ti = sta_dates(j);  //start date of geometric visibility
      tf = sta_dates(j+1);  //end date of geometric visibility
      tt = [ti:20*prec/86400:tf tf]; //time steps between ti and tf
      [star_tpv,star_attitude] = CL_ev_TPVAttitude(tbull,meanlyd,tt,att_plan,propagation_type,er,mu,zonals); //extrapolate position and attitude for tt dates
      star_events = CL_ev_geomF(star_tpv(1:4,:),event_type,list(stations(i),antennas),star_attitude,er=er,obla=obla); //compute function of events (change of sign if event)
      ii = star_events(:,1:$-1,:).*star_events(:,2:$,:) < 0; //positive and negative CL_crossing (search of 0's)
      [ir ic ia] = find(ii);  //find indexs where function changes sign (ic index of date t1 for dichotomie)
      n = size(ic,2); //number of events given by CL_ev_geomF (events of rf visibility for station i and time j)
      rfcode = CL_ev_ev2codeName(event_type);  //get code for rf visibility
      rf_c = rfcode*ones(1,n); //fill up events codes
      ic2 = ic+1; //index to get date t2
      for k=1:n //for each found event
        t1 = tt(ic(k)); //date just before change of sign of event function
        t2 = tt(ic2(k));  //date just after change of sign of event function
        v1 = star_events(ir(k),ic(k),ia(k));  //value of event function just before change of sign
        v2 = star_events(ir(k),ic2(k),ia(k)); //value of event function just after change of sign
        if (abs(v2-v1)>1e-10) & (v1.*v2<0)  //if difference v1 - v2 is bigger than 1e-10 and have differents signs
          rf_t(1,k) = (v2.*t1-v1.*t2)./(v2-v1); //get date by lineal interpolation for v3=0
        else  //if difference v1 - v2 is lower than 1e-10 of have same signs
          rf_t(1,k) = (t1+t2)./2; //get date by semisum of dates
        end //end if (abs(v2-v1)>1e-10) & (v1.*v2<0)
        rf_c(k) = rf_c(k) + ia(k)./100 + i./10;  //fill code with information: ia is the antenna, i is the station
        if star_events(ir(k),ic2(k),ia(k))>0 then rf_c(k)=-rf_c(k); end //if negative CL_crossing, code is end code (negative)
      end //end for k=1:n
      i_start = find(star_events(1,1,:)<0); //check if rf visibility is in progress at beginning (gets wich antenna)
      i_end = find(star_events(1,$,:)<0); //check if rf visibility is in progress at end  (gets wich antenna)
      if i_start~=[]  //if rf visibility in progress at beginning
        rf_t = [tt(1) rf_t];  //add initial date (start of rf visibility)
        rf_c = [rfcode+i_start/100+i/10 rf_c];  //add code (with antenna and station)
      end
      if i_end~=[]  //if rf visibility in progress at end
        rf_t = [rf_t tt($)];  //add end date (end of rf visibility)
        rf_c = [rf_c -(rfcode+i_end/100+i/10)]; //add code (negative with antenna and station)
      end
      rf_dates = [rf_dates rf_t]; //add events dates of time j to events of station i
      rf_code = [rf_code rf_c]; //same for codes
    end //end for j=1:2:Nsta
    rf_name = CL_ev_ev2codeName(int(rf_code)) + string(abs(100*(rf_code-int(rf_code)))); //get names for events
    rf_tpv = CL_ev_TPVAttitude(tbull,meanlyd,rf_dates,att_plan,propagation_type,er,mu,zonals);  //get position of events
    rf_pos = rf_tpv(2:4,:); //satellite position
    rf_events = rf_events + CL_ev_defEvent(rf_dates,rf_code,rf_name,rf_pos); //add events of station i to all rf events
  end //end for i
events = events + rf_events; //add rf visibility events to all events
clear gv_ant_events rf_events rf_code rf_name rf_tpv rf_pos rf_dates sensors antennas
clear rf_t star_events star_attitude star_tpv Mstar rf_t sta_code sta_dates
end //end if compute_event.sav


//SENSORS EVENT PREDICTION 'ssv'
if compev(10)
  event_type = 'ssv';
  sensors = param.ssv;  //get sensors
  ssv_events = CL_ev_geomF(tp,event_type,sensors,attitude,er=er,obla=obla); //compute function of events (change of sign if event)
  ii = ssv_events(:,1:$-1,:).*ssv_events(:,2:$,:) < 0; //positive and negative CL_crossing (search of 0's)
  [ir ic ia] = find(ii);  //find indexs where function changes sign (ic index of date t1 for dichotomie)
  n = size(ic,2); //number of events
  ssv_code = 500*ones(1,n);  //code for this type of events
  ic2 = ic+1; //index to get date t2 for dichotomie
  ssv_dates = []; //variable to store date of events
  for i=1:n //search of dates corresponding to 0's of event function
    t1 = dates(ic(i));  //date just before change of sign of event function
    t2 = dates(ic2(i)); //date just after change of sign of event function
    ssv_dates(1,i) = CL_ev_dicholin(t1,t2,ssv_events(ir(i),ic(i),ia(i)),ssv_events(ir(i),ic2(i),ia(i)),prec,tbull,meanlyd,...
                             event_type,sensors(ia(i)),attitude(:,:,ia(i)));  //date of 0 in event function (with prec given)
    ssv_code(i) = ssv_code(i)+ia(i)./10; //add number of sensor in code as decimal (only integer part will be shown, but information on station will be accessible in decimal part of code)
    if ssv_events(ir(i),ic2(i),ia(i))>0 then ssv_code(i)=-ssv_code(i); end  //change sign of code if end of visibility
end //end for i
  if ( ssv_dates ~= [] ) then
  ssv_tpv = CL_ev_TPVAttitude(tbull,meanlyd,ssv_dates,att_plan,propagation_type,er,mu,zonals);
  ssv_pos = ssv_tpv(2:4,:);
  ssv_name = CL_ev_ev2codeName(ssv_code) + string(abs(10*(ssv_code-int(ssv_code))))  //add number of station in name of event
  events = events + CL_ev_defEvent(ssv_dates,ssv_code,ssv_name,ssv_pos); //add visibility events to all events

end
clear ssv_events ssv_code ssv_dates ssv_name ssv_pos ssv_tpv sensors
end


endfunction
