//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL  license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'http://www.cecill.info'.

function [eclipseS,eclipseE,sun_orb,duration] = CL_gm_eclipseCir(cjd,sma,inc,raan,er,mu,j2)
// Analytic eclipse calculation for circular orbits
//
// Calling Sequence
// [eclipseS,eclipseE,sun_orb,duration] = CL_gm_eclipseCir(cjd,sma,inc,raan[,er[,mu[,j2]]])
//
// Description
// <itemizedlist><listitem>
// This function computes eclipse duration for circular orbits. Sun's position is defined by date (<emphasis role="bold">cjd</emphasis>). (computed with CL_mod_moonSunG50)
// <para>Orbit's node is defined by its right ascension (<emphasis role="bold">raan</emphasis>).</para>
// <para>The eclipse is defined as the satellite crossing a cylinder supported on sun's diameter.
// Calculation is purely geometric: sun-orbit geometry is constant</para>
// <para><emphasis role="bold">eclipseS and eclipseE</emphasis> contain the start and end of eclipse positions : [position on orbit(w+M);right ascension in Gamma50 (Veis);declination in Gamma50 (Veis)]</para>
// <para><emphasis role="bold">sun_orb</emphasis> contains sun's right ascension and declination in orbit frame</para>
// <para>Orbit frame is defined as follows:</para>
// <para>axe X: towards node</para>
// <para>axe Z: orbit's kinetic moment vector</para>
// <para>axe Y assures orthonormal direct frame</para>
// <para><emphasis role="bold">duration</emphasis> contains [total angle of eclipse on orbit;duration] (duration is computed using the meanmotion taking into account J2).</para>
// <para>NOTES: </para>
// <para> - planet is assumed spherical and it is necessarily the earth</para>
// <para> - <emphasis role="bold">sun_orb</emphasis>'s declination (in orbit frame) is the beta angle (angle between the satellite's orbital plane and the Sun)</para>
// </listitem>
// <listitem>Last update : 29/2/2008</listitem>
// </itemizedlist>
//
// Parameters
// cjd: CNES julian day (+ut/24, ut in hours) (1xN)
// sma: semi major axis [m] (1xN)
// inc: inclination [rad] (1xN)
// raan: right ascension of ascending node[rad] (1xN)
// er: (optional) equatorial radius [m] (default is %CL_eqRad)
// mu: (optional) geocentric gravitational constant [m^3/s^2] (default value is %CL_mu)
// j2: (optional) zonal coefficient (second zonal harmonic) (default is %CL_j2)
// eclipseS: start of eclipse [position on orbit(w+M);right ascension in Gamma50(Veis);declination in Gamma50(Veis)] [rad] (3xN)
// eclipseE: end of eclipse [position on orbit(w+M);right ascension in Gamma50(Veis);declination in Gamma50(Veis)] [rad] (3xN)
// sun_orb: sun on orbit frame [right ascension;declination (=beta)] [rad] (2xN)
// duration: eclipse duration [angle of position on orbit;duration] [rad;s] (2xN)
//
// Authors
// CNES - DCT/SB
//
// See also
// CL_gm_betaEclipse
// CL_mod_moonSunG50
//
// Examples
// //Example 1
// cjd=21482;
// raan=0;
// sma = 1000000+%CL_eqRad;
// inc = CL_deg2rad(98);
// [eS,eE,s_orb,deltaT] = CL_gm_eclipseCir(cjd,sma,inc,raan)
//
// // Example 2
// cjd = CL_dat_cal2cjd(2009,03,21,6); // march 21 2009, 6am
// inc = CL_deg2rad(90);
// sma = 6978.e3;
// raan = 0;
// [eS,eE,s_orb,deltaT] = CL_gm_eclipseCir(cjd,sma,inc,raan) // Eclipse position is around 180deg of pso...

// Declarations:
global %CL_eqRad %CL_j2 %CL_mu;

// Code:

Nsma = size(sma,2)
Ninc = size(inc,2)
Ncjd = size(cjd,2)
Nraan = size(raan,2)
N = max(Nsma,Ninc,Ncjd,Nraan)
coherence = (Nsma==1|Nsma==N)&(Ninc==1|Ninc==N)&(Ncjd==1|Ncjd==N)&(Nraan==1|Nraan==N)
if ~coherence then error('bad size of input arguments'); end
if N~=1
  if Nsma==1 then sma=sma*ones(1,N); end
  if Ninc==1 then inc=inc*ones(1,N); end
  if Ncjd==1 then cjd=cjd*ones(1,N); end
  if Nraan==1 then raan=raan*ones(1,N); end
end

if ~exists('er','local') then er=%CL_eqRad; end
if ~exists('mu','local') then mu=%CL_mu; end
if ~exists('j2','local') then j2=%CL_j2; end

//mean motion J2
xnmoy = CL_op_paramsJ2('mm',sma,0,inc,er,mu,j2)

//sun direction (right ascension, declination)
[r_sun,rs] = CL_mod_moonSunG50(cjd,'s') //in GAMMA50
sol_sph = CL_co_car2sph(r_sun.*(rs.*.ones(3,1)))
alphasol = sol_sph(1,:)
deltasol = sol_sph(2,:)

//calculation of node right ascension = sun's right ascenscion angle - ascending node right ascension
raan = raan

//calculation:

//sun in orbit's frame
  [alphasol_orb,deltasol_orb] = CL_gm_inertial2orbitSph(inc,raan,alphasol,deltasol)
  //earth semi angle of vision seen from satellite
  betaa = asin(er./(sma))
  //anomaly semi angle on eclipse
  dalpha = zeros(betaa)
  cosdalpha = zeros(betaa)
  aux = cos(betaa)./cos(deltasol_orb)
  i1 = find(   (cos(deltasol_orb)<=0) | (aux>1)  )
  dalpha(i1) = 0
  ii = find(~( (cos(deltasol_orb)<=0) | (aux>1) ))
  cosdalpha(ii) = cos(betaa(ii))./cos(deltasol_orb(ii))
  dalpha(ii) = acos(cosdalpha(ii))

//PSO start and end of eclipse
  dir_deb_orb = zeros(2,N)
  dir_fin_orb = zeros(2,N)
  dir_deb_orb(1,:) = alphasol_orb + %pi - dalpha
  dir_fin_orb(1,:) = alphasol_orb + %pi + dalpha
  dir_deb_orb(2,:) = 0
  dir_fin_orb(2,:) = 0

//right ascension / declination: eclipse start
  dir_deg_g50 = zeros(2,N)
  [aa,bb] = CL_gm_orbit2inertialSph(inc,raan,dir_deb_orb(1,:),dir_deb_orb(2,:))
  dir_deb_g50(1,:) = aa
  dir_deb_g50(2,:) = bb

//right ascension / declination : eclipse end
  dir_fin_g50 = zeros(2,N)
  [cc,dd] = CL_gm_orbit2inertialSph(inc,raan,dir_fin_orb(1,:),dir_fin_orb(2,:))
  dir_fin_g50(1,:) = cc
  dir_fin_g50(2,:) = dd

//local hour start/end eclipse
  hl_deb_ecl = modulo(alphasol-dir_deb_g50(1,:),2*%pi)*12/%pi+12
  hl_fin_ecl = modulo(alphasol-dir_fin_g50(1,:),2*%pi)*12/%pi+12
  jj = find(hl_fin_ecl<hl_deb_ecl)
  hl_fin_ecl(jj) = hl_fin_ecl(jj) + 24

//angle on eclipse
  delta_anom = 2*dalpha

//duration
  duree_eclipse = 2*dalpha./xnmoy

//angle sun / orbit plan
  dir_deb_orb(1,:) = modulo(dir_deb_orb(1,:),2*%pi)
  dir_fin_orb(1,:) = dir_deb_orb(1,:) + modulo(dir_fin_orb(1,:)-dir_deb_orb(1,:),2*%pi)

  dir_deb_g50(1,:) = modulo(dir_deb_g50(1,:),2*%pi)
  dir_fin_g50(1,:) = dir_deb_g50(1,:) + modulo(dir_fin_g50(1,:)-dir_deb_g50(1,:),2*%pi)

  alphasol_orb = pmodulo(alphasol_orb,2*%pi)

pso_start = dir_deb_orb(1,:)
ra_start = dir_deb_g50(1,:)
dec_start = dir_deb_g50(2,:)
pso_end = dir_fin_orb(1,:)
ra_end = dir_fin_g50(1,:)
dec_end = dir_fin_g50(2,:)
ra_sun_orb = alphasol_orb
dec_sun_orb = deltasol_orb

eclipseS = [pso_start;ra_start;dec_start]
eclipseE = [pso_end;ra_end;dec_end]
sun_orb =  [ra_sun_orb;dec_sun_orb]
duration = [delta_anom;duree_eclipse]

endfunction
