//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL  license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'http://www.cecill.info'.

function [eq_overlap]=CL_gm_equatorialOverlap(alt,ecc,inc,swath)
// Ratio between equatorial sensor swath  and equatorial ground track
//
// Calling Sequence
// [eq_overlap]=CL_gm_equatorialOverlap(alt,ecc,inc,swath)
//
// Description
// <itemizedlist><listitem>
// This function computes the ratio between equatorial swath and equatorial ground track. The equatorial swath considers the apparent inclination.
// </listitem>
// <listitem>Last update : 18/12/2009</listitem>
// </itemizedlist>
//
// Parameters
// alt: satellite altitude in meters [1,n]
// ecc: satellite eccentricity [1,n]
// inc: satellite inclination (rad)[1,n]
// swath: satellite sensor swath (rad) [1,n]
// eq_overlap: ratio [1,n]
//
// Authors
// CNES - DCT/SB
//
// Bibliography
// Satellites orbites et missions - Michel Capderou
//
// See also
// CL_gm_equatorialSwath
//
// Examples
// alt=822319;
// ecc=0.000;
// inc=CL_deg2rad(98.72)
// swath=CL_deg2rad(50.5)
// [eq_overlap]=CL_gm_equatorialOverlap([alt,alt],[ecc,ecc],[inc,inc],[swath,swath])

// Declarations:
global %CL_eqRad %CL_omEarth;

// Code:

sma=alt+%CL_eqRad
// Calcul de l'inclinaison apparrente :
[mmj2] = CL_op_paramsJ2('mm',sma,ecc,inc);
[drpom,drift_gom,drM,drpomdaei,drgomdaei,drMdaei]=CL_op_driftJ2(sma,ecc,inc);
incapp=atan((mmj2.*sin(inc))./(mmj2.*cos(inc)-(%CL_omEarth-drift_gom)))
if inc >%pi/2 then
  incapp=incapp+%pi;
end

[dlong] = CL_op_paramsJ2('lgap',sma,ecc,inc)
[alpha]=CL_gm_visiParams(sma,%CL_eqRad,"sat",swath,"cen")
eq_swath=2*alpha./(sin(inc)+cos(inc).*tan(inc-incapp))
eq_overlap=eq_swath./dlong

endfunction

