//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL  license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'http://www.cecill.info'.

function [arg_lat_1,arg_lat_2,inters] = CL_gm_intersectPlanes(inc1,raan1,inc2,raan2)
// Arguments of latitudes at the intersection of 2 planes
//
// Calling Sequence
// [arg_lat_1,arg_lat_2,inters] = CL_gm_intersectPlanes(inc1,raan1,inc2,raan2)
//
// Description
// <itemizedlist><listitem>
// Computes the arguments of latitudes (<emphasis role="bold">arg_lat_1 and arg_lat_2</emphasis>), at the intersection of 2 planes
// defined by their inclinations (<emphasis role="bold">inc1 and inc2</emphasis>) and their right ascension of ascending nodes.(<emphasis role="bold">raan1 and raan2</emphasis>)
// <para>Notes : - There is another solution at arg_lat_1+pi and arg_lat_2+pi</para>
// <para>        - If orbits are circular, then the intersection of planes is also the intersection of orbits.</para>
// <para>        - If planes are coplanar, then the number of intersections is infinite and inters=0</para>
// </listitem>
// <listitem>Last update : 8/4/2008</listitem>
// </itemizedlist>
//
// Parameters
// inc1: inclination of first orbit [rad] (1xN)
// raan1: right ascension of ascending node of first orbit [rad] (1xN)
// inc2: inclination of second orbit [rad] (1xN)
// raan2: right ascension of ascending node of second orbit [rad] (1xN)
// arg_lat_1: argument of latitude of the first orbit, at the intersection [rad] (1xN)
// arg_lat_2: argument of latitude of the second orbit, at the intersection [rad] (1xN)
// inters: flag indicating whether there are only 2 intersections (inters = 1) or an infinity of intersections (inters=0) (1xN)
//
// Authors
// CNES - DCT/SB
//
// Examples
// inc1 = CL_deg2rad(98) ;
// raan1 = CL_deg2rad(102) ;
// inc2 = CL_deg2rad(100) ;
// raan2 = CL_deg2rad(126) ;
// [arg_lat_1,arg_lat_2] = CL_gm_intersectPlanes(inc1,raan1,inc2,raan2)


// Coherence des entrees

// Declarations:


// Code:

Ninc1 = size(inc1,2);
Ninc2 = size(inc2,2);
Nraan1 = size(raan1,2);
Nraan2 = size(raan2,2);
N = max(Ninc1,Ninc2,Nraan1,Nraan2);
coherence = Ninc1==N & Ninc2==N & Nraan1==N & Nraan2==N;
if ~coherence then error('bad input argument size'); end

inters = ones(1,N);

// Moments cinetiques :
cin1 = [sin(raan1) .* sin(inc1)
    -cos(raan1) .* sin(inc1)
    cos(inc1) ];
cin2 = [sin(raan2) .* sin(inc2)
    -cos(raan2) .* sin(inc2)
    cos(inc2) ];

// vect = produit vectoriel des moments cinetiques :
vect = CL_cross(cin1,cin2);
Nvect = CL_colNorm(vect);

// normalisation et changement de signe :
jj = find(Nvect > %eps);
signeVectZ = sign(vect(3,:));
ii = find(signeVectZ == 0 );
signeVectZ(ii) = 1;

vect(1,jj) = vect(1,jj) .* signeVectZ(jj) ./ Nvect(jj);
vect(2,jj) = vect(2,jj) .* signeVectZ(jj) ./ Nvect(jj);
vect(3,jj) = vect(3,jj) .* signeVectZ(jj) ./ Nvect(jj);

arg_lat_1 = zeros(1,N);
arg_lat_2 = zeros(1,N);
cos_arg_lat_1 = cos(raan1(jj)) .* vect(1,jj) + sin(raan1(jj)) .* vect(2,jj);
cos_arg_lat_2 = cos(raan2(jj)) .* vect(1,jj) + sin(raan2(jj)) .* vect(2,jj);

arg_lat_1(jj) = real(acos( cos_arg_lat_1 ));
arg_lat_2(jj) = real(acos( cos_arg_lat_2 ));

ii = find(Nvect <= %eps);
if(ii~=[])
  inters(ii) = 0;
end


endfunction
