//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL  license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'http://www.cecill.info'.

function [loctime_asc,loctime_desc]=CL_gm_lat2locTime(lat,inc_sso,loctime_asc_node)
// Local time at given latitude for a sun synchronous orbit
//
// Calling Sequence
// [loctime_asc,loctime_desc]=CL_gm_lat2locTime(lat,inc_sso,loctime_asc_node)
//
// Description
// <itemizedlist><listitem>
// This function computes for a given heliosynchronous inclination (<emphasis role="bold">inc_sso</emphasis>)
// and a given local time of ascending node (<emphasis role="bold">loctime_asc_node</emphasis>), the satellite's local time for
// ascending arc (<emphasis role="bold">loctime_asc</emphasis>) and descending arc (<emphasis role="bold">loctime_desc</emphasis>)
// at a given latitude (<emphasis role="bold">lat</emphasis>).
// <para>Notes : </para>
// <para> - this function assumes a spherical planet</para>
// <para> - latitude must be inferior to the satellite's inclination (or %pi-inclination if inclination is greater than %pi/2)</para>
// </listitem>
// <listitem>Last update : 2/09/2008</listitem>
// </itemizedlist>
//
// Parameters
// lat: latitude [rad] (1xN)
// inc_sso: heliosynchronous inclination [rad] (1xN)
// loctime_asc_node: local hour of ascending node [hours] (1xN)
// loctime_asc: local hour of satellite [hours] (1xN)
// loctime_desc: local hour of satellite [hours] (1xN)
//
// Authors
// CNES - DCT/SB
//
// Bibliography
// Satellites orbites et missions - Michel Capderou
//
// Examples
// inc_sso = CL_deg2rad(98.6);
// lat = CL_deg2rad(-90+9:1:90-9);
// loctime_asc_node=22;
// [loctime_asc,loctime_desc]=CL_gm_lat2locTime(lat,inc_sso,loctime_asc_node);
// scf();
// plot(loctime_asc,lat*%CL_rad2deg,".r",loctime_desc,lat*%CL_rad2deg,".b")
//

// Declarations:
global %CL_rad2deg;

// Code:

inc_sso_test = inc_sso;
ind = find(inc_sso_test > %pi/2);
inc_sso_test(ind) = %pi - inc_sso_test(ind);

ind = find(abs(lat) > abs(inc_sso_test));

if(~isempty(ind))
  error('error : lat > inc (satellite cant reach a latitude higher that its inclination)');
end

deltatau=(%CL_rad2deg/15)*asin(tan(lat)./tan(inc_sso));
loctime_asc=loctime_asc_node + deltatau;
loctime_asc=pmodulo(loctime_asc,24);
loctime_desc=loctime_asc_node + 12-deltatau;
loctime_desc=pmodulo(loctime_desc,24);

endfunction
