//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL  license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'http://www.cecill.info'.

function [pso_orb_noon] = CL_gm_orbitalNoon(inc,raan,alpha_sun,delta_sun)
// Latitude argument of the orbital noon on the given orbit
//
// Calling Sequence
// [pso_orb_noon] = CL_gm_orbitalNoon(inc,raan,alpha_sun,delta_sun)
//
// Description
// <itemizedlist><listitem>
// The orbital noon is the position of the satellite where the angle between the Earth centre-to-satellite vector
// and the Earth centre-to-Sun vector is minimal.
// <para>In other words, it is the orbital position where the satellite crosses the direction of the sun projection perpendicularly to the orbital plane.</para>
// <para>This function computes the latitude argument(w+M) <emphasis role="bold">(pso_orb_noon)</emphasis> of the orbital noon on the given orbit (<emphasis role="bold">(raan and inc)</emphasis>,
// and for the given sun's right ascension <emphasis role="bold">(alpha_sun)</emphasis> and declination <emphasis role="bold">(delta_sun)</emphasis>.</para>
// <para>Orbital midnight on the orbit (where eclipses are located) is at the opposite of the orbital noon on the orbit (pso_orb_midnight = pso_orb_noon+%pi</para>
// <inlinemediaobject><imageobject><imagedata fileref="orbitalNoon.gif"/></imageobject></inlinemediaobject>
// </listitem>
// <listitem>Last update : 04/05/2007</listitem>
// </itemizedlist>
//
// Parameters
// inc: orbit's  inclination [rad] (1xN)
// raan: orbit's right ascension of ascending node [rad] (1xN)
// alpha_sun: sun's right ascension [rad] (1xN)
// delta_sun: sun declination [rad] (1xN)
// pso_orb_noon: latitude argument(w+M) of the solar midday on the given orbit [rad] (1xN)
//
// Authors
// CNES - DCT/SB
//
// See also
// CL_gm_raan2beta
// CL_gm_beta2raan
// CL_gm_eclipseCir
//
// Examples
// cjd=21482;
// raan=0;
// sma = 1000000+%CL_eqRad;
// inc = CL_deg2rad(98);
// [r_sun,rs] = CL_mod_moonSunG50(cjd,'s');
// sol_sph = CL_co_car2sph(r_sun.*(rs.*.ones(3,1)));
// alpha_sun = sol_sph(1,:);
// delta_sun = sol_sph(2,:);
// pso_orb_noon = CL_gm_orbitalNoon(inc,raan,alpha_sun,delta_sun)
//
// // Comparison with CL_gm_eclipseCir :
// [eS,eE,s_orb,deltaT] = CL_gm_eclipseCir(cjd,sma,inc,raan)
// pso_midnight = (eS(1) + eE(1) ) / 2 // pso_midnight = pso_orb_noon + %pi
// pso_noon = pso_midnight + %pi

//[betaa] = CL_gm_raan2beta(inc,raan,alpha_sun,delta_sun)
//pso_orb_noon = %pi/2 - real(asin( (sin(delta_sun)-sin(betaa).*cos(inc))./cos(betaa)./sin(inc) ) - %pi/2)


// Declarations:


// Code:

[alpha2,delta2] = CL_gm_inertial2orbitSph(inc,raan,alpha_sun,delta_sun);
pso_orb_noon = alpha2;

endfunction
