//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL  license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'http://www.cecill.info'.

function [K1,K2]=CL_gm_pixelSize(sat_alt,tilt,target_alt,er)
// Deformation ratio of a pixel for a given altitude and a given swath (tilt)
//
// Calling Sequence
// [K1,K2]=CL_gm_pixelSize(sat_alt,tilt,target_alt,[er])
//
// Description
// <itemizedlist><listitem>
// Computes the deformation ratio K1 (tilt direction) and K2 (direction perpendicular to tilt) of a pixel for a given altitude and a given swath (tilt).
// <para> Deformation ratio is defined as follows : </para>
// <inlinemediaobject><imageobject><imagedata fileref="pixelSize.gif"/></imageobject></inlinemediaobject>
// <para> Note : this function assumes a spherical earth</para>
// </listitem>
// <listitem>Last update : 09/11/2009</listitem>
// </itemizedlist>
//
// Parameters
// sat_alt: satellite altitude [m] (1xN)
// tilt: swath or tilt (depointing) angle [rad] (1xN)
// target_alt: (optional) target altitude [m]  (default is 0 m)(1xN)
// er : (optional) planet radius (default is earth equatorial radius %CL_eqRad) [m]
// K1: pixel ratio in tilt direction(1xN)
// K2: pixel ratio in perpendicular direction to tilt(1xN)
//
// Authors
// CNES - DCT/SB
//
// See also
// CL_gm_visiParams
//
// Bibliography
// Satellites Orbites et Missions (M Capderou)
//
// Examples
// // Example 1
// sat_alt = 700.e3;
// tilt = CL_deg2rad(20);
// [K1,K2]=CL_gm_pixelSize(sat_alt,tilt)
// // same computation, with CL_gm_visiParams :
// sat_radius = %CL_eqRad + sat_alt;
// target_radius = %CL_eqRad;
// [dist] = CL_gm_visiParams(sat_radius,target_radius,'sat',tilt,'dist') // distance for the given tilt
// [elev] = CL_gm_visiParams(sat_radius,target_radius,'sat',tilt,'elev') // elevation for the given tilt
// K1 = dist/(sat_alt * sin(elev))
// K2 = dist/sat_alt
//
// // Example 2
// tab_f=[];
// K1=[];
// K2=[];
// fground=[];
// leg=[];
// i=0;
// f = scf();
// drawlater();
// xset("colormap",jetcolormap(10));
//
// for sat_alt=[200000:200000:1400000];
//    i=i+1;
//    fground=[fground ,i];
//    leg=[leg,sat_alt/1000];
//    f=[0:1:50]*%CL_deg2rad;
//    [K1,K2]=CL_gm_pixelSize(sat_alt,f)  ;
//    tab_f=[tab_f;f];
//    plot(f*%CL_rad2deg,abs(K1))
//    p=gce();
//    p.children.foreground=i;
// end
//    a=gca();
//    a.title.text="DEFORMATION DES PIXELS (K1)";
//    a.title.foreground = 12;
//    a.title.font_size = 4;
//    a.x_label.text="alpha (deg)";
//    a.x_label.font_size = 4;
//    a.x_label.foreground = 12;
//    a.x_location = "bottom";
//    a.y_label.text=" K1";
//    a.y_label.font_size = 4;
//    a.y_label.foreground = 12;
//    a.y_location = "left";
//    a.foreground = 12;
//    a.data_bounds = [0,1;50,6];
//    legends("sat_alt: "+string(leg)+" km",fground,2);
//    xgrid();
//    drawnow();
//

// Declarations:
global %CL_eqRad;

// Code:

  if ~exists('er','local') then er=%CL_eqRad; end
  if ~exists('target_alt','local') then target_alt=0; end

  sat_radius = er + sat_alt;
  target_radius = er + target_alt;
  [dist] = CL_gm_visiParams(sat_radius,target_radius,'sat',tilt,'dist') // distance for the given tilt
  [elev] = CL_gm_visiParams(sat_radius,target_radius,'sat',tilt,'elev') // elevation for the given tilt
  K1 = dist./(sat_alt .* sin(elev))
  K2 = dist./sat_alt

endfunction
