//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL  license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'http://www.cecill.info'.

function dist = CL_gm_sphericDist(p1,p2,er)
// Great circle distance between 2 geographic coordinates
//
// Calling Sequence
// dist = CL_gm_sphericDist(p1,p2[,er])
//
// Description
// <itemizedlist><listitem>
// Given two points in geocentric coordinates (longitude and latitude), this functions gives the
// great circle distance between them approximating the planet as a sphere. The three following formulas
// give the exact great circle distance between two points (lambda: longitude, phi:latitude) for
// a sphere of radius 1. However, the first formula (1) is used on this funtion due to possible
// rounding errors of formulas (2) and (3) for small distances or antipodal points.
// </listitem>
// <listitem><para/><inlinemediaobject><imageobject><imagedata fileref="GCdist.gif"/></imageobject></inlinemediaobject></listitem>
// <listitem>Last update : 14/2/2008</listitem>
// </itemizedlist>
//
// Parameters
// p1: first point [LON;LAT] geocentric coordinates of first point [rad] (2xN)
// p2: second point [LON;LAT] geocentric coordinates of second point [rad] (2xN)
// er : (optional) earth radius [m] (default %CL_eqRad)
// dist:  great circle distance between p1 and p2 [m] (1xN)
//
// Authors
// CNES - DCT/SB
//
// Bibliography
// 1 R.W. Sinnott, "Virtues of the Haversine", Sky and Telescope, vol. 68, no. 2, 1984, p. 159
// 2 Weisstein, Eric W. "Great Circle." From MathWorld. http://mathworld.wolfram.com/GreatCircle.html
//
// See also
// CL_co_car2sph
// CL_co_sph2car
//
// Examples
// // Spherical distance between Toulouse (43deg 36' 19"N, 1deg 26' 34"E) and Barcelona (41deg 24' 7"N, 2deg 10' 17"E)
// tlse_lat = 43 + 36/60 + 19/3600;
// tlse_lon = 1 + 26/60 + 34/3600;
// bcn_lat = 41 + 24/60 + 7/3600;
// bcn_lon = 2 + 10/60 + 17/3600;
// tlse_coord = CL_deg2rad([tlse_lon;tlse_lat]);
// bcn_coord = CL_deg2rad([bcn_lon;bcn_lat]);
// tlse_bcn = CL_gm_sphericDist(tlse_coord,bcn_coord); //great circle distance toulouse - barcelona
// disp('Toulouse - Barcelona: '+string(tlse_bcn / 1000)+' km');
//

// Declarations:
global %CL_eqRad;

// Code:

if ~exists('er','local') then er=%CL_eqRad; end

lon1 = p1(1,:);
lon2 = p2(1,:);
lat1 = p1(2,:);
lat2 = p2(2,:);


dist = atan(..
             sqrt( ( cos(lat2).*sin(lon2-lon1) ).^2 + ( cos(lat1).*sin(lat2)-sin(lat1).*cos(lat2).*cos(lon2-lon1) ).^2 ),..//y
             sin(lat1).*sin(lat2)+cos(lat1).*cos(lat2).*cos(lon2-lon1).. //x
            ).*er;

endfunction
