//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL  license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'http://www.cecill.info'.

function [elev] = CL_gm_stationElevation(pos_sat_ter,stations,er,obla)
// Ground stations elevations of satellites positions.
//
// Calling Sequence
// [elev] = CL_gm_stationElevation(pos_sat_ter,stations,[er,[obla]])
//
// Description
// <itemizedlist><listitem>
// This function computes elevations (<emphasis role="bold">elev</emphasis>) for one or several <emphasis role="bold">stations</emphasis>.
// There is visibility when the elevation is greater than the station's mask.
// <para>Satellite's positions (<emphasis role="bold">pos_sat_ter</emphasis>) are given in a terrestrial frame and cartesian coordinates</para>
// <para>You can optionally change earth equatorial radius <emphasis role="bold">(er)</emphasis> and oblateness <emphasis role="bold">(obla)</emphasis>.
// Setting obla to 0 will result in a faster computation.</para>
// <para>See below for detailed examples</para>
// </listitem>
// <listitem>Last update : 20/08/2009</listitem>
// </itemizedlist>
//
// Parameters
// pos_sat_ter: satellites positions in the terrestrial frame, in cartesian coordinates [X,Y,Z] [m,m,m] (3xM)
// stations: stations positions in the terrestrial frame, in elliptical(geodetic) coordinates [long,lat,alt] [rad,rad,m] (3xN)
// er : (optional) Earth semi-major axis (default is earth equatorial radius %CL_eqRad) [m] (1x1)
// obla : (optional) Earth oblateness, setting it to 0 will result in a faster computation (default is earth oblateness %CL_obla) (1x1)
// elev: elevations for each stations and each satellite position : in each column j of elev are the elevations for station j [rad] (MxN)
//
// Authors
// CNES - DCT/SB
//
// See also
// CL_gm_visibility
//
// Examples
// //secular J2 extrapolation
// cjd0 = 21915;
// pas = 10./86400.0;
// cjd = cjd0 : pas : cjd0+1;
// kep0 = [7070 * 1000 ; 0.001 ; CL_deg2rad(98) ; CL_deg2rad(90) ; 0 ; 0];
// kep = CL_ex_secularJ2(cjd0, kep0, cjd);
//
// //conversion in terrestrial frame
// [pos_car,vel_car] = CL_oe_kep2car(kep);
// pos_sat_ter = CL_fr_G502ter(cjd,pos_car);
//
// //stations definition
// sta1 = [CL_deg2rad(2);CL_deg2rad(70);200]; // high latitude
// sta2 = [CL_deg2rad(20);CL_deg2rad(0);400]; // equator
// stations = [sta1,sta2];
//
// //elevations computation
// [elev] = CL_gm_stationElevation(pos_sat_ter,stations);
//
// // elevation in function of time
// scf();
// plot2d((cjd-cjd0)*24,CL_rad2deg(elev(:,1)),2) //station 1
// plot2d((cjd-cjd0)*24,CL_rad2deg(elev(:,2)),3) //station 2
//
// // visibility duration if station's mask is 5 degrees :
// ind_1 = find(elev(:,1) > CL_deg2rad(5));  //station 1
// ind_2 = find(elev(:,2) > CL_deg2rad(5));  //station 2
// dur_1_minutes = pas*length(ind_1)*1440.0
// dur_2_minutes = pas*length(ind_2)*1440.0
//
// // Same computation with obla=0 (faster)
// [elev] = CL_gm_stationElevation(pos_sat_ter,stations,obla=0);

// Declarations:
global %CL_eqRad %CL_obla;

// Code:

  if ~exists('er','local') then er=%CL_eqRad; end
  if ~exists('obla','local') then obla=%CL_obla; end

  N = size(stations , 'c');      // nombre de stations
  M = size(pos_sat_ter , 'c');   // nombre de satellites


  elev = zeros(M,N);

  // Terre spherique : simple produit scalaire entre 2 vecteurs
  if(obla == 0)

  pos_sta = CL_co_sph2car([stations(1:2,:);stations(3,:)+er]);  // warning spherical position takes distance from center (not altitude)
  // Boucle sur les stations
  if(N < M)
    for i=1:N
    pos_sta_i = [ ones(1,M) * pos_sta(1,i),
              ones(1,M) * pos_sta(2,i),
              ones(1,M) * pos_sta(3,i) ];      // station i(dimension 3,M)
    elev_sta_i = %pi/2 - acos(CL_dot(CL_unitVector(pos_sta_i),CL_unitVector(pos_sat_ter-pos_sta_i))); // produit scalaire de sta_i par pos_sat_ter-sta_i (dimension 1,M)
    elev(:,i) = elev_sta_i'; // on remplit la colonne i par l ensemble des resultats pour la station i
    end
  // Boucle sur les satellites
  else
      for i=1:M
    sat_i = [ ones(1,N) * pos_sat_ter(1,i),
           ones(1,N) * pos_sat_ter(2,i),
           ones(1,N) * pos_sat_ter(3,i) ];      // satellite i(dimension 3,N)
    elev_sat_i = %pi/2 - acos(CL_dot(CL_unitVector(pos_sta),CL_unitVector(sat_i-pos_sta))); // produit scalaire de pos_sta par sat_i-pos_sta (dimension 1,N)
    elev(i,:) = elev_sat_i; // on remplit la ligne i par l ensemble des resultats pour le satellite i
    end
  end

  // Terre elliptique : il faut passer par le repere topocentrique nord
  else
      // Boucle sur les stations
      for i=1:N
       //transformation of satellite's position to topocentric north frame centered at the station i
       pos_sat_topoN = CL_fr_ter2topoN(stations(:,i),pos_sat_ter)

    //transformation from cartesien to spherical coordinates (site,azimut,distance)
    pos_sat_sph = CL_co_car2sph(pos_sat_topoN)

    //elevation is pos_sat_sph(2,:)
    elev(:,i) = pos_sat_sph(2,:)'; // on remplit la colonne i par l ensemble des resultats pour la station i

  end
  end

endfunction
