//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL  license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'http://www.cecill.info'.

function [AngZen,AzSol] = CL_gm_sunZenithAngle(lat,decl,hloc)
// Sun zenith angle and sun azimuth in function of latitude, sun declination and local hour of a Earth's point.
//
// Calling Sequence
// [AngZen,AzSol] = CL_gm_sunZenithAngle(lat,decl,hloc)
//
// Description
// <itemizedlist><listitem>
// Given one Earth point's latitude,local hour and sun declination , this function computes the angle between the zenith of the point and sun direction (Sun Zenith Angle) and the corresponding sun azimuth
// </listitem>
// <listitem>Last update : 24/11/2008</listitem>
// </itemizedlist>
//
// Parameters
// AngZen: Earth point's sun zenith angle [deg] (MxNxP)
// AzSol: Earth point's sun azimuth angle [deg] (MxNxP)
// lat: vector of latitudes [deg] (Mx1)
// decl: vector of sun declinations [deg] (Nx1)
// hloc: vector of local hours [deg] (Px1)
//
// Bibliography
// Satellites Orbites et Missions (M Capderou)
//
// Authors
// CNES - DCT/SB
//
// See also
//
// Examples
//  //example 1
// lat = [10;20;30];
// decl= [0;15;23];
// hloc= [45;60];
// [AngZen,AzSol] = CL_gm_sunZenithAngle(lat,decl,hloc)
//
//  //example 2
// lat = [60];
// hloc = (0:-7.5:-172.5);
// hloc = [hloc,(-180:7.5:-7.5)];
// hloc = hloc';
// dates=(23011:23375);
// [r_sun,rs] = CL_mod_moonSunG50(dates,'s');
// pos_sph = CL_co_car2sph(r_sun);
// decl = pos_sph(2,:);
// decl = decl'*%CL_rad2deg;
// mtzPlot=[];
// sizHloc1=size(hloc);
// nLigHloc1=sizHloc1(1);
// sizDecl1=size(decl);
// nLigDecl1=sizDecl1(1);
// sizLat1=size(lat);
// nLigLat1=sizLat1(1);
// [AngZen,AzSol] = CL_gm_sunZenithAngle(lat,decl,hloc)
// mtzPlot=matrix(AngZen,nLigDecl1,nLigHloc1);
// mtzPlot=mtzPlot';
// xset("colormap",jetcolormap(180))
// Matplot(mtzPlot);
// ax=gca();
// ax.title.text= "Sun Zenith Angle";
// ax.x_label.text= "Temps(jours)";
// ax.y_label.text= "Heure locale";
// ax.font_size=2;
// ax.title.font_size=2;
// ax.x_label.font_size=2;
// ax.y_label.font_size=2;
// ax.y_ticks.labels=["11.5" "14" "16.5" "19" "21.5" "0" "2.5" "5" "7.5" "10" "12.5"];
//
//

// Declarations:
global %CL_deg2rad %CL_rad2deg;

// Code:

csAngZenP=[];
csAngZenP1=[];
csAngZenP2=[];
csAngZenP3=[];
csAngZenP4=[];
csAngZenS=[];
csAngZenS1=[];
csAngZenS2=[];
csAngZenS3=[];
csAngZenS4=[];
csAngZenPones =[];
csAngZen=[];
AngZen=[];

tgAzSolNones=[];
tgAzSolN1=[];
tgAzSolN2=[];
tgAzSolN3=[];
tgAzSolN4=[];
tgAzSolN5=[];
tgAzSolS=[];
tgAzSolS1=[];
tgAzSolS2=[];
tgAzSolS3=[];
tgAzSolS4=[];
tgAzSolT=[];
tgAzSolT1=[];
tgAzSolT2=[];
tgAzSolT3=[];
tgAzSolT4=[];
tgAzSolTones=[];
tgAzSolD=[];
tgAzSol1=[];
tgAzSol2=[];
AzSol1=[];
AzSol=[];

decl = decl';

sizHloc=size(hloc);
nLigHloc=sizHloc(1);
sizDecl=size(decl);
nLigDecl=sizDecl(2);
sizLat=size(lat);
nLigLat=sizLat(1);

//Calcul de l'angle zenithal solaire.
[csAngZenP]=sin(lat*%CL_deg2rad)*sin(decl*%CL_deg2rad);
[csAngZenS]=cos(lat*%CL_deg2rad)*cos(decl*%CL_deg2rad);

csAngZenPones = ones(1,nLigHloc);
csAngZenP1=matrix(csAngZenP,nLigDecl*nLigLat,1);
csAngZenP2=csAngZenP1*csAngZenPones;
csAngZenP3=matrix(csAngZenP2,nLigDecl*nLigLat*nLigHloc,1);
csAngZenP4=hypermat([nLigLat nLigDecl  nLigHloc],csAngZenP3);

csAngZenS1= matrix(csAngZenS,nLigDecl*nLigLat,1);
csAngZenS2= csAngZenS1*cos(hloc'*%CL_deg2rad);
csAngZenS3= matrix(csAngZenS2,nLigDecl*nLigLat*nLigHloc,1);
csAngZenS4=hypermat([nLigLat nLigDecl  nLigHloc],csAngZenS3);

csAngZen= csAngZenS4 + csAngZenP4;
[AngZen]=acos(csAngZen);
AngZen = AngZen.*%CL_rad2deg;

//Calcul de l'angle azimuthal solaire.
//Calcul du numerateur.
tgAzSolNones=ones(nLigLat,1);
tgAzSolN1=tgAzSolNones*cos(decl*%CL_deg2rad);
tgAzSolN2= matrix(tgAzSolN1,nLigDecl*nLigLat,1);
tgAzSolN3= tgAzSolN2*sin(hloc'*%CL_deg2rad);
tgAzSolN4= matrix(tgAzSolN3,nLigDecl*nLigLat*nLigHloc,1);
tgAzSolN5=hypermat([nLigLat nLigDecl  nLigHloc],tgAzSolN4);


//Calcul du denominateur.
[tgAzSolS]= sin(lat*%CL_deg2rad)*cos(decl*%CL_deg2rad);
[tgAzSolT]= cos(lat*%CL_deg2rad)*sin(decl*%CL_deg2rad);

tgAzSolTones = ones(1,nLigHloc);
tgAzSolT1=matrix(tgAzSolT,nLigDecl*nLigLat,1);
tgAzSolT2=tgAzSolT1*tgAzSolTones;
tgAzSolT3=matrix(tgAzSolT2,nLigDecl*nLigLat*nLigHloc,1);
tgAzSolT4=hypermat([nLigLat nLigDecl  nLigHloc],tgAzSolT3);

tgAzSolS1= matrix(tgAzSolS,nLigDecl*nLigLat,1);
tgAzSolS2= tgAzSolS1*cos(hloc'*%CL_deg2rad);
tgAzSolS3= matrix(tgAzSolS2,nLigDecl*nLigLat*nLigHloc,1);
tgAzSolS4=hypermat([nLigLat nLigDecl  nLigHloc],tgAzSolS3);

tgAzSolD= tgAzSolS4 - tgAzSolT4;
tgAzSol1=tgAzSolN5./tgAzSolD;

tgAzSol2 = matrix(tgAzSol1,nLigDecl*nLigLat*nLigHloc,1);
AzSol1=atan(tgAzSol2);
AzSol=hypermat([nLigLat nLigDecl  nLigHloc],AzSol1);

AzSol = AzSol.*%CL_rad2deg;
endfunction
