//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL  license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'http://www.cecill.info'.

function [par2]=CL_gm_visiParams(sat_radius,target_radius,type_par1,par1,type_par2)
// Visibility parameters (angles, distance, ...) for a spherical planet
//
// Calling Sequence
// [par2]=CL_gm_visiParams(sat_radius,target_radius,type_par1,par1,type_par2)
//
// Description
// <itemizedlist><listitem>
// The aim of this function is to compute relations between some instant visibility parameters:
// <para>- satellite's semi visibility field (swath); that is, angle between descending vertical and target direction : <emphasis role="bold">'sat'</emphasis></para>
// <para>- satellite's elevation from a ground point <emphasis role="bold">'elev'</emphasis></para>
// <para>- incidence of target direction <emphasis role="bold">'incid'</emphasis>(=pi/2 - elev)</para>
// <para>- distance between satellite and target point on the ground <emphasis role="bold">'dist'</emphasis></para>
// <para>- angle at center; that is, angle between directions (earth center -> satellite) and (earth center -> target point) <emphasis role="bold">'cen'</emphasis>. This angle measures the size of the surface seen on the ground.</para>
// <para>Given a satellite radius (<emphasis role="bold">sat_radius</emphasis> = satellite altitude + earth radius), a target radius (
// <emphasis role="bold">target_radius</emphasis> = target altitude + earth radius) and one of the following parameters (<emphasis role="bold">type_par1 and par1</emphasis>):</para>
// <para>type_par1 = 'sat' , par1 = satellite's semi visibility field (swath) in radians.</para>
// <para>type_par1 = 'elev' , par1 = elevation from the ground point in radians.</para>
// <para>type_par1 = 'incid' , par1 = incidence (=pi/2-elev) from the ground point in radians.</para>
// <para>type_par1 = 'dist' , par1 = distance between the ground point and the satellite in meters.</para>
// <para>type_par1 = 'cen' , par1 = angle at center between the ground point and the satellite in radians.</para>
// <para>The function computes <emphasis role="bold">par2</emphasis> which type is defined by <emphasis role="bold">type_par2</emphasis> and can be any of the 5 parameters above, or 'all'</para></listitem>
// <listitem><para/><inlinemediaobject><imageobject><imagedata fileref="visiParams.gif"/></imageobject></inlinemediaobject></listitem>
// <listitem><para>NOTES: </para>
// <para> - a spheric planet is assumed</para>
// <para> - radius of target point can be lower than observer's point (meaning that oberver point can be an interior point of the sphere of radius target_radius)</para>
// <para> - when choosing 'sat' as type_par1, if there are 2 solutions, only the solution of positive elevation is computed</para>
// </listitem>
// <listitem>Last update : 19/2/2008</listitem>
// </itemizedlist>
//
// Parameters
// sat_radius: satellite radius (satellite altitude + earth radius) [m] (1xN)
// target_radius: target radius (target altitude + earth radius) [m] (1xN)
// type_par1: string defining par1. It can be 'sat', 'elev', 'incid', 'dist', 'cen'
// par1: satellite's angle, elev, indicence, distance or angle at center [rad,m] (1xN)
// type_par2: string defining par2 to be computed. It can be 'sat', 'elev', 'incid', 'dist', 'cen' or 'all'
// par2: Computed value : satellite's angle, elevation, indicence, distance, angle at center or all of the previous (in the same order) [rad,m] (1xN or 5xN)
//
// Authors
// CNES - DCT/SB
//
// Examples
// sat_radius = %CL_eqRad + 700.e3; // 700 km
// target_radius = %CL_eqRad + 200; // 200 m
// [incid]=CL_gm_visiParams(sat_radius,target_radius,'dist',800.e3,'incid') // incidence for a 800km distance
// [elev]=CL_gm_visiParams(sat_radius,target_radius,'sat',CL_deg2rad(10),'elev') // elevation for a 10deg roll (swath)
// [dist]=CL_gm_visiParams(sat_radius,target_radius,'cen',CL_deg2rad(7),'dist') // distance for a 7deg center angle
// [all]=CL_gm_visiParams(sat_radius,target_radius,'sat',CL_deg2rad(37),'all') // all parameters for a 37deg swath
// all(2) // elev
// all(3) // incid
// all(4) // dist
// all(5) // cen


// Declarations:


// Code:

[lhs,rhs]=argn();
if(rhs ~= 5) then error('this function requires 5 input arguments'); end

//calculation of angle at center depending on input type_par1
if type_par1=='elev'|type_par1=='incid' //par1 elevation or incidence

  if type_par1=='incid'
    angsit = %pi/2 - par1
  else
    angsit = par1
  end
  i1 = find( abs(angsit)>%pi/2 )
  if i1~=[] then error('elevation greater than pi/2'); end
  angcen = acos( (target_radius./sat_radius).*cos(angsit) ) - angsit

elseif type_par1=='sat'  //par1 is satellite angle

  angsat = par1
  i2 = find( abs(angsat)>asin(target_radius./sat_radius) )
  if i2~=[] then error('Satellite angle greater than asin(target_radius/sat_radius) : no intersection with the earth'); end
  angsit = acos( (sat_radius./target_radius) .* sin(angsat) )
  //2 solutions
  angcen = %pi/2 - angsat - angsit
  //warning('only solution for elev>=0 is computed');
  //angcen = %pi/2 - angsat + angsit

elseif type_par1=='dist' //par1 is distance

  dist = par1
  i3 = find( dist<sat_radius-target_radius | dist>sat_radius+target_radius )
  if i3~=[] then error('Distance too low (<sat_radius-target_radius) or too high (>sat_radius+target_radius)'); end
  angcen = acos( (sat_radius.^2 + target_radius.^2 - dist.^2) ./ (2.*sat_radius.*target_radius))

elseif type_par1=='cen'  //par1 is angle at center

  angcen = par1

else
  error('unknown type_par1 value');
end

//calculation of output parameters in function of angle at center
angsit = atan(cos(angcen) - target_radius./sat_radius , sin(angcen))
ang_elev = angsit
ang_inc = %pi/2 - angsit
ang_sat = %pi/2 - angcen - angsit
dist = sqrt(sat_radius.^2 + target_radius.^2 - 2.*sat_radius.*target_radius.*cos(angcen))
ang_cen = angcen

//chosen output
if type_par2=='sat'
 par2 = ang_sat
elseif type_par2=='elev'
 par2 = ang_elev
elseif type_par2=='incid'
 par2 = ang_inc
elseif type_par2=='dist'
 par2 = dist
elseif type_par2=='cen'
 par2 = ang_cen
elseif type_par2=='all'
 par2 = [ ang_sat;ang_elev;ang_inc;dist;ang_cen]
end

endfunction
