//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL  license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'http://www.cecill.info'.

function [visi_dates] = CL_gm_visibility(cjd,mean_kep,stations,stations_masks,simulation_period,visi_min,prec,propagation_type,er,mu,zonals,obla)
// Geometric station visibilities start and end dates.
//
// Calling Sequence
// [visi_dates] = CL_gm_visibility(cjd,mean_kep,stations,stations_masks,simulation_period,[visi_min,prec,propagation_type,er,mu,zonals,obla])
//
// Description
// <itemizedlist><listitem>
// This function computes visibility start and end dates (<emphasis role="bold">visi_dates</emphasis>) for one or several <emphasis role="bold">stations</emphasis>,
// between dates defined in <emphasis role="bold">simulation_period</emphasis>.
// <para>There is visibility when the elevation over one of the stations is greater than the <emphasis role="bold">stations_masks</emphasis>.
// This means that the result (<emphasis role="bold">visi_dates</emphasis>) are the intervals when <emphasis role="bold">one or more</emphasis> stations are in visibility,
// and consecutive visibilities are concatenated</para>
// <para>Any visibility lasting less than <emphasis role="bold">visi_min</emphasis> seconds (default is 60) is not returned by this function.
// Choosing a small <emphasis role="bold">visi_min</emphasis> will increase computation time.</para>
// <para>Satellite's keplerian orbital elements<emphasis role="bold">(mean_kep)</emphasis> are given at date <emphasis role="bold">cjd</emphasis> in Gamma50 (Veis) frame.
// Extrapolation is done using Lyddane analytic model.</para>
// <para>You can optionally change earth equatorial radius <emphasis role="bold">(er)</emphasis> and oblateness <emphasis role="bold">(obla)</emphasis>.
// Setting obla to 0 will result in a faster computation.</para>
// <para>See below for detailed examples</para>
// </listitem>
// <listitem>If you want to manipulate intervals, you might want to use functions CL_intervUnion, CL_intervInters or CL_intervInv</listitem>
// <listitem>Last update : 20/08/2009</listitem>
// </itemizedlist>
//
// Parameters
// cjd: CNES julian date of orbital element  (1x1)
// mean_kep: satellite's keplerian mean (depending on propagation_type : Lyddane or Eckstein-Hechler or J2) orbital elements at date cjd in Gamma50 (Veis) frame [sma,ecc,inc,pom,gom,anm] (6x1)
// stations: stations positions in the terrestrial frame, in elliptical(geodetic) coordinates [long,lat,alt] [rad,rad,m] (3xN)
// stations_masks: stations masks There is visibility when the elevation is greater than the stations masks. You can have the same station mask for all stations or a different one for each [rad] (1xN or 1x1)
// simulation_period: CNES julian dates interval of visibility computation [cjd_start ; cjd_end](2x1)
// visi_min: (optional) minimum visibility time (default is 60 seconds) [sec] (1x1)
// prec: (optional) precision of computation in seconds  (default is 1 second) [sec] (1x1)
// propagation_type: (optional) 'j2' for secular J2, 'lyd' for lyddane or 'eh' for Eckstein Hechler (default is lyddane) (1x1)
// er : (optional) Earth semi-major axis (default is earth equatorial radius %CL_eqRad) [m] (1x1)
// mu: (optional) geocentric gravitational constant [m^3/s^2] (default value is %CL_mu)
// zonals: (optional) vector of zonals coefficients J1 to Jn (troncated to J5) to be used (default is %CL_j1jn(1:5)) (1 x N)
// obla : (optional) Earth oblateness, setting it to 0 will result in a faster computation (default is earth oblateness %CL_obla) (1x1)
// visi_dates: visibility start and end dates (i.e every time elevation crosses station_mask value) [cjd_visi_start ; cjd_visi_end](2xM)
//
// Authors
// CNES - DCT/SB
//
// See also
// CL_gm_stationElevation
// CL_ex_lyddane
// CL_intervUnion
// CL_intervInters
// CL_intervInv
//
// Examples
// cjd = 21915;
// mean_kep = [7070 * 1000 ; 0.001 ; CL_deg2rad(98) ; CL_deg2rad(90) ; 0 ; 0];
//
// //stations definition
// sta1 = [CL_deg2rad(2);CL_deg2rad(70);200]; // high latitude
// sta2 = [CL_deg2rad(20);CL_deg2rad(0);400]; // equator
// stations = [sta1,sta2];
// stations_masks = [ CL_deg2rad(10) , CL_deg2rad(2) ];
//
// simulation_period = [21915 ; 21918 ];  // 3 days
//
// //visibility computation
// [visi_dates] = CL_gm_visibility(cjd,mean_kep,stations,stations_masks,simulation_period);
//
// // visibilities duration in function of time plot :
// scf();
// dates_plot = 0:1/86400:3;
// ind_dates = [];
// visi = zeros(dates_plot);
// for k=1:size(visi_dates,2)
//   ind = find( simulation_period(1)+dates_plot &gt; visi_dates(1,k) &amp; simulation_period(1)+dates_plot &lt; visi_dates(2,k));
//   visi(ind) = (visi_dates(2,k)-visi_dates(1,k))*1440.0;
// end
// plot2d(dates_plot*24,visi,2)
//
// //histogram of visibilities duration
// scf();
// histplot(20,(visi_dates(2,:)-visi_dates(1,:))*1440.0,normalization=%f)
// a=gca();
// a.title.text = "Number of visibilities in function of visibility duration (minutes)";
//
// //stations elevations and visi start and end dates on same plot :
// scf();
// pas = 60.0/86400.0;
// dates = simulation_period(1):pas:simulation_period(2);
// [moy,osc] = CL_ex_lyddane(cjd,mean_kep,dates);
// [pos_sat_G50,vit_sat_G50] = CL_oe_kep2car(osc);
// M = CL_fr_G502terMat(dates);
// pos_sat_ter = M*pos_sat_G50;
// [elev] = CL_gm_stationElevation(pos_sat_ter,stations);
// plot2d((dates-cjd)*24,CL_rad2deg(elev(:,1)),2); //station 1
// plot2d((dates-cjd)*24,CL_rad2deg(elev(:,2)),3); //station 2
// [visi_dates_1] = CL_gm_visibility(cjd,mean_kep,stations(:,1),stations_masks(1),simulation_period);
// [visi_dates_2] = CL_gm_visibility(cjd,mean_kep,stations(:,2),stations_masks(2),simulation_period);
// plot((visi_dates_1(1,:)-cjd)*24,CL_rad2deg(stations_masks(1))*ones(visi_dates_1(1,:)),'bs');
// plot((visi_dates_1(2,:)-cjd)*24,CL_rad2deg(stations_masks(1))*ones(visi_dates_1(1,:)),'bs');
// plot((visi_dates_2(1,:)-cjd)*24,CL_rad2deg(stations_masks(2))*ones(visi_dates_2(1,:)),'gs');
// plot((visi_dates_2(2,:)-cjd)*24,CL_rad2deg(stations_masks(2))*ones(visi_dates_2(1,:)),'gs');
//
// // Long extrapolation : make loops
// visi_dates = [];
// stacksize('max');
// for k=0:3 // total = 400 days
// simulation_period=[21915+k*100 ; 21925+k*100 ];  // 100 days
// [visi_dates] = [visi_dates,CL_gm_visibility(cjd,mean_kep,stations,stations_masks,simulation_period)];
// end
//
//
// // Same computation with obla=0 (faster)
// [visi_dates] = CL_gm_visibility(cjd,mean_kep,stations,stations_masks,simulation_period,obla=0);


// Declarations:
global %CL_eqRad %CL_j1jn %CL_mu %CL_obla;

// Code:

  if ~exists('visi_min','local') then visi_min=60.0; end
  if ~exists('prec','local') then prec=1; end
  if ~exists('er','local') then er=%CL_eqRad; end
  if ~exists('propagation_type','local') then propagation_type='lyd'; end
  if ~exists('mu','local') then mu=%CL_mu; end
  if ~exists('zonals','local') then zonals=%CL_j1jn(1:5); end
  if ~exists('obla','local') then obla=%CL_obla; end

  Nsta = size(stations , 'c');      // nombre de stations
  Nmask = size(stations_masks , 'c');      // nombre de stations masks
  if(Nsta~=Nmask & Nmask~=1) then error('stations_masks must either be 1 or of same size as stations'); end

  if(Nmask==1)
    stations_masks = stations_masks*ones(1,Nsta);
  end


  Mt = size(cjd , 'c');          // nombre de dates
  if(Mt~=1) then error('cjd must be of size 1'); end

  Morb = size(mean_kep , 'c');   // nombre de satellites
  if(Morb~=1) then error('mean_kep must of size (6x1)'); end


  // Extrapolation + conversion en position/vitesse
  pas = visi_min/86400.0;
  dates = simulation_period(1):pas:simulation_period(2);
  if(propagation_type =='lyd')
  [moy,osc] = CL_ex_lyddane(cjd,mean_kep,dates,er,mu,zonals);
  [pos_sat_G50,vit_sat_G50] = CL_oe_kep2car(osc);
  elseif(propagation_type =='eh')
    [pos,vel] = CL_oe_kep2car(mean_kep);
  mean_cir = CL_oe_car2cir(pos,vel);
    [moy_cir,osc_cir] = CL_ex_eckHech(cjd,mean_cir,dates,er,mu,zonals);
  [pos_sat_G50,vit_sat_G50] = CL_oe_cir2car(osc_cir);
  elseif(propagation_type =='j2')
    [osc] = CL_ex_secularJ2(cjd,mean_kep,dates,er,mu,zonals(2));
  [pos_sat_G50,vit_sat_G50] = CL_oe_kep2car(osc);
  else
  error('propagation_type unknown');
  end

  // Passage en cartesien terrestre :
  pos_sat_ter = CL_fr_G502ter(dates,pos_sat_G50);


  // stations elevations recalees par rapport au masque station
  [elev] = CL_gm_stationElevation(pos_sat_ter,stations,er,obla);
  elev =  elev' - stations_masks'*ones(1,size(elev,1));
  elev_max = max(elev,'r');

  ite_max = 20;


  // Debut de visi : changement de signe negatif vers positif
  ind = find(elev_max(1:$-1)<0 & elev_max(2:$) > 0);
  elev_temp = zeros(Nsta,length(ind));
  elev_max_temp = zeros(ind);
  ii = 1 : length(ind);
  t1 = dates(ind); // date avant chgt signe
  v1 = elev_max(ind); // elev avant chgt signe
  t2 = dates(ind+1); // date apres chgt signe
  v2 = elev_max(ind+1); // elev apres chgt signe
  t3 = zeros(ind); // date au zero
  n_ite = 0;
  while(ii~=[] & n_ite < ite_max)
     n_ite = n_ite+1;
   new_dates = (v2(ii).*t1(ii)-v1(ii).*t2(ii))./(v2(ii)-v1(ii)); // dates ou elev devrait valoir 0
   ecart = abs(t3(ii) - new_dates);
     t3(ii) = new_dates;
   ii = find(ecart > prec);       // convergence non achevee pour ces indices
   if( ii~=[])
     [moy,osc] = CL_ex_lyddane(cjd,mean_kep,t3(ii),er,mu,zonals);
     [pos_sat_G50,vit_sat_G50] = CL_oe_kep2car(osc);
     pos_sat_ter = CL_fr_G502ter(t3(ii),pos_sat_G50);
     elev_tmp = CL_gm_stationElevation(pos_sat_ter,stations,er,obla)
     elev_temp(:,ii) = elev_tmp' - stations_masks'*ones(1,size(elev_tmp,1));
     elev_max_temp(ii) = max(elev_temp(:,ii),'r');

     // le point t3 calcule est toujours negatif : on recalcule entre t3 et t2.
       jj = find(elev_max_temp(ii) < 0)
     t1(ii(jj)) = t3(ii(jj));
     v1(ii(jj)) = elev_max_temp(ii(jj));

     // le point t3 calcule est positif : on recalcule entre t1 et t3.
     jj = find(elev_max_temp(ii) > 0)
     t2(ii(jj)) = t3(ii(jj));
     v2(ii(jj)) = elev_max_temp(ii(jj));
    end
  end
  if(n_ite == ite_max)
    error('no convergence in CL_gm_visibility');
  end
  start_dates = t3;


  // Fin de visi : changement de signe positif vers negatif
  ind = find(elev_max(1:$-1)>0 & elev_max(2:$) < 0);
  elev_temp = zeros(Nsta,length(ind));
  elev_max_temp = zeros(ind);
  ii = 1 : length(ind);
  t1 = dates(ind); // date avant chgt signe
  v1 = elev_max(ind); // elev avant chgt signe
  t2 = dates(ind+1); // date apres chgt signe
  v2 = elev_max(ind+1); // elev apres chgt signe
  t3 = zeros(ind);
  n_ite = 0;
  while(ii~=[] & n_ite < ite_max)
     n_ite = n_ite+1;
   new_dates = (v2(ii).*t1(ii)-v1(ii).*t2(ii))./(v2(ii)-v1(ii)); // date ou elev devrait valoir 0
   ecart = abs(t3(ii) - new_dates);
     t3(ii) = new_dates;
   ii = find(ecart > prec);       // convergence non achevee pour ces indices
   if( ii~=[])
     [moy,osc] = CL_ex_lyddane(cjd,mean_kep,t3(ii),er,mu,zonals);
     [pos_sat_G50,vit_sat_G50] = CL_oe_kep2car(osc);
     pos_sat_ter = CL_fr_G502ter(t3(ii),pos_sat_G50);
     elev_tmp = CL_gm_stationElevation(pos_sat_ter,stations,er,obla)
     elev_temp(:,ii) = elev_tmp' - stations_masks'*ones(1,size(elev_tmp,1));
     elev_max_temp(ii) = max(elev_temp(:,ii),'r');

     // le point t3 calcule est toujours positif : on recalcule entre t3 et t2.
     jj = find(elev_max_temp(ii) > 0)
     t1(ii(jj)) = t3(ii(jj));
     v1(ii(jj)) = elev_max_temp(ii(jj));

     // le point t3 calcule est negatif : on recalcule entre t1 et t3.
     jj = find(elev_max_temp(ii) < 0)
     t2(ii(jj)) = t3(ii(jj));
     v2(ii(jj)) = elev_max_temp(ii(jj));
    end
  end
  if(n_ite == ite_max)
    error('no convergence in CL_gm_visibility');
  end
  end_dates = t3;

  // Tableau de sortie :
  visi_dates = zeros(2,length(start_dates));
  // on etait en visi des le debut :
  if(elev_max(1) > 0)
    visi_dates = zeros(2,length(start_dates)+1);
    visi_dates(1,:) = [simulation_period(1),start_dates];
  else
    visi_dates(1,:) = start_dates;
  end
  // on termine dans une visi :
  if(elev_max($) > 0)
    visi_dates(2,:) = [end_dates,simulation_period(2)];
  else
    visi_dates(2,:) = end_dates;
  end

  // Supprimer les visis de moins de visi_min secondes :
  ind = find(visi_dates(2,:)-visi_dates(1,:) > visi_min/86400.0);
  visi_dates = visi_dates(:,ind);

endfunction
