//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL  license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'http://www.cecill.info'.

function env = CL_3b_environment(bodies,Lpoint)
// Creates environment for 3-body problem functions
//
// Calling Sequence
// env = CL_3b_environment(bodies,Lpoint)
//
// Description
// <itemizedlist><listitem>
// Creates a tlist of type environment around given <emphasis role="bold">bodies</emphasis> and <emphasis role="bold">Lpoint</emphasis>.
// <para>Description of env fields:</para>
// <para>-->env.Lpoint: (string 'l1', 'l2' or 'l3') Libration point</para>
// <para>-->env.gammal: adimensional distance between the libration point and the closest primary M1-L1 or M1-L2 or M2-L3</para>
// <para>-->env.MU: constant mu: ratio of the mass of the biggest primary with the mass of the smallest primary</para>
// <para>-->env.omega: pulsation of the primaries around there center of mass G</para>
// <para>-->env.k:   </para>
// <para>-->env.gl: distance G-Lpoint (G: center of mass of the primaries)</para>
// <para>-->env.pas: integration step</para>
// <para>-->env.OMEGA: pulsation of the primaries around there center of mass G</para>
// <para>-->env.D: distance between the primaries</para>
// <para>-->env.c2: </para>
// <para>-->env.nu: </para>
// <para>-->env.invM: </para>
// <para>-->env.ESCAPEDIR: </para>
// <para>-->env.ESCAPEDIRNORM: </para>
// <para>-->env.bodies: same as the entry, 'S-EM' or 'E-M'</para>
// The stacksize is also put at 1.5E8 in order to perform the Manyfolds computation
// </listitem>
// <listitem>Last update : 29/1/2009</listitem>
// </itemizedlist>
//
// Parameters
// bodies: string defining bodies system. It can be 'S-EM' for Sun-EarthMoon or 'E-M' for Earth-Moon system
// Lpoint:string defining Libration point. It can be 'L1', 'L2' or 'L3'
// env: environment around a Libration point. Tlist of type environment
//
// See also
// CL_3b_halo
// CL_3b_lissajous
// CL_3b_manifolds
//
// Authors
// CNES - DCT/SB
//
// Examples
// envL1 = CL_3b_environment('S-EM','L1')
//
// See 'CelestLab > Demos > 3_BODY_PROBLEM > halo_orbit' for more examples
//

// Declarations:


// Code:

Lpoint = strsubst(Lpoint,'l','L');
stacksize('max');
select bodies
  case 'S-EM' //Sun-EarthMoon system
    gl1 = 0.9899860549;
    gl2 = 1.010075127;
    gl3 = -1.000001267;
    OMEGA = 0.0000001990986687;
    MU = 0.000003040357143;
    D = 149597871410;
    environment=bodies
  case 'E-M'  //Earth-Moon system
    gl1 = 0.8369155787;
    gl2 = 1.155681819;
    gl3 = -1.00506261;
    OMEGA = 0.0000026617;
    MU = 0.012150668;
    D = 384399060;
  else
    error('unknown bodies system');
end

select Lpoint
  case 'L1'
    gl = gl1;
    gammal = 1 - MU - gl;
    c2 = 1/(gammal^3)*(MU+(1-MU)*(gammal^3)/((1-gammal)^3));
  case 'L2'
    gl = gl2;
    gammal = gl - 1 + MU;
    c2 = 1/(gammal^3)*(MU+(1-MU)*(gammal^3)/((1+gammal)^3));
  case 'L3'
    gl = gl3;
    gammal = -gl - MU;
    c2 = 1/(gammal^3)*(1-MU+MU*(gammal^3)/((1+gammal)^3));
  else
    error('not valid Libration point');
end

lambda = sqrt(1/2*(-2+c2+sqrt(9*c2^2-8*c2)));
omega = sqrt(1/2*(2-c2+sqrt(9*c2^2-8*c2)));
nu = sqrt(c2);
c = (lambda^2-1-2*c2)/(2*lambda);
k = (omega^2+1+2*c2)/(2*omega);
b1 = c*lambda + k*omega;
b2 = c*omega - k*lambda;

//Linearisation du mouvement
//matrice des coordonnees x-gl, y, z, vx, vy, vz,(lignes)
//dans la base des vecteurs propres (colonnes):
//  A*e^(lambda*t), B*e^-(lambda*t), C*e^(i*omega*t), D*e^(-i*omega*t), E*e^(i*nu*t), F*e^(-i*nu*t)
M = [1         1         1         1        0      0;
     c        -c         %i*k     -%i*k     0      0;
     0         0         0         0        1      1;
     lambda   -lambda    %i*omega -%i*omega 0      0;
     lambda*c  lambda*c -omega*k  -omega*k  0      0;
     0         0         0         0        %i*nu -%i*nu];

//matrice des vecteurs propres (lignes):
//  A*e^(lambda*t), B*e^-(lambda*t), C*e^(i*omega*t), D*e^(-i*omega*t), E*e^(i*nu*t), F*e^(-i*nu*t)
//dans la base des coordonnees x-gl, y, z, vx, vy, vz, (colonnes): (inverse de la precedente en analytique)
invM = 1/2* [k*omega/b1   omega/b2     0 -k/b2     1/b1  0;
             k*omega/b1  -omega/b2     0  k/b2     1/b1  0;
             c*lambda/b1  %i*lambda/b2 0 -%i*c/b2 -1/b1  0;
             c*lambda/b1 -%i*lambda/b2 0  %i*c/b2 -1/b1  0;
             0            0            1  0        0    -%i/nu;
             0            0            1  0        0     %i/nu];

//vecteur escape_direction dans le plan vx, vy
ESCAPEDIR = real(invM(1,4:5).');
ESCAPEDIRNORM = 1/norm(ESCAPEDIR)*ESCAPEDIR;

//vecteur non_escape_direction dans le plan vx, vy
NESCAPEDIR = [1/b1;k/b2];
NESCAPEDIRNORM = 1/norm(NESCAPEDIR)*NESCAPEDIR;

//definition de la variable globale pas
//pas = 1.e-3;
pas = 1.e-2;  //pas integrateur_fix


//definition variable type environment
env = tlist(['environment','Lpoint','gammal','MU','omega','k','gl','pas','OMEGA','D','c2',..
'nu','invM','ESCAPEDIR','ESCAPEDIRNORM','bodies']);

env.Lpoint = Lpoint;
env.gammal = gammal;
env.MU = MU;
env.omega = omega;
env.k = k;
env.gl = gl;
env.pas = pas;
env.OMEGA = OMEGA;
env.D = D;
env.c2 = c2;
env.nu = nu;
env.invM = invM;
env.ESCAPEDIR = ESCAPEDIR;
env.ESCAPEDIRNORM = ESCAPEDIRNORM;
env.bodies=bodies
endfunction
