//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL  license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'http://www.cecill.info'.

function [orb,omegahalo] = CL_3b_halo(env,Az,sens,t)
// Halo orbit
//
// Calling Sequence
// [orb,omegahalo] = CL_3b_halo(env,Az,sens,date_init,ttot)
//
// Description
// <itemizedlist><listitem>
// Computes a halo orbit around the Lagrangian point defined in <emphasis role="bold">env.Lpoint</emphasis>
// </listitem>
// <listitem>Last update : 27/6/2008</listitem>
// </itemizedlist>
//
// Parameters
// env: environment around a Libration point. Tlist of type environment (see CL_3b_environment)
// Az: z amplitude at first order of the halo orbit (adimensional)
// sens: halo sens, 0 for North orientation Halos or 1 for South orientation Halos
// t: discretisation points of the final orbit where :
//  t(1) is the initial time and t($) is the final time
// orb: matrix 7xN. Stabilized orbit where each column contains a point on the trajectory (position, velocity, date) [x;y;z;vx;vy;vz;t]
// omegahalo: halo orbit pulsation
//
// Bibliography
// 1 Introduction au Probleme a Trois Corps et Dynamique Linearisee autour des Points de Lagrange, G. Collange, Note Technique CCT Mecanique Orbitale num.7, CNES 2006
// 2 Estimation numerique des varietes stables et instables des orbites quasi-periodiques de Lissajous autour des Points d'Euler (Lagrange L1, L2, L3), R. Alacevich, CNES septembre 2006
// 3 Rapport de Stage: Exploration Numerique d'orbites Homoclines et Heteroclines autour de L1 et L2 dans le probleme restreint trois corps, A. Martinez Maida, DCT/SB/MO 2007.0029301, CNES 4 septembre 2007
//
// See also
// CL_3b_environment
// CL_3b_lissajous
// CL_3b_manifolds
//
// Authors
// CNES - DCT/SB
//
// Examples
// env = CL_3b_environment('S-EM','l2');
// Az = 150e6/env.D;
// sens = 0;
// t=linspace(0,.9*%pi,50);
// [orb,omega] = CL_3b_halo(env,Az,sens,t);
//
//  //m It takes around 5 secs.
// See 'CelestLab > Demos > 3_BODY_PROBLEM > halo_orbit' for more examples
//

// Declarations:


// Code:

mprintf('Calculating Initial Conditions \n')


[X0,omegacorr] = CL_3b_condInitHalo(Az,sens,env);
//clf;plot(X0(1,:),X0(2,:),'g*')
mprintf('Fitting every point \n')
[correction,XXtt,omegahalo] = CL_3b_genHaloMat(X0,omegacorr,t(1),t($),env);
//plot(XXtt(1,:),XXtt(2,:),'r*')

mprintf('Formatting the orbit \n')
orb=CL_3b_formatOrbits(XXtt,t,env);
//plot(orb(1,:),orb(2,:),'b')
endfunction
