//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL  license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'http://www.cecill.info'.

function [orb,omega,nu] = CL_3b_lissajous(env,Ax,phix,Az,phiz,epsilon,t)
// Lissajous orbit
//
// Calling Sequence
// [orb,omega,nu] = CL_3b_lissajous(env,Ax,phix,Az,phiz,date_init,Rstab,Tstab,ttot)
//
// Description
// <itemizedlist><listitem>
// Computes a lissajous orbit around the Lagrangian point defined in <emphasis role="bold">env.Lpoint</emphasis>
// </listitem>
// <listitem>Last update : 17/1/2009</listitem>
// </itemizedlist>
//
// Parameters
// env: environment around a Lagrangian point. Tlist of type environment (see CL_3b_environment)
// Ax: amplitude in x (adimensional)
// phix: phase in x
// Az: amplitude in z (adimensional)
// phiz: phase in z
// date_init: initial date
// Epsilon: Degree of precision
// t: discretisation points of the final orbit where :
//  t(1) is the initial time and t($) is the final time
// orb: matrix 7xN. Stabilized orbit where each column contains a point on the trajectory (date, position and velocity) [x;y;z;vx;vy;vz;t]
//
// Bibliography
// 1 Introduction au Probleme a Trois Corps et Dynamique Linearisee autour des Points de Lagrange, G. Collange, Note Technique CCT Mecanique Orbitale num.7, CNES 2006
// 2 Estimation numerique des varietes stables et instables des orbites quasi-periodiques de Lissajous autour des Points d'Euler (Lagrange L1, L2, L3), R. Alacevich, CNES septembre 2006
// 3 Rapport de Stage: Exploration Numerique d'orbites Homoclines et Heteroclines autour de L1 et L2 dans le probleme restreint a trois corps, A. Martinez Maida, DCT/SB/MO 2007.0029301, CNES 4 septembre 2007
//
// See also
// CL_3b_environment
// CL_3b_halo
// CL_3b_lissajous
// CL_3b_manifolds
//
// Authors
// CNES - DCT/SB
//
// Examples
// env = CL_3b_environment('S-EM','l2');
// Ax = -30e6/env.D;
// phix = 0;
// Az = 100e6/env.D;
// phiz = 0;
// date_init = 0;
// epsilon=1e-10
// t = 7.6*%pi;
// orb = CL_3b_lissajous(env,Ax,phix,Az,phiz,epsilon,t);
//
// See 'CelestLab > Demos > 3_BODY_PROBLEM > lissajous_orbit' for more examples
//

// calcul des conditions initiales lineaires

// Declarations:


// Code:

mprintf('Calculating Initial Conditions \n')
points_per_orbit=5
//In order to avoid the first and last points with enough accuracy
//  we must add pi/omega at the beginning
t0=t(1)-%pi/env.omega
tf=t($)+%pi/env.omega
t_init=linspace(t0,tf,int(tf-t0/env.omega)*points_per_orbit);
[X0,omega,nu] = CL_3b_condInitLiss(Ax,phix,Az,phiz,env,t_init);

mprintf('Fitting every point \n')
[XXtt] = CL_3b_shooting(X0,t_init,env,epsilon);
//clf;plot(XXtt(1,:),XXtt(2,:),'r*')

mprintf('Formatting the orbit \n')
orb=CL_3b_formatOrbits(XXtt,t,env);
//plot(orb(1,:),orb(2,:),'g')
endfunction
