//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL  license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'http://www.cecill.info'.

function varargout = CL_3b_manifolds(env,orb,periode,epsilon,tint,pars)
// Manifolds (divergent and convergent) from Halo and Lissajous
//
// Calling Sequence
// manifolds = CL_3b_lissajous_manifold(env,lissajous_orb,epsilon,tint,pars)
// [manifold1,...,manifoldN] = CL_3b_lissajous_manifold(env,orb,epsilon,tint,pars)
//
// Description
// <itemizedlist><listitem>
// Computes manifolds around the Lagrangian point defined in <emphasis role="bold">env.Lpoint</emphasis> (variable <emphasis role="bold">env</emphasis> can be
// obtained as output of function CL_3b_environment).
// <para>This function accepts diferent number of outputs. Parameter <emphasis role="bold">pars</emphasis> is a string vector used to specifiy which
// outputs are computed. 'div' calculates divergent
// manifold using epsilon value while '-div' computes also divergent manifold but using -epsilon. In the same way,
// you can use 'conv' et '-conv' to compute convergent manifolds.</para>
// <para>Outputs will be in the same order as  in <emphasis role="bold">pars</emphasis>. If size of pars is NP and number of outputs demanded is
// NO, then: for NP==NO each output contains a manifold (matrix). For NO==1 and NP>1, single output is a list of manifolds
// (list of matrixs).</para>
// </listitem>
// <listitem>Last update : 21/1/2009</listitem>
// </itemizedlist>
//
// Parameters
// env: environment around a Lagrangian point. Tlist of type environment (see CL_3b_environment)
// orb: lissajous or halo orbit, matrix 7xN (see CL_3b_lissajous) [m,m/s,t]
// epsilon: epsilon In the literature it's said about 200 Km ~1e-9, but this method is accurate enough, we recommended 1e-5
// periode: in order to estimate the monodromy
// tint: integration time
// pars: vector of strings specifying manifolds to calculate. Possible values are 'div','-div','conv' or '-conv'.
// manifolds: matrix or list of matrixs
//
// Bibliography
// 1 Introduction au Probleme a Trois Corps et Dynamique Linearisee autour des Points de Lagrange, G. Collange, Note Technique CCT Mecanique Orbitale num.7, CNES 2006
// 2 Estimation numerique des varietes stables et instables des orbites quasi-periodiques de Lissajous autour des Points d'Euler (Lagrange L1, L2, L3), R. Alacevich, CNES septembre 2006
// 3 Rapport de Stage: Exploration Numerique d'orbites Homoclines et Heteroclines autour de L1 et L2 dans le probleme restreint a trois corps, A. Martinez Maida, DCT/SB/MO 2007.0029301, CNES 4 septembre 2007
//
// See also
// CL_3b_environment
// CL_3b_lissajous
// CL_3b_halo
//
// Authors
// CNES - DCT/SB
//
// Examples
//  // First, we must build an orbit
// env = CL_3b_environment('S-EM','L2');
// Az = 150e6/env.D;
// sens = 0;
// t=[0:0.01:0.99*%pi];
// [orb,omega] = CL_3b_halo(env,Az,sens,t);
//  // Then the manifolds itself
// epsilon=1e-5;
// tint =.4*%pi;
// [div_in,conv_in,conv_out,div_out] = CL_3b_manifolds(env,orb,omega,epsilon,tint,['div','conv','-conv','-div']);
// [div] = CL_3b_manifolds(env,orb,omega,epsilon,tint,['div']);

//
// divs = tb_manifold(env,lissajous,epsilon,tint,['div';'-div']);
//
// See 'CelestLab > Demos > 3_BODY_PROBLEM > lissajous_manifold' for more examples
//

// Declarations:


// Code:

mprintf('Convergent and divergent directions computation\n');

[orb_dir] = CL_3b_dirDirConv(orb,periode/2,env);
mprintf('Manifolds generation\n');


manifolds = list();
Np = size(pars,'*');

for i=1:Np

  par = pars(i);
  select par
    case 'div'  //calcul de la variete divergente avec epsilon
      manifold = CL_3b_traceManifolds(orb_dir,epsilon,tint,env.MU,0);
    case '-div' //calcul de la variete divergente avec -epsilon
      manifold = CL_3b_traceManifolds(orb_dir,-epsilon,tint,env.MU,0);
    case 'conv' //calcul de la variete convergente avec epsilon
      manifold = CL_3b_traceManifolds(orb_dir,epsilon,tint,env.MU,1);
    case '-conv'//calcul de la variete convergente avec -epsilon
      manifold = CL_3b_traceManifolds(orb_dir,-epsilon,tint,env.MU,1);
    else
      error('unknown parameter... i''m sorry!');
  end

  manifolds($+1) = manifold;

end

lhs = argn(1);

if lhs==1 & Np~=1
  varargout = list(manifolds);
elseif lhs==Np
  varargout = manifolds;
else
  error('bad size on input/output arguments');
end

endfunction
