//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL  license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'http://www.cecill.info'.

function [M] = CL_kp_E2M(e,E)
// Eccentric anomaly to mean anomaly for elliptic and hyperbolic orbits
//
// Calling Sequence
// M = CL_kp_E2M(e,E)
//
// Description
// <itemizedlist><listitem>
// Solves Kepler's equation E - e*sin(E) = M for the mean anomaly <emphasis role="bold">M</emphasis>,
// given the eccentricity <emphasis role="bold">e</emphasis> and the eccentric anomaly <emphasis role="bold">E</emphasis> for elliptic and hyperbolic orbits.
// </listitem>
// <listitem>Last update : 4/3/2008</listitem>
// </itemizedlist>
//
// Parameters
// E: eccentric anomaly [rad] (1xN)
// e: eccentricity (1xN)
// M: mean anomaly [rad] (1xN)
//
// Authors
// CNES - DCT/SB
//
// Bibliography
// 1 Orbital Mechanics for Engineering Students, H D Curtis, Chapter 3, equation 3.11
// 2 Mecanique spatiale, CNES - Cepadues 1995, Tome I, section 4.6.6, page 181
//
// See also
// CL_kp_M2Egene
// CL_kp_M2E
// CL_kp_E2v
// CL_kp_v2E
//
// Examples
// // Reciprocity test
// E=[1.7281 5.7281];
// e=[0.37255 0.55];
// M = CL_kp_E2M(e,E);
// E2 = CL_kp_M2E(e,M);

// Declarations:
global %CL_epsilon;

// Code:

Ne = size(e,2)
NE = size(E,2)
N = max(Ne,NE)
coherence = (NE==1) | (Ne==1) | (NE==Ne);
if coherence
  if Ne==1 then e=e*ones(1,N); end
  if NE==1 then E=E*ones(1,N); end
else
  error('bad size of input arguments');
end

epsilon = %CL_epsilon.parab

if find(e>=1-epsilon & e<1+epsilon)~=[] then error('limit parabole case'); end

M = zeros(E)

i_e = find(e<1-epsilon) //ellipse
  M(i_e) = E(i_e) - e(i_e).*sin(E(i_e))

i_h = find(~(e<1-epsilon))  //hyperbolic
  M(i_h) = e(i_h).*sinh(E(i_h)) - E(i_h)

endfunction


