//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL  license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'http://www.cecill.info'.

function [v] = CL_kp_E2v(e,E)
// Eccentric anomaly to true anomaly
//
// Calling Sequence
// v = CL_kp_E2v(e,E)
//
// Description
// <itemizedlist><listitem>
// Computes true anomaly given eccentricity and eccentric anomaly for elliptic and hyperbolic orbits.
// <para>NOTE: this function does not handle parabolic orbits (eccentricity around 1+-epsilon). </para>
// </listitem>
// <listitem>Last update : 4/3/2008</listitem>
// </itemizedlist>
//
// Parameters
// E: eccentric anomaly [rad] (1xN)
// e: eccentricity (1xN)
// v: true anomaly [rad] (1xN)
//
// Authors
// CNES - DCT/SB
//
// Bibliography
// 1 Orbital Mechanics for Engineering Students, H D Curtis, Chapter 3, equation 3.10a
// 2 Mecanique spatiale, CNES - Cepadues 1995, Tome I, section 4.6.6, page 179
//
// See also
// CL_kp_M2Egene
// CL_kp_M2E
// CL_kp_v2E
//
// Examples
// // Reciprocity test
// E=[3.4794 5.7281];
// e=[0.37255 0.55];
// v = CL_kp_E2v(e,E);
// E2 = CL_kp_v2E(e,v);

// Declarations:
global %CL_epsilon;

// Code:

NE = size(E,2)
Ne = size(e,2)
N = max(NE,Ne)
coherence = (NE==1) | (Ne==1) | (NE==Ne);
if coherence
  if NE==1 then E=E*ones(1,N); end
  if Ne==1 then e=e*ones(1,N); end
else
  error('bad size of input arguments');
end

epsilon = %CL_epsilon.parab

if find(e>=1-epsilon & e<1+epsilon)~=[] then error('limit parabole case'); end

v = zeros(E)

i_e = find(e<1-epsilon) //elliptic
if i_e~=[]
  betaa = e(i_e)./(1+sqrt(1-e(i_e).^2))
  v(i_e) = E(i_e) + 2*atan(betaa.*sin(E(i_e)),1-betaa.*cos(E(i_e)))
end

i_h = find(~(e<1-epsilon))  //hyperbolic
if i_h~=[]
  w = sqrt((e(i_h)+1)./(e(i_h)-1)).*tanh(E(i_h)./2)
  v(i_h) = 2.0.*atan(w)
end

endfunction


