//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL  license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'http://www.cecill.info'.

function E = CL_kp_M2E(ecc,M)
// Solution of kepler's equation (ecc,M to E)
//
// Calling Sequence
// E = CL_kp_M2E(ecc,M)
//
// Description
// <itemizedlist><listitem>
// Solves Kepler's equation for hyperoblic or elliptic orbits.
// <para>Given the eccentricity <emphasis role="bold">ecc</emphasis> and the mean anomaly <emphasis role="bold">M</emphasis>,
// this function computes the eccentric anomaly <emphasis role="bold">E</emphasis></para>
// </listitem>
// <listitem>Last update : 30/10/2007</listitem>
// </itemizedlist>
//
// Parameters
// M: mean anomaly [rad] (1xN)
// ecc: hyperbolic or elliptical eccentricity (ecc>1) (1xN)
// E: eccentric anomaly [rad] (1xN)
//
// Authors
// CNES - DCT/SB
//
// Bibliography
// 1 CNES - MSLIB FORTRAN 90, Volume V (mvi_kepler_hyperb)
//
// See also
// CL_kp_M2Egene
//
// Examples
// E = CL_kp_M2E(0.001,3*%pi/8)

//eccentricity control

// Declarations:
global %CL_epsilon;

// Code:

if (ecc < 0)
  error("Negative eccentricity");
end

  Nm = size(M,2)
  Ne = size(ecc,2)
  N = max(Nm,Ne)
  coherence = (Nm==1) | (Ne==1) | (Nm==Ne);
  if coherence
    if Nm==1 then M=M*ones(1,N); end
    if Ne==1 then ecc=ecc*ones(1,N); end
  else
    error('bad size of input arguments');
  end


// Hyperbole
  indH=(find(ecc >= 1))
  szH = size(indH)


if ( szH(2)> 0 ) then
  eH = ecc(indH)
  MH = M(indH)

  Mmin = 1            //valeur de test pour l'initialisation de l'anomalie moyenne
  emin = 1.05              //valeur de test pour l'initialisation de l'eccentricite
  nb_max_iter = 20         //nombre maximum d'iterations

  eps100 = 100*%eps      // constante eps pour les reels

  ii = find( (abs(MH)<=Mmin)&(eH <=emin) )
  ii2 = find( (abs(MH)<=Mmin)&(eH <=emin)&(MH<0) )
  jj = find( ~( (abs(MH)<=Mmin)&(eH <=emin) ) )
  e0 = zeros(eH)
  e0(ii) = (6.*abs(MH(ii))).^(1/3)
  e0(ii2) = -e0(ii2)
  e0(jj) = log( MH(jj)./eH(jj) + sqrt(MH(jj).^2./eH(jj)./eH(jj)+1) )

  // resolution iterative par methode de newton
  //    f(x) = 0
  //    xi+1 = xi - (f(xi) / f'(xi))
  // -------------------------------------------
  ei = e0
  //initialisation des criteres de convergence et du nombre d'iterations
  err1 = ones(eH)
  err2 = ones(eH)
  iter = zeros(eH)
  c = zeros(eH)
  eip1 = zeros(eH)
  kk = find( (err1>eps100) & (err2>eps100) & (iter~=nb_max_iter) )
  while kk~=[]

     iter(kk) = iter(kk) + 1
     c(kk) = tanh(ei(kk)./2)
     eip1(kk) = ei(kk) - ( (MH(kk)+ei(kk)).*(1-c(kk).^2) - 2*eH(kk).*c(kk) )./( 1-c(kk).^2 - eH(kk).*(1+c(kk).^2) )

     err1(kk) = abs(eip1(kk)-ei(kk))./max(abs(eip1(kk)),1)    //  calcul des criteres de convergence
     err2(kk) = eH(kk).*sinh(eip1(kk)) - eip1(kk) - MH(kk)
     err2(kk) = abs(err2(kk))./max(abs(MH(kk)),1)
     ei(kk) = eip1(kk)

     kk = find( (err1>eps100) & (err2>eps100) & (iter~=nb_max_iter) )
  end

  pp = find( (err1>eps100) & (err2>eps100) )
  if pp~=[] then error('no convergence after 20 iterations'); end //pas de convergence au bout de 20 iterations ->  arret

  E(indH) = ei

  end

// Ellipse

  indE=(find(ecc < 1))
      szE = size(indE)

if ( szE(2)> 0 ) then

  eE = ecc(indE)
  ME = M(indE)


  eps100 = 100*%eps  // constante eps for reals
  eps_cir = %CL_epsilon.cir
  eps_parab = %CL_epsilon.parab
  max_nb_iter=20

  EE = zeros(ME);

  //SPECIAL CASES
  //circular orbit
  nn = find(eE<eps_cir) //circular orbit
  EE(nn) = 1; //nonsense 1. At the end E(nn) = M(nn). This is to avoid division by zero.

  sinM = sin(ME)  //Mean anomaly sinus calculation
  cosM = cos(ME)  //Mean anomaly cosinus calculation

  //M is almost a pi multiple. M=M0+kpi*pi with M0 0 or almost 0
  kpi = zeros(ME);
  M0 = zeros(ME);
  kpi = round(ME./%pi);
  M0 = ME - kpi.*%pi;
  ii = find( (abs(sinM)<eps100)&(modulo(kpi,2) == 0) ); //determining kpi parity
  jj = find( (abs(sinM)<eps100)&(modulo(kpi,2) ~= 0) );
  sinM(ii) = 1; //nonsense 1. At the end we have E(ii) = kpi(ii).*%pi + M0(ii)./(1 - ecc(ii)). This is to avoid division by zero.
  sinM(jj) = 1; //nonsense 1. At the end we have E(jj) = kpi(jj).*%pi + M0(jj)./(1 + ecc(jj)). This is to avoid division by zero.

  //NOMINAL CASES
  pp = find( ~(abs(sinM)<eps100) )
  if pp~=[]
    //ENCKE INITIALISATION
    az = zeros(eE);
    rapport = (eE - cosM) ./ sinM
    az = eE ./ sqrt (1+(rapport.*rapport))

    pp = find(sinM < 0);
    az(pp) = -az(pp);
    az2 = zeros(az);
    az2 = az.*az;

    e0 = ME + az - ( az2.*az2.*cosM.*(1-eE.*eE)./(6.*sinM) )

    //resolution iterative par methode de newton
    //    f(x) = 0
    //    xi+1 = xi - (f(xi) / f'(xi))
    eip1 = e0
    iter = zeros(eE)          //Initialisations before while
    err1 = ones(eE)
    err2 = ones(eE)

    kk = find( (err1>eps100)&(err2>eps100)&(iter~=max_nb_iter) )
    while kk~=[]
    ei = eip1(kk)
    iter(kk) = iter(kk)+1
    eip1(kk) = ei - ((ME(kk) + eE(kk).*sin(ei) - ei)./(eE(kk).*cos(ei) - 1.0))   //current iteration calculation
    err1(kk) = abs(eip1(kk)-ei)./max(abs(eip1(kk)),1.0)   //residu calculation
    err2(kk) = abs(ME(kk) + eE(kk).*sin(eip1(kk)) - eip1(kk))./max(abs(ME(kk)),1.)
    kk = find( (err1>eps100)&(err2>eps100)&(iter~=max_nb_iter) )
    end

    hh = find((err1 > eps100) & (err2 > eps100))

    if hh~=[] //no convergence at the end of maximum number of iterations
    disp("no convergence at the end of maximum number of iterations (" + string(max_nb_iter) + ") at index "+string(hh));
    end
    EE = eip1;
  end

  //SPECIAL CASES
  EE(ii) = kpi(ii).*%pi + M0(ii)./(1 - eE(ii));
  EE(jj) = kpi(jj).*%pi + M0(jj)./(1 + eE(jj));
  //circular orbits
  EE(nn) = ME(nn);

  E(indE) = EE

 end

endfunction
