//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL  license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'http://www.cecill.info'.

function E = CL_kp_M2Egene(ex,ey,M)
// Solution of generalized kepler's equation (ex, ey, M to E)
//
// Calling Sequence
// E = CL_kp_M2Egene(ex,ey,M)
//
// Description
// <itemizedlist><listitem>
// This function computes eccentric anomaly <emphasis role="bold">E</emphasis> from mean anomaly <emphasis role="bold">M</emphasis>,
// solving generalized Kepler's equation. (given the eccentricity vector components <emphasis role="bold">ex</emphasis> and <emphasis role="bold">ey</emphasis>)
// </listitem>
// <listitem><para/><inlinemediaobject><imageobject><imagedata fileref="kep_gen.gif"/></imageobject></inlinemediaobject></listitem>
// <listitem>Last update : 24/05/2007</listitem>
// </itemizedlist>
//
// Parameters
// ex: x component of eccentricity vector (1xN)
// ey: y component of eccentricity vector (1xN)
// M: mean anomaly [rad] (1xN)
// E: eccentric anomaly [rad] (1xN)
//
// Authors
// CNES - DCT/SB
//
// Bibliography
// 1 CNES - MSLIB FORTRAN 90, Volume V (mv_kepler_gene)
//
// See also
// CL_kp_M2E
//
// Examples
// kep = [7078.e3 ; 0.001 ; 0.001 ; %pi/8 ; 0.1 ; 3*%pi/8];
// [pos_car,vel_car] = CL_oe_kep2car(kep);
// cir = CL_oe_car2cir(pos_car,vel_car);
// ex = cir(2);
// ey = cir(3);
// E = CL_kp_M2Egene(ex,ey,3*%pi/8)
//
// // Now using CL_kp_M2E :
// E = CL_kp_M2E(0.001,3*%pi/8) // results slightly differ because convergence criteria is not the same.

// Declarations:
global %CL_epsilon;

// Code:

Nex = size(ex,2)
Ney = size(ey,2)
NmeanAnom = size(M,2)
N = max(Nex,Ney,NmeanAnom)
coherence = (Nex==N|Nex==1) & (Ney==N|Ney==1) & (NmeanAnom==N|NmeanAnom==1)
if ~coherence then error('bad size of input arguments'); end
if N~=1
  if Nex==1 then ex=ex*ones(1,N); end
  if Ney==1 then ey=ey*ones(1,N); end
  if NmeanAnom==1 then M=M*ones(1,N); end
end

eps100 = 100 * %eps
nb_max_iterations = 20
eps_cir = %CL_epsilon.cir
eps_parab = %CL_epsilon.parab

//test sur l'eccentricite
e2 = (ex.^2) + (ey.^2)  //carre de l'eccentricite
eccen = sqrt(e2)        //eccentricite

ii_parhyper = find(eccen>1-eps_parab); //orbite parabolique ou hyperbolique
ii_circ = find(eccen < eps_cir);       //orbite circulaire
ex(ii_parhyper) = 0.5; //arbitrary value, at the end E substitued by M
ex(ii_circ) = 0.5;     //arbitrary value, at the end E substitued by M
ey(ii_parhyper) = 0.5; //arbitrary value, at the end E substitued by M
ey(ii_circ) = 0.5;     //arbitrary value, at the end E substitued by M
eccen(ii_parhyper) = 0.5; //arbitrary value, at the end E substitued by M
eccen(ii_circ) = 0.5;     //arbitrary value, at the end E substitued by M

//initialisation de encke
//!=======================
cos_meanAnom = cos (M)
sin_meanAnom = sin (M)

x = (ex .* cos_meanAnom) + (ey .* sin_meanAnom)
y = (ex .* sin_meanAnom) - (ey .* cos_meanAnom)

E = zeros(M)
alpha = zeros(ex)
anomalie_moyenne = zeros(ex)
kpi = zeros(ex)
anomalie_moyenne0 = zeros(ex)

ii = find(abs(y)./eccen <= eps100)
//y/e < ou = eps100 -> traitement special
//pas de code retour a tester en retour de mu_angle2 car ex et ey
//ne peuvent etre nuls en meme temps car a ce stade des calculs
//l'eccentricite est non nulle
alpha(ii) = CL_sc2angle(ex(ii),ey(ii))
anomalie_moyenne(ii) = M(ii) - alpha(ii)
kpi(ii) = round(anomalie_moyenne(ii)./%pi)
anomalie_moyenne0(ii) = anomalie_moyenne(ii) - kpi(ii).*%pi  //M est "presque" un multiple de pi: M = M0 + kpi.pi avec M0 nul ou proche de 0

ii1 = find( (modulo(kpi,2) == 0) & (abs(y)./eccen <= eps100) ) //determination de la parite de kpi
ii2 = find( (~(modulo(kpi,2) == 0)) & (abs(y)./eccen <= eps100) )
E(ii1) = alpha(ii1) + kpi(ii1).*%pi + anomalie_moyenne0(ii1)./(1 - eccen(ii1))
E(ii2) = alpha(ii2) + kpi(ii2).*%pi + anomalie_moyenne0(ii2)./(1 + eccen(ii2))

rapport = zeros(ex)
az = zeros(ex)
rapport = (e2 - x)./y
az = eccen./sqrt(1 + rapport.*rapport)

jj = find((y<0) & (~(abs(y)./eccen <= eps100)))
az(jj) = -az(jj)

az2 = az.*az
f0 = M + az - (az2.*az2.*x.*(1-e2))/(6.*y)


//resolution iterative par methode de newton
// f(x) = 0
// xi+1 = xi - (f(xi) / f'(xi))
//==============================================
fi = f0        //Initialisations avant iterations
err1= ones(ex)
err2= ones(ex)
nb_iterations=zeros(ex)

kk = find(((err1 > eps100) & (err2 > eps100) & (nb_iterations ~= nb_max_iterations)) & (~(abs(y)./eccen <= eps100)))

while kk~=[]

   fip1(kk) = fi(kk) - ((fi(kk) - (ex(kk).*sin(fi(kk))) + (ey(kk).*cos(fi(kk))) - M(kk)) ./ ...
   (1 - (ex(kk).*cos(fi(kk))) - (ey(kk).*sin(fi(kk)))))

   err1(kk) = abs(fip1(kk)-fi(kk))./max(abs(fip1(kk)),1) //calcul des criteres de convergence err1 et err2
   err2(kk) = M(kk) - fip1(kk) + ex(kk).*sin(fip1(kk)) - ey(kk).*cos(fip1(kk))
   err2(kk) = abs(err2(kk))./max(abs(M(kk)),1)

   fi(kk) = fip1(kk)

   nb_iterations(kk) = nb_iterations(kk) + 1 //incrementation du compteur d'iterations
   E(kk) = fi(kk)
   kk = find(((err1 > eps100) & (err2 > eps100) & (nb_iterations ~= nb_max_iterations)) & (~(abs(y)./eccen <= eps100)))

end

nn = find( (err1 > eps100) & (err2 > eps100) & ( nb_iterations~=0 ) )
if nn~=[] then error('no convergence after max # of iterations'); end

E(ii_parhyper) = M(ii_parhyper); //arbitrary value, E substitued by M
E(ii_circ) = M(ii_circ);     //arbitrary value, E substitued by M

endfunction

