//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL  license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'http://www.cecill.info'.

function [v] = CL_kp_M2v(ecc,M)
// True anomaly to mean anomaly
//
// Calling Sequence
// [v] = CL_kp_M2v(ecc,M)
//
// Description
// <itemizedlist><listitem>
// Computes true anomaly given mean anomaly and eccentricity, for elliptic and hyperbolic orbits.
// <para>NOTE: this function does not handle parabolic orbits (eccentricity around 1+-epsilon). </para>
// </listitem>
// <listitem>Last update : 4/3/2008</listitem>
// </itemizedlist>
//
// Parameters
// ecc: eccentricity (1xN)
// M: mean anomaly [rad] (1xN)
// v: true anomaly [rad] (1xN)
//
// Authors
// CNES - DCT/SB
//
// Bibliography
// 1 Orbital Mechanics for Engineering Students, H D Curtis, Chapter 3, equation 3.10a
// 2 Mecanique spatiale, CNES - Cepadues 1995, Tome I, section 4.6.6, page 179
//
// See also
// CL_kp_v2M
//
// Examples
// // Example
// ecc = 0.1;
// M = 0.2;
// v = CL_kp_M2v(ecc,M)


// Declarations:


// Code:

NM = size(M,2)
Necc = size(ecc,2)
N = max(NM,Necc)
coherence = (NM==1) | (Necc==1) | (NM==Necc);
if coherence
  if NM==1 then M=M*ones(1,N); end
  if Necc==1 then ecc=ecc*ones(1,N); end
else
  error('bad size of input arguments');
end

E = CL_kp_M2E(ecc,M);
v = CL_kp_E2v(ecc,E);

endfunction


