//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL  license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'http://www.cecill.info'.

function kep = CL_kp_characteristics(sma,ecc,v,er,mu)
// Various Keplerian characteristics
//
// Calling Sequence
// kep = CL_kp_characteristics(sma,ecc,v[,er[,mu]])
//
// Description
// <itemizedlist><listitem>
// This function computes many keplerian parameters for elliptic or hyperbolic orbits (parabolic orbits
// are not handled):
// <para>- orbit parameters: semi major axis, eccentricity, apoapsis radius, periapsis radius,
// apoapsis altitude, periapsis altitude, apoapsis velocity, periapsis velocity, energy,
// area law constant, period, semi minor axis, conic's p parameter, conic's c parameter (kinetic
// moment), mean motion, velocity at infinity on an hyperbola, distance from asymptote to
// focus for an hyperbola, apoapsis liberation velocity, periapsis liberation velocity,
// apoapsis liberation delta-v, periapsis liberation delta-v.</para>
// <para>- position on orbit parameters: true anomaly, mean anomaly, eccentric anomaly, time
// from periapsis, radius, x cartesian coordinate, y cartesian coordinate, velocity slope,
// velocity modulus.</para>
// <para>Parameters with no sense are given the value 0 (example: kep.period is 0 for hyperbolic orbits)</para>
// <para>All this parameters are given as output in the <emphasis role="bold">kep</emphasis> variable, which is a tlist.
// <emphasis role="bold">kep</emphasis> is defined as follows:</para>
// <para>--> kep.orb_type: type of orbit (1->elliptic, 2->hyperbolic)</para>
// <para>--> kep.sma: semi major axis [m]</para>
// <para>--> kep.ecc: eccentricity</para>
// <para>--> kep.ra: apoapsis radius [m]</para>
// <para>--> kep.rp: periapsis radius [m]</para>
// <para>--> kep.ha: apoapsis altitude (kep.ra - planet radius) [m]</para>
// <para>--> kep.hp: periapsis altitude (kep.rp - planet radius) [m]</para>
// <para>--> kep.va: apoapsis velocity modulus [m/s]</para>
// <para>--> kep.vp: periapsis velocity modulus [m/s]</para>
// <para>--> kep.vinf: velocity at infinity for an hyperbolic orbit (0 if elliptic orbit) [m/s]</para>
// <para>--> kep.period: orbit's period [s]</para>
// <para>--> kep.mm: orbit's mean motion [rad/s]</para>
// <para>--> kep.k: orbit's energy [J]</para>
// <para>--> kep.C_area: C constant for the areas law [m^2/s]</para>
// <para>--> kep.l_con: conic's l parameter (semi latus rectum)</para>
// <para>--> kep.c_con: conic's c parameter (linear eccentricity) [m]</para>
// <para>--> kep.smb: semi minor axis [m]</para>
// <para>--> kep.d_foy: distance asymptote-focus for an hyperbolic orbit (0 if elliptic orbit) [m]</para>
// <para>--> kep.vlp: periapsis liberation velocity modulus [m/s]</para>
// <para>--> kep.vla: apoapsis liberation velocity modulus [m/s]</para>
// <para>--> kep.dvp: delta-v for liberation at periapsis [m/s]</para>
// <para>--> kep.dva: delta-v for liberation at apoapsis [m/s]</para>
// <para>--> kep.v: true anomaly [rad]</para>
// <para>--> kep.M: mean anomaly [rad]</para>
// <para>--> kep.E: eccentric anomaly [rad]</para>
// <para>--> kep.anvinf: true anomaly at infinity (only for hyperbolic orbits, 0 if elliptic) [rad]</para>
// <para>--> kep.date: time from periapsis [s]</para>
// <para>--> kep.R: modulus of radius vector [m]</para>
// <para>--> kep.x: x cartesian coordinate [m]</para>
// <para>--> kep.y: y cartesian coordinate [m]</para>
// <para>--> kep.v_slope: velocity slope [rad]</para>
// <para>--> kep.vel: velocity modulus [m/s]</para>
// <para>NOTE: the linear eccentricity (kep.c_con) is the distance between the center and the focus
// (or one of the two foci). The latus rectum is the chord parallel to the directrix and passing
// through the focus (or one of the two foci). The semi latus rectum (kep.l_con) is half
// of the latus rectum.</para>
// </listitem>
// <listitem>Last update : 5/3/2008</listitem>
// </itemizedlist>
//
// Parameters
// sma: semi major axis [m] (1xN)
// ecc: eccentricity  (1xN)
// v: true anomaly [rad] (1xN)
// mu: (optional) geocentric gravitational constant [m^3/s^2] (default value is %CL_mu)
// er: (optional) astral radius [m] (default is earth radius %CL_eqRad)
// kep: tlist containing computed keplerian parameters. See description for units. Each field is (1xN)
//
// See also
// CL_kp_params
// CL_kp_E2v
// CL_kp_v2E
// CL_kp_E2M
// CL_kp_M2E
// CL_op_rarp2ae
// CL_op_rava2ae
// CL_op_rpvinf2ae
// CL_op_rpvp2ae
//
// Authors
// CNES - DCT/SB
//
// Examples
// sma = 7078.e3;
// ecc = 0.0001;
// v = %pi/4;
// kep = CL_kp_characteristics(sma,ecc,v)

// Declarations:
global %CL_eqRad %CL_mu;

// Code:

Nsma = size(sma,2)
Necc = size(ecc,2)
Nv = size(v,2)
N = max(Nsma,Necc,Nv)
coherence = (Nsma==1|Nsma==N) & (Necc==1|Necc==N) & (Nv==1|Nv==N)
if ~coherence then error('bad input sizes'); end
if N~=1
  if Nsma==1 then sma=sma*ones(1,N); end
  if Necc==1 then ecc=ecc*ones(1,N); end
  if Nv==1 then v=v*ones(1,N); end
end

if ~exists('mu','local') then mu=%CL_mu; end
if ~exists('er','local') then er=%CL_eqRad; end

//keplerian parameters
  orb_type = zeros(ecc)
  ih = find(ecc>1)
  ie = find(ecc<1)
  orb_type(ih) = 2 //hyperbolic orbit
  orb_type(ie) = 1 //elliptic orbit
  if find(ecc==1)~=[] then error('eccentricity=1 not handled'); end

  //calculation of principal and secondary kepler parameters
  [ra,rp,ha,hp,va,vp,nmoy,period,vinf,anvinf,dinf,vlibp,vliba,l_con,c_con,k,caire,smb,..
  anm,anex,temps,r,x,y,pen,vit] = CL_kp_keplerParams(sma,ecc,v,orb_type,er,mu)

  //calculation of velocity increments (apoapsis and periapsis) only for elliptic orbits
  delta_va = zeros(va)
  delta_vp = zeros(vp)
  delta_va(ie) = vliba(ie)-va(ie)
  delta_vp(ie) = vlibp(ie)-vp(ie)

  kep = tlist(['kep_par','orb_type','sma','ecc','ra','rp','ha','hp'..
  'va','vp','vinf','period','mm','k','C_area','l_con','c_con','smb','d_foy','vlp','vla','dvp','dva','v','M','E',..
  'anvinf','date','R','x','y','v_slope','vel']);
  kep.orb_type = orb_type
  kep.sma = sma
  kep.ecc = ecc
  kep.ra = ra
  kep.rp = rp
  kep.ha = ha
  kep.hp = hp
  kep.va = va
  kep.vp = vp
  kep.vinf = vinf
  kep.period = period
  kep.mm = nmoy
  kep.k = k
  kep.C_area = caire
  kep.l_con = l_con
  kep.c_con = c_con
  kep.smb = smb
  kep.d_foy = dinf
  kep.vlp = vlibp
  kep.vla = vliba
  kep.dvp = delta_vp
  kep.dva = delta_va
  kep.v = v
  kep.M = anm
  kep.E = anex
  kep.anvinf = anvinf
  kep.date = temps
  kep.R = r
  kep.x = x
  kep.y = y
  kep.v_slope = pen
  kep.vel = vit

endfunction
