//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL  license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'http://www.cecill.info'.

function [varargout] = CL_kp_params(type_output,sma,mu,om_earth)
// Orbit properties with a Keplerian motion (mean motion, period, longitude gap between 2 orbits)
//
// Calling Sequence
// [par] = CL_kp_params(type_output,sma,[mu,om_earth])
// [par1,par2] = CL_kp_params([type_output1,type_output2],sma,[mu,om_earth])
// [par1,par2,par3] = CL_kp_params([type_output1,type_output2 type_output3],sma,[mu,om_earth])
//
// Description
// <itemizedlist><listitem>
// This function computes orbit properties related to the mean Keplerian motion
// <para><emphasis role="bold">type_output</emphasis> can be : </para>
// <para>  - 'mm' for the mean motion : <inlinemediaobject><imageobject><imagedata fileref="meanmotion.gif"/></imageobject></inlinemediaobject></para>
// <para>  - 'per' for the period : <inlinemediaobject><imageobject><imagedata fileref="period.gif"/></imageobject></inlinemediaobject></para>
// <para>  - 'lgap' for the longitude gap between 2 orbits : <inlinemediaobject><imageobject><imagedata fileref="longitudegap.gif"/></imageobject></inlinemediaobject></para>
// <para>Note : type_output can be an array (example: ['mm' 'nodper'])</para>
// </listitem>
// <listitem>Last update : 15/10/2009</listitem>
// </itemizedlist>
//
// Parameters
// type_output : array containing outputs to be computed. Available outputs are : 'mm','per','lgap' (1 x Noutput)
// sma: semi major axis [m] (1xN)
// mu: (optional) geocentric gravitational constant [m^3/s^2] (default value is %CL_mu)
// om_earth: (optional) angular rotation speed of the Earth (default is %CL_omEarth)
// par: mean motion, period or longitude gap between 2 orbits [rad/s,s,rad] (1xN)
//
// Authors
// CNES - DCT/SB
//
// See also
// CL_op_paramsJ2
//
// Examples
// // Longitude gap between 2 consecutive orbits :
// sma = 7078.e3 ;
// [lgap] = CL_kp_params('lgap',sma)
//
// // Mean motion and period
// sma = 7078.e3 ;
// [mm,per] = CL_kp_params(['mm' 'per'],sma)

// Declarations:
global %CL_mu %CL_omEarth;

// Code:

if ~exists('mu','local') then mu=%CL_mu; end
if ~exists('om_earth','local') then om_earth=%CL_omEarth; end

varargout = list();

mm = (mu./(sma.^3)).^(0.5);
Noutput = max(size(type_output));
for k = 1 : Noutput
  if(type_output(k) == 'mm')
    varargout(k) = mm;
  elseif(type_output(k) == 'per')
    varargout(k) = 2*%pi ./ mm;
  elseif(type_output(k) == 'lgap')
    varargout(k) = 2*%pi .* (om_earth) ./ mm;
  else
    error('Unknown type of type_output');
  end
end

endfunction
