//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL  license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'http://www.cecill.info'.

function [E] = CL_kp_v2E(ecc,v)
// True anomaly to eccentric anomaly
//
// Calling Sequence
// E = CL_kp_v2E(ecc,v)
//
// Description
// <itemizedlist><listitem>
// Computes eccentric anomaly <emphasis role="bold">E</emphasis> given eccentricity <emphasis role="bold">ecc</emphasis>
// and true anomaly <emphasis role="bold">v</emphasis>for elliptic and hyperbolic orbits.
// </listitem>
// <listitem>Last update : 4/3/2007</listitem>
// </itemizedlist>
//
// Parameters
// v: true anomaly [rad] (1xN)
// ecc: eccentricity (1xN)
// E: eccentric anomaly [rad] (1xN)
//
// Authors
// CNES - DCT/SB
//
// Bibliography
// 1 Orbital Mechanics for Engineering Students, H D Curtis, Chapter 3, equation 3.10a
// 2 Mecanique spatiale, CNES - Cepadues 1995, Tome I, section 4.6.6, page 179
//
// See also
// CL_kp_M2Egene
// CL_kp_M2E
// CL_kp_E2v
//
// Examples
// // Reciprocity test
// v=[3.4794 5.7281];
// ecc=[0.37255 0.55];
// E = CL_kp_v2E(ecc,v);
// v2 = CL_kp_E2v(ecc,E);

// Declarations:
global %CL_epsilon;

// Code:

Nv = size(v,2)
Ne = size(ecc,2)
N = max(Nv,Ne)
if ~( (Nv==1|Nv==N) & (Ne==1|Ne==N) ) then error('bad size of input arguments'); end
if N~=1
  if Nv==1 then v=v*ones(1,N); end
  if Ne==1 then ecc=ecc*ones(1,N); end
end

epsilon = %CL_epsilon.parab  //limit parabole case

if find(ecc>=1-epsilon & ecc<1+epsilon)~=[] then error('limit parabole case'); end

E = zeros(v)

i_e = find(ecc<1-epsilon) //ellipse
if i_e~=[]

  betaa = ecc(i_e)./(1+sqrt(1-ecc(i_e).^2))
  E(i_e) = v(i_e) - 2*atan(betaa.*sin(v(i_e)),1+betaa.*cos(v(i_e)))

end

i_h = find(~(ecc<1-epsilon))  //hyperbole
if i_h~=[]

  aux = -1.0./ecc(i_h)
  v_h = v(i_h)

  if find(cos(v_h)<aux)~=[] then error('hyperbole: true anomaly at infinity'); end

  if find(abs(v_h)>=2*%pi)~=[] then error('hyperbole: true anomaly > 360deg'); end

  arg = sqrt((ecc(i_h)-1)./(ecc(i_h)+1)).*tan(v(i_h)./2)

  E(i_h) = log((arg+1)./(1-arg))

end

endfunction
