//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL  license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'http://www.cecill.info'.

function [drift_pom,drift_gom,drift_M,dpomdaei,dgomdaei,dMdaei] = CL_op_driftJ2(sma,ecc,inc,er,mu,j2)
// Drift of orbital elements due to J2
//
// Calling Sequence
// [drift_pom,drift_gom,drift_M[,dpomdaei[,dgomdaei[,dMdaei]]]] = CL_op_driftJ2(sma,ecc,inc[,er[,mu[,j2]]])
//
// Description
// <itemizedlist><listitem>
// Computes the 'secular drift' (delta_pom,delta_gom,delta_M) of keplerian elements (pom,gom,M) for the movement
// perturbed by the first zonal harmonic J2 and derivatives of these variations with respect to semi major axis,
// eccentricity and inclination.
// <para>Results are obtained computing the following expressions. On this formula, <emphasis role="bold">a</emphasis> is semi majors axis (sma),
// <emphasis role="bold">e</emphasis> is
// eccentricity (ecc) and <emphasis role="bold">i</emphasis> is inclination (inc). <emphasis role="bold">drpom</emphasis>, <emphasis role="bold">drgom</emphasis> and <emphasis role="bold">drM</emphasis>
// are, respectively, drift of the perigee
// argument, ascendant node longitude and mean anomaly due to j2. <emphasis role="bold">dpomdaei</emphasis>, <emphasis role="bold">dgomdaei</emphasis> and
// <emphasis role="bold">dMdaei</emphasis> are derivatives of, respectively,
// drpom, drgom and drM with respect to a, e and i. <emphasis role="bold">n</emphasis> is the mean motion.</para>
// </listitem>
// <listitem><para/><inlinemediaobject><imageobject><imagedata fileref="driftJ2.gif"/></imageobject></inlinemediaobject></listitem>
// <listitem>Last update : 15/5/2008</listitem>
// </itemizedlist>
//
// Parameters
// sma: semi major axis [m] (1xN)
// ecc: eccentricity  (1xN)
// inc: inclination [rad] (1xN)
// er: (optional) equatorial radius [m] (default is %CL_eqRad)
// mu: (optional) geocentric gravitational constant [m^3/s^2] (default value is %CL_mu)
// j2: (optional) geopotentiel elliptic factor (second zonal harmonic) (default is %CL_j2)
// drift_pom: drift of the perigee argument [rad/s] (1xN)
// drift_gom: drift of the ascendant node longitude [rad/s] (1xN)
// drift_M: drift of the mean anomaly [rad/s] (1xN)
// dpomdaei: derivative of drift_pom with respect to sma, ecc and inc (3xN)
// dgomdaei: derivative of drift_gom with respect to sma, ecc and inc (3xN)
// dMdaei: derivative of drift_M with respect to sma, ecc and inc (3xN)
//
// Bibliography
// 1 CNES - MSLIB FORTRAN 90, Volume E (me_deriv_secul_J2)
// 2 AM_varj2d.F (AMLIB) by O. Defonte, 14/2/1995 (Le mouvement du satellite, Conferences et exercices de Mecanique Spatiale, Toulouse 1983, Cepadues Editions)
//
// See also
//
// Authors
// CNES - DCT/SB
//
// Examples
// sma=[%CL_eqRad+350000;%CL_eqRad+700000];
// ecc=[0.001;0.001];
// inc=CL_deg2rad([51.6;51.6]);
// [drift_pom,drift_gom,drift_M]=CL_op_driftJ2(sma,ecc,inc)
//

// Declarations:
global %CL_eqRad %CL_j2 %CL_mu;

// Code:

lhs = argn(1)

Nsma = size(sma,2)
Necc = size(ecc,2)
Ninc = size(inc,2)

N = max(Nsma,Ninc,Necc)
coherence = (Nsma==N|Nsma==1) & (Necc==N|Necc==1) & (Ninc==N|Ninc==1)
if ~coherence then error('bad size of input arguments'); end
if N~=1
  if Nsma==1 then sma=sma*ones(1,N); end
  if Necc==1 then ecc=ecc*ones(1,N); end
  if Ninc==1 then inc=inc*ones(1,N); end
end

if ~exists('mu','local') then mu=%CL_mu; end
if ~exists('er','local') then er=%CL_eqRad; end
if ~exists('j2','local') then j2=%CL_j2; end

C20 = -j2

rcosi = cos(inc)
rmm = CL_kp_params('mm',sma,mu);
rtet2 = 1-ecc.^2
rtet = sqrt(rtet2)
rcosi2 = rcosi.^2

rc1 = -3*rmm.*C20.*er.^2
rc2 = 2*(sma.^2).*rtet2
rg2 = rc1./rc2

//secular drift of perigee argument
rc1 = rg2.*(5*rcosi2-1)
rc2 = 2*rtet2
drift_pom = rc1./rc2

//secular drift of ascendant node longitude
rc1 = -rg2.*rcosi
rc2 = rtet2
drift_gom = rc1./rc2

//secular drift of mean anomaly
rc1 = rg2.*(3*rcosi2-1)
rc2 = 2*rtet
drift_M = rmm + rc1./rc2

//derivatives
if lhs>3
  dpomdaei = zeros(3,N)
  dgomdaei = zeros(3,N)
  dMdaei = zeros(3,N)

  //derivatives with respect to semi major axis
  dpomdaei(1,:) = -3.5 * drift_pom ./ sma  //dpomdaei/dsma
  dgomdaei(1,:) = -3.5 * drift_gom ./ sma  //dgomdaei/dsma
  dMdaei(1,:) = (2 * rmm - 3.5 * drift_M) ./ sma  //dMdaei/dsma

  //derivatives with respect to eccentricity
  e_rtet2 = ecc ./ rtet2
  dpomdaei(2,:) = 4 * e_rtet2 .* drift_pom //dpomdaei/de
  dgomdaei(2,:) = 4 * e_rtet2 .* drift_gom //dgomdaei/de
  dMdaei(2,:) = 3 * e_rtet2 .* (drift_M - rmm) //dMdaei/de

  //derivatives with respect to inclination
  coef = (-0.75 * rmm .* j2 .* er.^2) ./ (rtet2.^2 .* sma.^2)
  sini = sin(inc)
  sin2i = 2 * rcosi .* sini
  dpomdaei(3,:) = 5 * coef .* sin2i
  dgomdaei(3,:) = -2 * coef .* sini
  dMdaei(3,:) = 3 * coef .* rtet .* sin2i
end

endfunction
