//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL  license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'http://www.cecill.info'.

function [par2] = CL_op_locTime(cjd,type_par1,par1,type_par2,ut1_utc)
// Local time or right ascension at date
//
// Calling Sequence
// [par2] = CL_op_locTime(cjd,type_par1,par1,type_par2,[ut1_utc])
//
// Description
// <itemizedlist><listitem>
// Given a date <emphasis role="bold">cjd</emphasis> and a parameter <emphasis role="bold">par1</emphasis> of type
// <emphasis role="bold">type_par1</emphasis>, this function computes <emphasis role="bold">par2</emphasis> of type <emphasis role="bold">type_par2</emphasis>.
// <para><emphasis role="bold">type_par1</emphasis> can be :</para>
// <para> - 'lon' for longitude (rad)</para>
// <para> - 'mlh' for mean local time (hours)</para>
// <para> - 'tlh' for true local time (hours)</para>
// <para> - 'ra' for right ascension (rad)</para>
// <para><emphasis role="bold">type_par2</emphasis> can be any of the above and also :</para>
// <para> - 'sidt' for sideral time (rad)</para>
// <para> - 'smra' for sun mean right ascension (rad)</para>
// <para> - 'smta' for sun true right ascension (rad)</para>
// <para> - 'all' for all of the above (in the same order)</para></listitem>
// <listitem>
// Mean local time corresponds to a mean 'sun' whose fictive position has a uniform movement on the equator
// (that is, true local hour corriged with time equation).
// <para>All calculations are computed in Gamma 50 Veis frame (see CL_fr_G502J2000Mat), and equation (1) is used.</para>
// <para>For a given direction, equations (2) to (4) are used (mha is mean angle and tha is true angle).</para>
// <para>To change from angle (h) to hour (hl), equation (5) is used.</para>
// <para>Sideral time is the same as the one computed in CL_mod_sidTimeG50</para>
// <inlinemediaobject><imageobject><imagedata fileref="localhour.gif"/></imageobject></inlinemediaobject></listitem>
// <listitem>Last update : 19/2/2008</listitem>
// </itemizedlist>
//
// Parameters
// cjd: CNES julian dates (1xN)
// type_par1: string defining par1. It can be 'lon', 'mlh', 'tlh', 'ra'.
// par1: longitude, mean local time (hours), true local time (hours) or right ascension [rad,decimal hour] (1xN)
// type_par2: string defining par2. It can be 'lon', 'mlh', 'tlh', 'ra', 'sidt', 'smra', 'smta' or 'all'
// par2: longitude, mean local time (hours), true local time (hours), right ascension, sideral time, sun mean right ascension or sun true right ascension or all (1xN or 7xN)
//
// Authors
// CNES - DCT/SB
//
// See also
// CL_fr_G502J2000Mat
// CL_mod_sidTimeG50
//
// Examples
// cjd = CL_dat_cal2cjd(2009,09,07,10,29,30.5); // September 7th 2009, 10h29min30.5s
// [mlh] = CL_op_locTime(cjd,'lon',CL_deg2rad(2),'mlh')  // mean local hour at longitude 2deg
// [tlh] = CL_op_locTime(cjd,'lon',CL_deg2rad(2),'tlh')  // true local hour at longitude 2deg
// [ra] = CL_op_locTime(cjd,'lon',CL_deg2rad(2),'ra')  // right ascension at longitude 2deg
// [tlh] = CL_op_locTime(cjd,'mlh',15.5,'tlh')  // mean local hour to true local hour
// [mlh] = CL_op_locTime(cjd,'tlh',15.5,'mlh')  // true local hour to mean local hour
// [smra] = CL_op_locTime(cjd,'mlh',15.5,'smra')  // sun's mean right ascension at mean local hour
// [all] = CL_op_locTime(cjd,'mlh',15.5,'all')  // all parameters at mean local hour


// Declarations:


// Code:

if ~exists('ut1_utc','local') then ut1_utc=zeros(cjd); end


[lhs,rhs]=argn();
if(rhs ~= 4) then error('this function requires 4 input arguments'); end

Ncjd = size(cjd,2)
Npar1 = size(par1,2)
N = max(Ncjd,Npar1)
coherence = (Ncjd==1|Ncjd==N)&(Npar1==1|Npar1==N)
if ~coherence then error('bad size of input arguments'); end
if N~=1
  if Ncjd==1 then cjd=cjd*ones(1,N); end
  if Npar1==1 then par1=par1*ones(1,N); end
end


//get sun cartesian coordinates in G50
[dir_sun,rs] = CL_mod_moonSunG50(cjd,'s')
//transformation from cartesian to shperic coordinates (asc,decl,r) in G50
sun_sph = CL_co_car2sph(rs.*.ones(3,1) .* dir_sun)
stra = sun_sph(1,:)

//calculation of sidereal time
cjd_ut1 = cjd + ut1_utc/86400.;
tsid = CL_mod_sidTimeG50(cjd_ut1)

//right ascension of mean sun in G50
smra = tsid + %pi - (cjd - int(cjd))*2*%pi

//calculation according type_par1
if type_par1=='lon'
  lon = pmodulo(par1,2*%pi)
  ra = pmodulo(tsid+par1,2*%pi)
  anghv = ra - stra
  anghm = ra - smra
  tlh = pmodulo((anghv+%pi)*12/%pi,24)
  mlh = pmodulo((anghm+%pi)*12/%pi,24)
elseif type_par1=='mlh'
  mlh = pmodulo(par1,24)
  anghm = (%pi/12)*par1 - %pi
  ra = pmodulo(anghm + smra,2*%pi)
  lon = pmodulo(ra - tsid,2*%pi)
  anghv = ra - stra
  tlh = pmodulo((anghv+%pi)*12/%pi,24)
elseif type_par1=='tlh'
  tlh = pmodulo(par1,24)
  anghv = (%pi/12)*par1 - %pi
  ra = pmodulo(anghv + stra,2*%pi)
  lon = pmodulo(ra - tsid,2*%pi)
  anghm = ra - smra
  mlh = pmodulo((anghm+%pi)*12/%pi,24)
elseif type_par1=='ra'
  ra = pmodulo(par1,2*%pi)
  lon = pmodulo(ra - tsid,2*%pi)
  anghv = ra - stra
  anghm = ra - smra
  tlh = pmodulo((anghv+%pi)*12/%pi,24)
  mlh = pmodulo((anghm+%pi)*12/%pi,24)
else
  error('unknown type_par1 value')
end

tsid = pmodulo(tsid,2*%pi)
stra = pmodulo(stra,2*%pi)
smra = pmodulo(smra,2*%pi)

//output according type_par2
if type_par2=='lon'
  par2=lon;
elseif type_par2=='mlh'
  par2=mlh;
elseif type_par2=='tlh'
  par2=tlh;
elseif type_par2=='ra'
  par2=ra;
elseif type_par2=='sidt'
  par2=tsid;
elseif type_par2=='smra'
  par2=smra;
elseif type_par2=='stra'
  par2=stra;
elseif type_par2=='all'
  par2=[lon;mlh;tlh;ra;tsid;smra;stra];
else
  error('unknown type_par2 value')
end
endfunction
