//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL  license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'http://www.cecill.info'.

function [varargout] = CL_op_paramsJ2(type_output,sma,ecc,inc,er,mu,j2,om_earth)
// Orbit properties including J2 effects (mean motion, period, nodal period, longitude gap between 2 orbits)
//
// Calling Sequence
// [par] = CL_op_paramsJ2(type_output,sma,ecc,inc,[er,mu,j2,om_earth])
// [par1,par2] = CL_op_paramsJ2([type_output1,type_output2],sma,ecc,inc,[er,mu,j2,om_earth])
// [par1,par2,par3] = CL_op_paramsJ2([type_output1,type_output2 type_output3],sma,ecc,inc,[er,mu,j2,om_earth])
//
// Description
// <itemizedlist><listitem>
// This function computes orbit properties related to the mean motion, taking account of the effects of J2
// <para><emphasis role="bold">type_output</emphasis> can be : </para>
// <para>  - 'mm' for the mean motion : <inlinemediaobject><imageobject><imagedata fileref="meanmotionJ2.gif"/></imageobject></inlinemediaobject></para>
// <para>  - 'per' for the period : <inlinemediaobject><imageobject><imagedata fileref="periodJ2.gif"/></imageobject></inlinemediaobject></para>
// <para>  - 'nodper' for the nodal period : <inlinemediaobject><imageobject><imagedata fileref="nodalperiodJ2.gif"/></imageobject></inlinemediaobject></para>
// <para>  - 'lgap' for the longitude gap between 2 orbits : <inlinemediaobject><imageobject><imagedata fileref="longitudegapJ2.gif"/></imageobject></inlinemediaobject></para>
// <para>Note : type_output can be an array (example: ['mm' 'nodper'])</para>
// </listitem>
// <listitem>Last update : 15/10/2009</listitem>
// </itemizedlist>
//
// Parameters
// type_output : array containing outputs to be computed. Available outputs are : 'mm','per','nodper','lgap' (1 x Noutput)
// sma: semi major axis [m] (1xN)
// ecc: eccentricity  (1xN)
// inc: inclination [rad] (1xN)
// er: (optional) equatorial radius [m] (default is %CL_eqRad)
// mu: (optional) geocentric gravitational constant [m^3/s^2] (default value is %CL_mu)
// j2: (optional) zonal coefficient (second zonal harmonic) (default is %CL_j2)
// om_earth: (optional) angular rotation speed of the Earth (default is %CL_omEarth)
// par: mean motion, period, nodal period or longitude gap between 2 orbits [rad/s,s,s,rad] (1xN)
//
// Authors
// CNES - DCT/SB
//
// See also
// CL_kp_params
//
// Examples
// // Longitude gap between 2 consecutive orbits :
// sma = 7078.e3 ;
// ecc = 0.001 ;
// inc = CL_deg2rad(98);
// [lgap] = CL_op_paramsJ2('lgap',sma,ecc,inc)
//
// // Mean motion and nodal period
// sma = 7078.e3 ;
// ecc = 0.001 ;
// inc = CL_deg2rad(98);
// [mm,nodper] = CL_op_paramsJ2(['mm' 'nodper'],sma,ecc,inc)

// Declarations:
global %CL_eqRad %CL_j2 %CL_mu %CL_omEarth;

// Code:

if ~exists('er','local') then er=%CL_eqRad; end
if ~exists('mu','local') then mu=%CL_mu; end
if ~exists('j2','local') then j2=%CL_j2;end
if ~exists('om_earth','local') then om_earth=%CL_omEarth; end

varargout = list();

[dpom,dgom,dM] = CL_op_driftJ2(sma,ecc,inc,er,mu,j2);
Noutput = max(size(type_output));
for k = 1 : Noutput
  if(type_output(k) == 'mm')
    varargout(k) = dpom + dM;
  elseif(type_output(k) == 'per')
    varargout(k) = 2*%pi ./ dM;
  elseif(type_output(k) == 'nodper')
    varargout(k) = 2*%pi ./ (dpom +dM);
  elseif(type_output(k) == 'lgap')
    varargout(k) = 2*%pi .* (om_earth-dgom) ./ (dpom +dM);
  else
    error('Unknown type of type_output');
  end
end

endfunction
