//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL  license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'http://www.cecill.info'.

function [sma,inc] = CL_op_repeat2smaInc(N,P,Q,ecc,sso,incInput,er,mu,j2,om_earth,om_sun)
// Repeat orbit parameters (N,P,Q) to semi major axis and inclination
//
// Calling Sequence
// [sma,inc] = CL_op_repeat2smaInc(N,P,Q,ecc,sso,[incInput],[er,[mu,[j2,[om_earth,[om_sun]]]]])
//
// Parameters
// N: Values of N (1 x No)
// P: Values of P (1 x No)
// Q: Values of Q (1 x No)
// ecc: eccentricity (1 x No)
// sso: 1 = search for sun synchronous orbits, 0 = orbits have a given inclination (1 x No)
// incInput : (mandatory in case sso=0, optional otherwise) given inclinations (rad) (1 x No)
// er: (optional) equatorial radius [m] (default is %CL_eqRad)
// mu: (optional) geocentric gravitational constant [m^3/s^2] (default value is %CL_mu)
// j2: (optional) zonal coefficient (second zonal harmonic) (default is %CL_j2)
// om_earth : (optional) angular rotation speed of the Earth (default is om_earth) (1 x 1)
// om_sun : (optional) mean apparent revolution speed of the Sun around the Earth (default is %CL_omSun) (1 x 1)
// sma: semi majors axis of orbits (m) (1 x No)
// inc: inclinations of orbits (rad) (1 x No)
//
// Description
// <itemizedlist><listitem>
// This function computes (including J2 secular effects) semi major axes of repeat(or phased) orbits of given N,P,Q and eccentricity.
// <para>An orbit is considered phased if after an even number of planet revolutions (with respect to orbit's node),
// the satellite has done an even number of orbits(exactly N+P*Q) and is back on the same ground tracks</para>
// <para>N+P/Q is the fractionary number of orbits per orbital day</para>
// <para>An orbital day is defined as the time between 2 consecutive crossings of a same meridian by
// the orbit ascending node direction. For a sun synchronous orbit, an orbital day is a solar day.</para>
// <para> If sso = 1 then the function computes sun synchronous orbits (incInput is omitted)</para>
// <para> If sso = 0 then the function uses incInput as inclinations for the orbits</para>
// </listitem></itemizedlist>
//
// Authors
// CNES - DCT/SB
//
// See also
// CL_op_repeatGroundTracks
// CL_op_searchRepeatOrbits
// CL_op_ssoJ2
//
// Examples
// N = 15;
// P = 3;
// Q = 16;
// ecc = 0.01;
// sso = 0;
// incInput = CL_deg2rad(98);
// [sma,inc] = CL_op_repeat2smaInc(N,P,Q,ecc,sso,incInput)
//
// N = [15,15];
// P = [3,6];
// Q = [16,6];
// ecc = [0.01 , 0.00001];
// sso = [1 , 1];
// [sma,inc] = CL_op_repeat2smaInc(N,P,Q,ecc,sso)



  function [res,derivee] = fonctionACalculer(sma,ecc,sso,inc,er,mu,j2,om_earth,om_sun)
    // Si on est heliosynchrone
    if(sso == 1)
    [pomp, gomp, anmp, dpomp,dgomp,danmp] = CL_op_driftJ2(sma,ecc,inc,er,mu,j2);
    res = (pomp + anmp) ./ (om_earth - om_sun);
    // la derivee vaut : df/da + df/di * di/da; avec di/da calcule avec l'heliosynchronisme (cosi = K(e) * a^7/2 )
    derivee = (dpomp(1) + danmp(1) + (dpomp(3) + danmp(3)) * (-7/2) ./ (tan(inc) .* sma)) ./ (om_earth - om_sun);

  // Si on est pas heliosynchrone
  else
    [pomp, gomp, anmp, dpomp,dgomp,danmp] = CL_op_driftJ2(sma,ecc,inc,er,mu,j2);
      res = (pomp + anmp) ./ (om_earth - gomp);
      derivee = ((dpomp(1) + danmp(1)) .* (om_earth-gomp) + (anmp+gomp).*dgomp(1)) ./ (om_earth-gomp).^2;
    end
  endfunction

// Declarations:
global %CL_eqRad;
global %CL_mu;
global %CL_j2;
global %CL_omEarth;
global %CL_omSun;

// Code:  
  
  
if ~exists('om_earth','local') then om_earth=%CL_omEarth; end
if ~exists('om_sun','local') then om_sun=%CL_omSun; end
if ~exists('er','local') then er=%CL_eqRad; end
if ~exists('mu','local') then mu=%CL_mu; end
if ~exists('j2','local') then j2=%CL_j2; end


  // Valeur de reference a atteindre
  fref = (N + P./Q);
  f = zeros(1,length(N));
  fprime = zeros(1,length(N));
  fconv = zeros(1,length(N));
  delta_sma = zeros(1,length(N));

  // Resultat approche
  nmoy = fref .* om_earth;
  sma = (mu./nmoy.^2).^(1/3);

  // Verification des entrees
  if(sso ==0)
    if ~exists('incInput','local') then
    error('You must give incInput when sso = 0');
    end
    inc = incInput;
  end



  // Critere de convergence (ecart de longitude entre la premiere et la derniere trace)
  iterMax = 20;
  iter = 1;
  precision = 1e-12;
  fconv_ref = 2*%pi*Q ./ (N + P./Q);
  fconv = 0;
  ind = 1 : length(N);

  // Iterations (tant qu'il reste un indice qui n'a pas converge, ou qu'on a atteint le nombre d'iterations max)
  while(~isempty(ind) & iter < iterMax)
   if(sso == 1)
      inc(ind) = CL_op_ssoJ2('i',sma(ind),ecc(ind),er,mu,j2,om_sun);
   end
   [f(ind),fprime(ind)] = fonctionACalculer(sma(ind),ecc(ind),sso,inc(ind),er,mu,j2,om_earth,om_sun);

   delta_sma(ind) = (fref(ind) - f(ind))./fprime(ind);
   sma(ind) = sma(ind) + delta_sma(ind);
   fconv(ind) = 2*%pi*Q(ind) ./ f(ind);
   iter = iter+1;
   ind = find( abs(fconv_ref - fconv) > precision);
  end

  if(iter >= iterMax)
  error('CL_op_repeat2smaInc : No convergence (too many iterations)');
  end


endfunction
