//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL  license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'http://www.cecill.info'.

function [params]=CL_op_searchRepeatOrbits(smaMin,smaMax,QMin,QMax,ecc,sso,incInput,er,mu,j2,om_earth,om_sun)
// Search for repeat orbits
//
// Calling Sequence
// [params]=CL_op_searchRepeatOrbits(smaMin,smaMax,QMin,QMax,ecc,sso,incInput,[er,[mu,[j2,[om_earth,[om_sun]]]]])
//
// Description
// <itemizedlist><listitem>
// This function computes all repeat (or phased) orbits for a given range of semi major axes and number of earth revolutions per cycle.
// <para>An orbit is considered phased if after an given number of planet revolutions (with respect to the orbit's node),
// the satellite has done a whole number of orbits and is back on the same ground track.</para>
// <para>Each solution is defined by 3 integers (N,P,Q) such that: </para>
// <para>N+P/Q is the (fractionary) number of orbits per orbital day, where an orbital day is defined as the time
// between 2 consecutive crossings of the ascending node by a same meridian. Q is the number of orbital days per
// cycle. N*Q+P is the number of orbit periods per cycle. </para>
// <para> For a sun synchronous orbit, an orbital day is the same as a usual day (86400 s).</para>
// <para> If sso = 1 then the computed orbits are sun-synchronous orbits (inclination is defined implicitly).</para>
// <para> If sso = 0 then the computed orbits have the inclination as given.</para></listitem>
// <listitem><para><emphasis role="bold">params</emphasis> is a Mx11 matrix (M number of solutions).</para>
// <para>--> params(:,1): semi major axis [m]</para>
// <para>--> params(:,2): eccentricity</para>
// <para>--> params(:,3): inclination [rad]</para>
// <para>--> params(:,4:6): N, P, Q </para>
// <para>--> params(:,7): number of nodal periods per phasing cycle (=N*Q+P)</para>
// <para>--> params(:,8): nodal period [s]</para>
// <para>--> params(:,9): cycle duration in days [days]</para>
// <para>--> params(:,10): longitude gap between 2 closest ascending nodes [rad] = 2*%pi/(N*Q+P)</para>
// <para>--> params(:,11): longitude gap after one cycle modulo 2*pi [rad] (for checking)</para>
// </listitem></itemizedlist>
//
// Parameters
// smaMin: Minimum semi major axis (m) (1 x 1)
// smaMax: Maximum semi major axis  (m) (1 x 1)
// QMin: Minimum number of orbital days per repeat cycle (integer) (1 x 1)
// QMax: Maximum number of orbital days per repeat cycle (integer) (1 x 1)
// ecc : eccentricity (1 x 1)
// sso: 1 = search for sun-synchronous orbits, 0 = search for orbits with imposed inclination (1 x 1)
// incInput : (optionnal; mandatory if sso=0) inclination (rad) (1 x 1)
// er: (optional) equatorial radius [m] (default is %CL_eqRad)
// mu: (optional) gravitational constant [m^3/s^2] (default value is %CL_mu)
// j2: (optional) zonal coefficient (second zonal harmonic) (default is %CL_j2)
// om_earth : (optional) angular rotation rate of the Earth (default is %CL_om_earth) (1 x 1)
// om_sun : (optional) mean apparent revolution speed of the Sun around the Earth (default is %CL_omSun) (1 x 1)
// params : results (See description) (Mx11) with M : number of solutions
//
// Authors
// CNES - DCT/SB
//
// See also
// CL_op_repeatGroundTracks
// CL_op_repeat2smaInc
// CL_op_ssoJ2
//
// Examples
// smaMin = 7000.0 * 1000;
// smaMax = 7050.0 * 1000;
// QMin = 2;
// QMax = 6;
// ecc = 0.0001;
// sso = 1;
// [params]=CL_op_searchRepeatOrbits(smaMin,smaMax,QMin,QMax,ecc,sso)
//
// sso = 0;
// incInput = CL_deg2rad(98.5);
// [params]=CL_op_searchRepeatOrbits(smaMin,smaMax,QMin,QMax,ecc,sso,incInput)

  function [N,P,Q]=CL_findNPQ(xNmin,xNmax,Qmin,Qmax)
  // Recherche de couples N,P,Q (N et Q dans des intervalles bornes)
  //
  // Calling Sequence
  // [N,P,Q]=CL_findNPQ(xNmin,xNmax,Qmin,Qmax)
  //
  // Parameters
  // xNmin: Nombre d'orbite pas jour minimum (reel)
  // xNmax: Nombre d'orbite pas jour maximum (reel)
  // Qmin: Nombre de jours de phasage minimum (entier)
  // Qmax: Nombre de jours de phasage maximum (entier)
  // N : valeurs de N (1 x Nt)
  // P : valeurs de P (1 x Nt)
  // Q : valeurs de Q (1 x Nt)
  //
  // Description
  // <itemizedlist><listitem>
  // Cette fonction renvoie toutes les combinaisons de N, P et Q qui respectent les conditions :
  // <para> xNmin &lt; N+P/Q &lt; xNmax </para>
  // <para> Qmin &lt; Q &lt; Qmax </para>
  // <para> P et Q premiers entre eux (le reste de la division de Q par P est different de 0)</para>
  // <para> 0 =&lt; P &lt; Q  (P=0 n'est sorti que si Q=1)</para>
  // <para> Note : la taille des vecteurs N,P,Q en sortie n'est pas connue a l'avance.</para>
  // </listitem></itemizedlist>
  //
  // Authors
  // A. LAMY - CNES - DCT/SB/MS
  // G. AZEMA - Thales Services pour le CNES - DCT/SB/MS
  //
  // See also
  //
  // Examples
  // xNmin = 14.25;
  // xNmax = 15.75;
  // Qmin = 3;
  // Qmax = 6;
  // [N,P,Q]=CL_findNPQ(xNmin,xNmax,Qmin,Qmax)

    Nmin = floor(xNmin);
    Nmax = floor(xNmax);

    // Recherche des couples P,Q tels que P et Q soient premiers entre eux :
    Ptrouve=[];
    Qtrouve=[];
    for Qcourant = Qmin:Qmax
    // Cas particulier : couple P=0, Q=1
    if(Qcourant == 1)
      Ptrouve=[Ptrouve,0];
      Qtrouve=[Qtrouve,1];
    elseif(Qcourant == 2)
      Ptrouve=[Ptrouve,1];
      Qtrouve=[Qtrouve,2];
    else
      // Cas particulier : couple P=1, Qcourant>=3
      Ptrouve=[Ptrouve,1];
      Qtrouve=[Qtrouve,Qcourant];

      // Cas normaux : on garde les couples P et Q premiers entre eux (i.e leur pgcd vaut 1)
      vectP = 2:Qcourant-1;
      pgcd = CL_colGcd(Qcourant*ones(1,size(vectP,'c')),vectP);
      indices = find(pgcd == 1);
      Ptrouve=[Ptrouve,vectP(indices)];
      Qtrouve=[Qtrouve,Qcourant*ones(1,size(vectP(indices),'c'))];
    end
    end

    // On reconstruit l'ensemble des couples N,P,Q
    Ncouple=[];
    Pcouple=[];
    Qcouple=[];
    for N=Nmin:Nmax
    Ncouple=[Ncouple,N*ones(1,size(Ptrouve,'c'))];
    Pcouple=[Pcouple,Ptrouve];
    Qcouple=[Qcouple,Qtrouve];
    end

    // On filtre les couples N,P,Q qui ne respectent pas xNmin < N+P/Q < xNmax
    indices=find(Ncouple+Pcouple./Qcouple < xNmax & Ncouple+Pcouple./Qcouple > xNmin);
    N=zeros(size(indices));
    P=zeros(size(indices));
    Q=zeros(size(indices));
    N=Ncouple(indices);
    P=Pcouple(indices);
    Q=Qcouple(indices);

  endfunction

// Declarations:
global %CL_eqRad;
global %CL_mu;
global %CL_j2;
global %CL_omEarth;
global %CL_omSun;

// Code:

  // Verification des entrees
  if(sso ==0)
    if ~exists('incInput','local') then
      error('You must give incInput when sso = 0');
    else
      incInputMin = incInput;
        incInputMax = incInput;
    end
  elseif(sso == 1)
    incInputMin = CL_op_ssoJ2('i',smaMin,ecc);
    incInputMax = CL_op_ssoJ2('i',smaMax,ecc);
  end

  if ~exists('om_earth','local') then om_earth=%CL_omEarth; end
  if ~exists('om_sun','local') then om_sun=%CL_omSun; end
  if ~exists('er','local') then er=%CL_eqRad; end
  if ~exists('mu','local') then mu=%CL_mu; end
  if ~exists('j2','local') then j2=%CL_j2; end

  // On associe un xNmin, xNmax a smaMin smaMax
  [pomp, gompMin, anmp] = CL_op_driftJ2(smaMin,ecc,incInputMin,er,mu,j2);
  [pomp, gompMax, anmp] = CL_op_driftJ2(smaMax,ecc,incInputMax,er,mu,j2);
  durJourOrbMin = 2*%pi / (om_earth - gompMin) ;
  durJourOrbMax = 2*%pi / (om_earth - gompMax) ;
  xNmax = durJourOrbMin /CL_op_paramsJ2('nodper',smaMin,ecc,incInputMin,er,mu,j2);
  xNmin = durJourOrbMax /CL_op_paramsJ2('nodper',smaMax,ecc,incInputMax,er,mu,j2);


  // On recupere l'ensemble des couples N,P,Q
  [N,P,Q]=CL_findNPQ(xNmin,xNmax,QMin,QMax);

  // Dupliquer les entrees aux nombre de couples N P Q
  ecc_vect = ecc * ones(1,size(N,'c'));
  sso_vect = sso * ones(1,size(N,'c'));
  inc_vect = incInputMin * ones(1,size(N,'c'));

  // On calcule pour chacun des couples le sma et l'inclinaison associee
  [sma,inc] = CL_op_repeat2smaInc(N,P,Q,ecc_vect,sso_vect,inc_vect,er,mu,j2,om_earth,om_sun);

  ecc = ecc * ones(1,length(sma));

  // Calcul des autres resultats
  nombreOrbites = N.*Q + P;
  periodeNodale = CL_op_paramsJ2('nodper',sma,ecc,inc,er,mu,j2);
  ecart2TraceVoisines = CL_rad2deg(2*%pi./nombreOrbites);
  dureeReelle = nombreOrbites.*periodeNodale;
  [pomp, gomp, anmp] = CL_op_driftJ2(sma,ecc,inc,er,mu,j2);
  longitudeGapApresUnCycle = CL_rMod(dureeReelle .* (om_earth - gomp),-%pi,%pi);

  params = [sma;ecc;inc;N;P;Q;nombreOrbites;periodeNodale;dureeReelle/86400.0;ecart2TraceVoisines;longitudeGapApresUnCycle]';


endfunction

