//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL  license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'http://www.cecill.info'.

function [output] = CL_op_ssoJ2(type_output,par1,par2,er,mu,j2,om_sun)
// Semi major axis, eccentricity or inclination for a sun synchronous orbit (J2)
//
// Calling Sequence
// [output] = CL_op_ssoJ2(type_output,par1,par2[,er[,mu[,j2[,om_sun]]]])
// [sma] = CL_op_ssoJ2('a',ecc,inc[,er[,mu[,j2[,om_sun]]]])
// [ecc] = CL_op_ssoJ2('e',sma,inc[,er[,mu[,j2[,om_sun]]]])
// [inc] = CL_op_ssoJ2('i',sma,ecc[,er[,mu[,j2[,om_sun]]]])
//
// Description
// <itemizedlist><listitem>
// This function computes orbital parameters (semi major axis, eccentricity or inclination) ensuring a sun synchronous orbit.
// This means that the orbit's ascending node precession rate is equal to the sun's inertial pulsation <emphasis role="bold">om_sun</emphasis> .(See equation (1))
// <para> - If <emphasis role="bold">type_output</emphasis>='a', <emphasis role="bold">par1</emphasis> is eccentricity, <emphasis role="bold">par2</emphasis> is inclination and semi major axis is computed (<emphasis role="bold">output</emphasis>).</para>
// <para> - If <emphasis role="bold">type_output</emphasis>='e', <emphasis role="bold">par1</emphasis> is semi major axis, <emphasis role="bold">par2</emphasis> is inclination and eccentricity is computed (<emphasis role="bold">output</emphasis>).</para>
// <para> - If <emphasis role="bold">type_output</emphasis>='i', <emphasis role="bold">par1</emphasis> is semi major axis, <emphasis role="bold">par2</emphasis> is eccentricity and inclination is computed (<emphasis role="bold">output</emphasis>)</para>
// <para>Note : If there is no solution, %nan is returned.</para>
// </listitem>
// <listitem><para/><inlinemediaobject><imageobject><imagedata fileref="helio.gif"/></imageobject></inlinemediaobject></listitem>
// <listitem>Last update : 8/4/2008</listitem>
// </itemizedlist>
//
// Parameters
// type_output: string defining parameter to compute (output). It can be 'a' for semi major axis, 'e' for eccentricity or 'i' for inclination
// par1: semi major axis if type_output is 'e' or 'i' ; eccentricity if type_output is 'a' [m] (1xN)
// par2: inclination if type_output is 'a' or 'e' ; eccentricity if type_output is 'i' [rad] (1xN)
// er: (optional) equatorial radius [m] (default is %CL_eqRad)
// mu: (optional) geocentric gravitational constant [m^3/s^2] (default value is %CL_mu)
// j2: (optional) second zonal harmonic (default is %CL_j2)
// om_sun : (optional) mean apparent revolution speed of the Sun around the Earth (default is %CL_omSun) (1 x 1)
// output: semi major axis if type_output is 'a' ; eccentricity if type_output is 'e' ; inclination if type_output is 'i' [m rad] (1xN)
//
// Authors
// CNES - DCT/SB
//
// Examples
// // Compute inclination for sunsynchronism
// sma = 7078.e3;
// ecc = 0.01;
// inc = CL_op_ssoJ2 ('i', sma, ecc)
//
// // Compute eccentricity for sunsynchronism
// sma = 7078.e3;
// inc = CL_deg2rad(98.15);
// ecc = CL_op_ssoJ2 ('e', sma, inc)
//
// // Compute semi major axis for sunsynchronism
// ecc = 0.01;
// inc = CL_deg2rad(97);
// sma = CL_op_ssoJ2 ('a', ecc, inc)

// Declarations:
global %CL_eqRad %CL_j2 %CL_mu %CL_omSun;

// Code:

Npar1 = size(par1,2)
Npar2 = size(par2,2)
N = max(Npar1,Npar2)
if ~( (Npar1==1|Npar1==N) & (Npar2==1|Npar2==N) ) then error('bad size of input arguments'); end

if ~exists('om_sun','local') then om_sun=%CL_omSun; end
if ~exists('er','local') then er=%CL_eqRad; end
if ~exists('mu','local') then mu=%CL_mu; end
if ~exists('j2','local') then j2=%CL_j2; end

ecc_null = -1e-8  //test parameter of eccentricity with respect to zero
am_zero = 1e-15 //inclination test parameter
aux = (-3.0 .* j2 .* er .* er .* sqrt(mu)) ./ (2.0 .* om_sun) //inititalisation of auxiliar variable

if type_output=='a'
  ecc = par1; inc=par2;
  //calculation of semi major axis
  cosi = cos(inc)
  ue2 = (1-ecc.^2) .* (1-ecc.^2)
  a_puis = (aux .* cosi) ./ ue2
  inds= find(a_puis<0);
  inds2 = find(a_puis>=0);
  if inds~=[]
    sma(inds) = %nan
    sma(inds2) = a_puis(inds2).^(1/3.5)
  else
    sma = a_puis.^(1/3.5)
  end
  output = sma

elseif type_output=='e'

  sma=par1; inc=par2;
  szs = size(sma,2);
  //calculation of eccentricity
  cosi = cos(inc)
  a_puis = sma.*sma.*sma .* sqrt(sma)
  var1 = (aux .* cosi) ./ a_puis
  inds0 = find(var1>=0);
  inds1 = find(var1<0);
  ecc(1,1:szs) = 0;
  ecc(inds1)= %nan;
  var2 = -1;
  var2(inds0) = 1-sqrt(var1(inds0));
  inds2 = find(var2<0);
  inds3 = find(var2<ecc_null);

  ecc(inds2)= %nan;
  ecc(inds3)= %nan;

  for i = 1:1:szs
   if ( ecc(i) == 0 ) then
    ecc(i) = sqrt(var2(i));
   end
  end
  output = ecc

elseif type_output=='i'

  sma=par1; ecc=par2;
  //inclination calculation
  e2 = ecc.*ecc
  ue2 = (1-e2).*(1-e2)
  a_puis = sma.^3 .* sqrt(sma)
  cosi = (a_puis .* ue2) ./ aux
  inds = find((abs(cosi)-1)>am_zero)
  inds2 = find((abs(cosi)-1)<=am_zero)
  if inds~=[]
    inc(inds) = %nan;
    inc(inds2) = acos(cosi(inds2));
  else
    inc = acos(cosi)
  end
  output = inc

else

  error('unknown value for type_output');

end

endfunction
