//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL  license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'http://www.cecill.info'.

function [N]=CL_cw_Nmatrix(alt,delta_t,er,mu)
// Clohessy-Wiltshire N Matrix
//
// Calling Sequence
// [N]=CL_cw_Nmatrix(alt,delta_t[,er,mu])
//
// Description
// <itemizedlist><listitem>
// This function computes transfer matrix <emphasis role="bold">N</emphasis> with Clohessy-Wiltshire equations.
// <para>See CL_fr_lvlhMat for the definition of the reference frame tied to the target</para>
// </listitem>
// <listitem><para/><inlinemediaobject><imageobject><imagedata fileref="cw.gif"/></imageobject></inlinemediaobject></listitem>
// <listitem>Last update : 26/07/2007</listitem>
// </itemizedlist>
//
// Parameters
// alt:  target's altitude [N]
// delta_t: time step [s]
// er: (optional) equatorial radius [m] (default is %CL_eqRad)
// mu: (optional) geocentric gravitational constant [m^3/s^2] (default value is %CL_mu)
// N: Clohessy-Wiltshire N transfer Matrix
//
// Authors
// CNES - DCT/SB
//
// See also
// CL_cw_Mmatrix
// CL_cw_twoImpulse
//
// Bibliography
// Mecanique spatiale, CNES - Cepadues 1995, Tome II, chapter 16 (Le rendez-vous orbital)
//
// Examples
// [N]=CL_cw_Nmatrix(350000,100)
//

// Declarations:
global %CL_eqRad %CL_mu;

// Code:

if ~exists('mu','local') then mu=%CL_mu; end
if ~exists('er','local') then er=%CL_eqRad; end

S = size(alt,2)

N = hypermat([6 3 S])

omega = CL_kp_params('mm',alt+er,mu)
teta = omega.*delta_t
c = cos(teta)
s = sin(teta)

N(1,1,:) = (1.0./omega.^2).*( 4*(1-c) - (3/2).*teta.^2 )
N(3,1,:) = (2.0./omega.^2).*(s-teta)
N(4,1,:) = (4*s-3*teta)./omega
N(6,1,:) = (2.0./omega).*(c-1)
N(2,2,:) = (1-c)./omega.^2
N(5,2,:) = s./omega
N(1,3,:) = (2.0./omega.^2).*(teta-s)
N(3,3,:) = (1-c)./omega.^2
N(4,3,:) = (2.0./omega).*(1-c)
N(6,3,:) = s./omega

if S==1 then N=N(:,:,1); end

endfunction
