//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL  license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'http://www.cecill.info'.

function [rel_dates,rel_pos_vel]=CL_cw_impulPropa(rel_date_ini,rel_pos_vel_ini,man_dates,man,alt,rel_date_end,delta_t,ballistic_coef_chaser,ballistic_coef_target,er,mu)
// Propagation of a chaser relatively to a target, with impulsives maneuvers
//
// Calling Sequence
// [rel_dates,rel_pos_vel]=CL_cw_impulPropa(rel_date_ini,rel_pos_vel_ini,man_dates,man,alt,rel_date_end,[delta_t,ballistic_coef_chaser,ballistic_coef_target,er,mu])
//
// Description
// <itemizedlist><listitem>
// This function computes (with maneuvers) positions and velocities of a chaser relatively to a target.
// <emphasis role="bold">rel_dates</emphasis> contains time in seconds, <emphasis role="bold">rel_pos_vel</emphasis> contains
// positions and velocities of the chaser relative to target's LVLH reference frame (described in CL_fr_lvlhMat).
// If ballistic coefficients are not given, the function does not take into account acceleration due to drag.
// </listitem>
// <listitem><para/><inlinemediaobject><imageobject><imagedata fileref="impulPropa_X.gif"/></imageobject></inlinemediaobject></listitem>
// <listitem><para/><inlinemediaobject><imageobject><imagedata fileref="impulPropa_Z.gif"/></imageobject></inlinemediaobject></listitem>
// <listitem>Last update : 29/10/2009</listitem>
// </itemizedlist>
//
// Parameters
// rel_date_ini: initial date in seconds [seconds] (1x1)
// rel_pos_vel_ini: initial position and velocity of chaser in target's LVLH frame [rx;ry;rz;vx;vy;vz] [m;m/s] (6x1)
// man_dates: dates of maneuvers in seconds [seconds] (1xM)
// man: maneuvers [dvx, dvy, dvz] [m/s] (3xM)
// alt: altitude of target [m]
// rel_date_end: relative end date in seconds [seconds] (1x1)
// delta_t: (optional) time step [s] (default is 100)
// ballistic_coef_chaser: (optional) Ballistic coefficient of chaser : S*cx/m S=equivalent drag surface, cx = drag coefficient, m = mass [m^2/kg] (default is 0)
// ballistic_coef_target: (optional) Ballistic coefficient of target : S*cx/m S=equivalent drag surface, cx = drag coefficient, m = mass [m^2/kg] (default is 0)
// er: (optional) equatorial radius [m] (default is %CL_eqRad)
// mu: (optional) geocentric gravitational constant [m^3/s^2] (default value is %CL_mu)
// rel_dates: dates in seconds [seconds] (1xN)
// rel_pos_vel: positions and velocities of chaser in target's LVLH frame at corresponding dates [rx;ry;rz;vx;vy;vz] (6xN)
//
// Authors
// CNES - DCT/SB
//
// See also
// CL_cw_ballisticPropa
// CL_cw_contPropa
//
// Bibliography
// Mecanique spatiale, CNES - Cepadues 1995, Tome II
//
// Examples
// rel_date_ini = 0;
// rel_pos_vel_ini = [0;0;0;0;0;0];
// alt = 450.e3;
// ballistic_coef_chaser = 100 * 1 / 20.e3;
// ballistic_coef_target = 200 * 1 / 400.e3;
// rel_date_end = 4*3600; // 4 hours of ballistic propagation
// delta_t = 100; // 100 seconds time step
// man_dates = 100;
// man = [1;0;0];
// [rel_dates,rel_pos_vel] = CL_cw_impulPropa(rel_date_ini,rel_pos_vel_ini,man_dates,man,alt,rel_date_end,delta_t,ballistic_coef_chaser,ballistic_coef_target)
//
//  //see 'CelestLab > Demos > RELATIVE MOTION > CW_propagation' for more examples
//

// Declarations:
global %CL_eqRad %CL_mu;

// Code:

if ~exists('ballistic_coef_chaser','local') then ballistic_coef_chaser=0; end
if ~exists('ballistic_coef_target','local') then ballistic_coef_target=0; end
if ~exists('delta_t','local') then delta_t=100; end
if ~exists('mu','local') then mu=%CL_mu; end
if ~exists('er','local') then er=%CL_eqRad; end

// This functions deals with ballistic arcs between impulse maneuvers

// First point :
rel_dates = rel_date_ini;
rel_pos_vel = rel_pos_vel_ini;

// Number of maneuvers :
nbman = size(man,2);

if (nbman>0)
  for i=1:nbman
    [dates,pos_vel] = CL_cw_ballisticPropa(rel_date_ini,rel_pos_vel_ini,alt,man_dates(i),delta_t,ballistic_coef_chaser,ballistic_coef_target,er,mu); //ballistic arc until maneuver
    rel_dates = [ rel_dates(1:$-1) , dates ];
  rel_pos_vel = [ rel_pos_vel(:,1:$-1) , pos_vel ];
  rel_pos_vel(4:6,$) = rel_pos_vel(4:6,$) + man(:,i); //add maneuver at last ephemeride
    rel_date_ini = rel_dates($);             //set last ephemeride position and velocity for next propagation
  rel_pos_vel_ini = rel_pos_vel(:,$);
  end
end

// Ballistic propa until end :
[dates,pos_vel] = CL_cw_ballisticPropa(rel_date_ini,rel_pos_vel_ini,alt,rel_date_end,delta_t,ballistic_coef_chaser,ballistic_coef_target,er,mu);
rel_dates = [ rel_dates(1:$-1) , dates ];
rel_pos_vel = [ rel_pos_vel(:,1:$-1) , pos_vel ];

endfunction
