//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL  license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'http://www.cecill.info'.

function [Zm]=CL_cw_meanChaser(Z0,dX0,alt,er,mu)
// Mean chaser
//
// Calling Sequence
// [Zm] = CL_cw_meanChaser(Z0,dX0,alt[,er,mu])
//
// Description
// <itemizedlist><listitem>
// When chaser is in circular orbit, coplanar to target orbit, and an impulsional manoeuver in X axis is performed,
// resultant relative motion is equal to a cycloid. This kind of motion can be formulated with two parameters:
// "mean chaser" and "excentrical chaser".
// This fonction computes the mean chaser. The mean chaser corresponds to the semi-major axis and gives the secular motion.
// Target frame defined in CL_cw_twoImpulse.
// </listitem>
// <listitem><para/><inlinemediaobject><imageobject><imagedata fileref="meanchaser.gif"/></imageobject></inlinemediaobject></listitem>
// <listitem>Last update : 29/10/2009</listitem>
// </itemizedlist>
//
// Parameters
// Z0: altitude of chaser at t=t0 in target frame[m] (1xN)
// dX0: first component of chaser velocity in target frame at t=t0 [m/s] (1xN)
// alt: target altitude (from planet surface) [m] (1xN)
// er: (optional) equatorial radius [m] (default is %CL_eqRad)
// mu: (optional) geocentric gravitational constant [m^3/s^2] (default value is %CL_mu)
// Zm: mean chaser [m] (1xN)
//
// Authors
// CNES - DCT/SB
//
// See also
// CL_kp_params
// CL_cw_twoImpulse
//
// Bibliography
// Mecanique spatiale, CNES - Cepadues 1995, Tome II, 16.3.2.4.2
//
// Examples
// Z0 = [30 20 15];
// dX0 = [7 8 9];
// alt = [260000 180000 300000];
// zm = CL_cw_meanChaser(Z0,dX0,alt);
// zm = CL_cw_meanChaser(Z0,dX0,alt(1));
// zm = CL_cw_meanChaser(Z0,dX0(1),alt(3));
//

// Declarations:
global %CL_eqRad %CL_mu;

// Code:

if ~exists('mu','local') then mu=%CL_mu; end
if ~exists('er','local') then er=%CL_eqRad; end

omega = CL_kp_params('mm',er+alt,mu)
Zm = 4.0.*Z0 - 2.0.*dX0./omega

endfunction
