//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL  license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'http://www.cecill.info'.

function cov=CL_cor2cov(cor,sd)
// Correlation to covariance matrix
//
// Calling Sequence
// cov = CL_cor2cov(cor,sd)
//
// Description
// <itemizedlist><listitem>
// Computes the the covariance matrix <emphasis role="bold">cov</emphasis> from the correlation matrix <emphasis role="bold">cor</emphasis>
// and the standard deviations vector <emphasis role="bold">sd</emphasis>
// <para>Diagonal of covariance matrix contains variances : cov(i,i) = var(i) = sd(i)^2</para>
// <para>Covariance and correlation matrix are linked by : cor(i,j) = cov(i,j) / (sd(i)*sd(j)) thus diagonal of correlation is cor(i,i) = 1</para>
// <para>Correlation matrix is dimensionless</para>
// </listitem>
// <listitem>Last update : 14/10/2009</listitem>
// </itemizedlist>
//
// Parameters
// cor: correlation (hyper)matrix (p x p x N)
// sd: standard deviations vectors (p x N)
// cov: covariance (hyper)matrix (p x p x N)
//
// Authors
// CNES - DCT/SB
//
// See also
// CL_cov2cor
//
// Examples
// // Random covariance matrix :
// p=6; N=2;
// cov = matrix(rand(1,p*p*N,"gaussian"),p,p,N);
// cov = cov'*cov;  // symmetric
// [cor,sd] = CL_cov2cor(cov);
//
// // Going back to the correlation :
// cov2 = CL_cor2cov(cor,sd) ;

// Declarations:


// Code:

[p p2] = size(cor);
[p3 p4] = size(sd);
if(p ~= p2 | p ~= p3)
  error('first and second dimension must be of same size');
end
// Matrix :
if(length(size(cor))==2)
  w = diag(sd);
  cov = w*cor*w;
// Hypermatrix :
else
    [p p2 N] = size(cor);
  cov = zeros(cor);
  for I = 1 : p
    for J = 1 : p
    if I == J
      cov(I,J,:) = sd(I,:).^2;
    else
      cov(I,J,:) = cor(I,J,:) .* matrix(sqrt(sd(I,:).^2 .* sd(J,:).^2 ),1,1,N);
    end
    end
  end
end

endfunction
