//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL  license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'http://www.cecill.info'.

function [cor,sd]=CL_cov2cor(cov)
// Covariance to correlation matrix
//
// Calling Sequence
// [cor,sd] = CL_cov2cor(cov)
//
// Description
// <itemizedlist><listitem>
// Computes the correlation matrix <emphasis role="bold">cor</emphasis> and the standard deviations vector <emphasis role="bold">sd</emphasis>
// from the covariance matrix <emphasis role="bold">cov</emphasis>.
// <para>Diagonal of covariance matrix contains variances : cov(i,i) = var(i) = sd(i)^2</para>
// <para>Covariance and correlation matrix are linked by : cor(i,j) = cov(i,j) / (sd(i)*sd(j)) thus diagonal of correlation is cor(i,i) = 1</para>
// <para>Correlation matrix is dimensionless</para>
// </listitem>
// <listitem>Last update : 14/10/2009</listitem>
// </itemizedlist>
//
// Parameters
// cov: covariance (hyper)matrix (p x p x N)
// cor: correlation (hyper)matrix (p x p x N)
// sd: standard deviations vectors (p x N)
//
// Authors
// CNES - DCT/SB
//
// See also
// CL_cor2cov
//
// Examples
// // Random covariance matrix :
// p=6; N=2;
// cov = matrix(rand(1,p*p*N,"gaussian"),p,p,N);
// cov = cov'*cov;  // symmetric
// [cor,sd] = CL_cov2cor(cov);
//
// // Going back to the correlation :
// cov2 = CL_cor2cov(cor,sd) ;



// Declarations:


// Code:

[p p2]=size(cov);
if(p ~= p2)
  error('first and second dimension must be of same size');
end

// Matrix :
if(length(size(cov))==2)
  w = inv(sqrt(diag(diag(cov))));
  cor = w*cov*w;
  sd = sqrt(diag(cov));
// Hypermatrix :
else
    [p p2 N] = size(cov);
  cor = zeros(cov);
  sd = zeros(p,N);
  for I = 1 : p
    for J = 1 : p
    if I == J
      cor(I,J,:) = matrix(ones(N,1),1,1,N);
    else
      cor(I,J,:) = cov(I,J,:) ./ ( sqrt( cov(I,I,:) .* cov(J,J,:)) );
    end
    end
    sd(I,:) = sqrt(cov(I,I,:));
  end
end


endfunction
