//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL  license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'http://www.cecill.info'.

function CL_plot_earthMap(winId,colorId,tickInterval,dataBounds)
// Plots an earth map (continents, lakes and islands)
//
// Calling Sequence
// CL_plot_earthMap([[,winId][,colorId][,tickInterval][,dataBounds])
//
// Parameters
// winId: (optionnal) figure Id - Id of the current window if omitted.
// colorId: (optionnal) color index - Default is 1.
// tickInterval : (optionnal) tick interval of the grid (longitude and latitude) in degrees - [60 ,30] if missing.
// dataBounds : (optionnal) zoombox (longmin, latmin; longmax, latmax]. [-180,-90;180,90] if missing(entire view).
//
// Description
// <itemizedlist><listitem>
// This function plots an Earth map in the selected figure (winId).
// <para>color can be used to chose the color of the map.</para>
// <para>tickInterval can be used to modify the ticks of the grid.</para>
// <para>dataBounds can be used to zoom in.</para>
// <para>Note: The data are loaded from files located 'data/util' directory.
// These files contain 2 columns longitudes and latitudes. Blocks are separated by "999 999".</para>
// </listitem></itemizedlist>
//
// Authors
// CNES - DCT/SB
//
// See also
// CL_plot_ephem
//
// Examples
// // basic plot
// CL_plot_earthMap();
//
// // plot inside figure 10, with color 2
// CL_plot_earthMap(winId=10,colorId=2);
//
// // plot inside figure 10, with color 2, and zoom on Europe
// CL_plot_earthMap(winId=10,colorId=2,dataBounds=[-10,30;40,60);

// Declarations:


// Code:

 if exists('winId','local') then
  f = scf(winId)
 else
      f = gcf();
 end

 if ~exists('colorId','local') then
  colorId = 1;
 end

 if ~exists('tickInterval','local') then
  tickInterval = [60 , 30];
 end

 if ~exists('dataBounds','local') then
  dataBounds = [-180,-90;180,90];
 end


 // Trace :
 a=gca();
 oldpixmap=f.pixmap;
 f.pixmap = "on";


 // Extrait le chemin vers les cartes :
 nomfonc = 'CL_plot_earthMap';
 sep = system_getproperty('file.separator');
 path_function=get_function_path(nomfonc);
 dir = dirname(dirname(dirname(path_function)))+sep+'data'+sep+'utils';

 // Lecture des cartes :

 nomfic = dir + sep + 'terre.pla';
 contours=fscanfMat(nomfic);

 // Indice des sauts a 180deg
 i_cont=find(contours(:,1) == 999.);

 for k=1:size(i_cont,'c')-1
  plot2d(contours(i_cont(k)+1:i_cont(k+1)-1,1),contours(i_cont(k)+1:i_cont(k+1)-1,2),style=colorId);
 end

 a=gca();
 a.data_bounds = dataBounds;
 a.tight_limits="on";
 a.x_ticks=tlist(["ticks", "locations", "labels"], -180:tickInterval(1):180, string(-180:tickInterval(1):180));
 a.y_ticks=tlist(["ticks", "locations", "labels"], -90:tickInterval(2):90, string(-90:tickInterval(2):90));
 a.sub_ticks=[0,0];
 a.grid=[1,1];
 a.font_size=3;
 a.box="on";
 show_pixmap();
 f.pixmap = oldpixmap;

endfunction


