//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL  license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'http://www.cecill.info'.

function [] = CL_plot_ephem(pos_car_ter,winId,colorId)
// Plots ground tracks of the given position ephemeris
//
// Calling Sequence
// CL_plot_ephem(pos_car_ter [,winId][,colorId][,tickInterval][,dataBounds])
//
// Parameters
// pos_car_ter: positions in cartesian coordinates, in the terrestrial frame [X;Y;Z] (3xN)
// winId: (optionnal) figure Id - Id of the current window if omitted.
// colorId : (optionnal) color index - Default is 1.
// tickInterval : (optionnal) tick interval of the grid (longitude and latitude) in degrees - [60 ,30] if missing.
// dataBounds : (optionnal) zoombox (longmin, latmin; longmax, latmax]. [-180,-90;180,90] if missing(entire view).
//
// Description
// <itemizedlist><listitem>
// This function plots the ground trace corresponding to the trajectory as given.
// <para>The latitude plotted is the geodesic latitude.</para></listitem>
// </itemizedlist>
//
// Authors
// CNES - DCT/SB
//
// See also
// CL_plot_earthMap
//
// Examples
// // Plot groundtracks (different color for each orbit)
// t1 = 21915;
// pas = 10 / 86400;
// kep_t1=[7070 * 1000; 0.001; CL_deg2rad(98); CL_deg2rad(90); CL_deg2rad(100); CL_deg2rad(270)];
// nodper = CL_op_paramsJ2('nodper', kep_t1(1) , kep_t1(2) , kep_t1(3));
// scf();
//
// for n_orb = 1 : 3
//  t2 = t1 + (n_orb-1)*nodper/86400 : pas : t1 + n_orb*nodper/86400;
//  kep_t2 = CL_ex_secularJ2(t1, kep_t1, t2);
//  param_car = CL_oe_kep2car(kep_t2);
//  pos_car_ter =  CL_fr_G502ter(t2,param_car) ;
//  CL_plot_ephem(pos_car_ter, colorId=n_orb);
// end
//
// CL_plot_earthMap(colorId=2);
//
// Declarations:
global %CL_rad2deg;

// Code:

 if exists('winId','local') then
  f = scf(winId)
 else
      f = gcf();
 end

 if ~exists('colorId','local') then
    colorId=1;
 end

 if ~exists('tickInterval','local') then
    tickInterval = [60 , 30];
 end

 if ~exists('dataBounds','local') then
  dataBounds = [-180,-90;180,90];
 end

 a=gca();
 oldpixmap=f.pixmap;
 f.pixmap = "on";

 // Passage de pos en coordonnees spheriques
 ephem_geo_sph = CL_co_car2ell(pos_car_ter);
 // Recaler entre -%pi et %pi
 ephem_geo_sph(1:2,:) = CL_rMod(ephem_geo_sph(1:2,:) , -%pi, %pi);

 // Recherches des indices ou on a un saut de 180 deg
 delta_long=zeros(1,size(ephem_geo_sph,'c')+1);
 delta_long(1)=999;
 delta_long(2:$-1) = abs(ephem_geo_sph(1,2:$)-ephem_geo_sph(1,1:$-1));
 delta_long(size(ephem_geo_sph,'c')+1)=999;
 indices=find(delta_long>%pi);

 // Trace :
 k=1;
 plot2d(ephem_geo_sph(1,indices(k):indices(k+1)-1)*%CL_rad2deg, ephem_geo_sph(2,indices(k):indices(k+1)-1)*%CL_rad2deg, style=colorId);
 for k=2:size(indices,'c')-2
   plot2d(ephem_geo_sph(1,indices(k):indices(k+1)-1)*%CL_rad2deg, ephem_geo_sph(2,indices(k):indices(k+1)-1)*%CL_rad2deg, style=colorId);
   // Calcul des points intermediaires, pour le raccordement a 180deg ou a -180deg.
   pointA = ephem_geo_sph(1:2,indices(k+1)-1);
   pointB = ephem_geo_sph(1:2,indices(k+1));
   if(abs(pointA(1)-%pi) > abs(pointA(1)+%pi))  // si on est proche de -180deg:
   poidB = (pointA(2) +%pi)/(2*%pi+pointA(2)-pointB(2)) ;
   poidA = (%pi-pointB(2))/(2*%pi+pointA(2)-pointB(2)) ;
   pointIntermediaire(1)=-%pi;
   pointIntermediaire(2)=pointA(2)*poidA+pointB(2)*poidB;
   else                      // si on est proche de 180deg:
   poidA = (pointB(2) +%pi)/(2*%pi+pointB(2)-pointA(2)) ;
   poidB = (%pi-pointA(2))/(2*%pi+pointB(2)-pointA(2)) ;
   pointIntermediaire(1)=%pi;
   pointIntermediaire(2)=pointA(2)*poidA+pointB(2)*poidB;
   end
   plot2d([ephem_geo_sph(1,indices(k+1)-1),pointIntermediaire(1)]*%CL_rad2deg,[ephem_geo_sph(2,indices(k+1)-1),pointIntermediaire(2)]*%CL_rad2deg, style=colorId);
   plot2d([-pointIntermediaire(1),ephem_geo_sph(1,indices(k+1))]*%CL_rad2deg,[pointIntermediaire(2),ephem_geo_sph(2,indices(k+1))]*%CL_rad2deg, style=colorId);
 end
 k=size(indices,'c')-1;
 plot2d(ephem_geo_sph(1,indices(k):indices(k+1)-1)*%CL_rad2deg, ephem_geo_sph(2,indices(k):indices(k+1)-1)*%CL_rad2deg, style=colorId);

 a=gca();
 a.data_bounds = dataBounds;
 a.tight_limits="on";
 a.x_ticks=tlist(["ticks", "locations", "labels"], -180:tickInterval(1):180, string(-180:tickInterval(1):180));
 a.y_ticks=tlist(["ticks", "locations", "labels"], -90:tickInterval(2):90, string(-90:tickInterval(2):90));
 a.sub_ticks=[0,0];
 a.grid=[1,1];
 a.font_size=3;
 a.box="on";
 show_pixmap();
 f.pixmap = oldpixmap;

endfunction
